@@ -14,12 +14,16 @@
  */
 
 #include	"render.h"
+#include	"xbuf.h"
+#include	"utils.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
 #endif
 
-static int			N_pages, Page;		/* w.r.t. unrotated coords */
+int			Obj;
+static int			N_pages = 1;	/* w.r.t. unrotated coords */
+static int			Page;			/* w.r.t. unrotated coords */
 static int			Layer,Nlayers;
 static char			**LayerID;
 static point		First,Major,Minor;
@@ -36,7 +40,8 @@ void emit_reset(Agraph_t* g)
 {
 	Agnode_t	*n;
 
-	N_pages = Page = 0;
+	N_pages = 1;
+	Page = 0;
 	Layer = Nlayers = 0;
 	LayerID = (char **) 0;
 	First.x = First.y = 0;
@@ -51,7 +56,7 @@ void emit_reset(Agraph_t* g)
 	Deffontname = (char *) 0;
 
 	for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
-		n->u.state = 0;
+		ND_state(n) = 0;
 	}
 
 	if(CodeGen) CodeGen->reset();
@@ -73,8 +78,8 @@ void emit_graph(graph_t* g, int flags)
 			CodeGen->set_fillcolor(str);
 		if (((str = agget(g,"fontcolor")) != 0) && str[0])
 			CodeGen->set_pencolor(str);
-		for (c = 1; c <= g->u.n_cluster; c++) { 
-	                subg = g->u.clust[c];
+		for (c = 1; c <= GD_n_cluster(g); c++) { 
+	                subg = GD_clust(g)[c];
 			if (((str = agget(subg,"color")) != 0) && str[0])
 				CodeGen->set_pencolor(str);
 			if (((str = agget(subg,"fillcolor")) != 0) && str[0])
@@ -101,16 +106,23 @@ void emit_graph(graph_t* g, int flags)
 	do {
 		if (Nlayers > 0) emit_layer(Layer);
 		for (curpage = First; validpage(curpage); curpage = pageincr(curpage)) {
+			Obj = NONE;
 			setup_page(g,curpage);
-			if (g->u.label) emit_label(g->u.label,g);
-			emit_clusters(g);
+			if (GD_label(g)) emit_label(GD_label(g),g);
+			Obj = CLST;
+			/* when drawing, lay clusters down before nodes and edges */
+			if (!(flags & EMIT_CLUSTERS_LAST)) {
+				emit_clusters(g,flags);
+			}
 			if (flags & EMIT_SORTED) {
 				/* output all nodes, then all edges */
+				Obj = NODE;
 				CodeGen->begin_nodes();
 				for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 					emit_node(n);
 				}
 				CodeGen->end_nodes();
+				Obj = EDGE;
 				CodeGen->begin_edges();
 				for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 					for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
@@ -122,13 +134,21 @@ void emit_graph(graph_t* g, int flags)
 			else {
 				/* output in breadth first graph walk order */
 				for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+					Obj = NODE;
 					emit_node(n);
 					for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
+						Obj = NODE;
 						emit_node(e->head);
+						Obj = EDGE;
 						emit_edge(e);
 					}
 				}    
 			}
+			/* when mapping, detect events on clusters after nodes and edges */
+			if (flags & EMIT_CLUSTERS_LAST) {
+				emit_clusters(g,flags);
+			}
+			Obj = NONE;
 			CodeGen->end_page();
 		}
 		Layer++;
@@ -141,16 +161,20 @@ void emit_eof(void)
 	if (Page > 0) CodeGen->end_job();
 }
 
-void emit_clusters(graph_t* g)
+void emit_clusters(graph_t* g, int flags)
 {
 	int			i,c,filled;
 	graph_t		*subg;
 	point		A[4];
 	char		*str,**style;
 
-	for (c = 1; c <= g->u.n_cluster; c++) {
-		subg = g->u.clust[c];
+	for (c = 1; c <= GD_n_cluster(g); c++) {
+		subg = GD_clust(g)[c];
 		if (clust_in_layer(subg) == FALSE) continue;
+		/* when mapping, detect events on clusters after sub_clusters */
+		if (flags & EMIT_CLUSTERS_LAST) {
+			emit_clusters(subg,flags);
+		}
 		CodeGen->begin_cluster(subg);
 		CodeGen->begin_context();
 		filled = FALSE;
@@ -159,120 +183,121 @@ void emit_clusters(graph_t* g)
 			for (i = 0; style[i]; i++) 
 				if (strcmp(style[i],"filled")==0) {filled = TRUE; break;}
 		}
-		emit_background(subg, subg->u.bb.LL, subg->u.bb.UR);
-		if (((str = agget(subg,"color")) != 0) && str[0])
+		if (((str = agget(subg,"pencolor")) != 0) && str[0])
+			CodeGen->set_pencolor(str);
+		else if (((str = agget(subg,"color")) != 0) && str[0])
+			CodeGen->set_pencolor(str);
+		/* bgcolor is supported for backward compatability */
+		else if (((str = agget(subg,"bgcolor")) != 0) && str[0])
 			CodeGen->set_pencolor(str);
 		if (((str = agget(subg,"fillcolor")) != 0) && str[0])
 			CodeGen->set_fillcolor(str);
 		else if (((str = agget(subg,"color")) != 0) && str[0])
 			CodeGen->set_fillcolor(str);
-		A[0] = subg->u.bb.LL;
-		A[2] = subg->u.bb.UR;
+		/* bgcolor is supported for backward compatability */
+		else if (((str = agget(subg,"bgcolor")) != 0) && str[0]) {
+			filled = TRUE;
+			CodeGen->set_fillcolor(str);
+		}
+		A[0] = GD_bb(subg).LL;
+		A[2] = GD_bb(subg).UR;
 		A[1].x = A[2].x; A[1].y = A[0].y;
 		A[3].x = A[0].x; A[3].y = A[2].y;
 		CodeGen->polygon(A,4,filled);
-		if (subg->u.label) emit_label(subg->u.label,subg);
+		if (GD_label(subg)) emit_label(GD_label(subg),subg);
 		CodeGen->end_context();
 		CodeGen->end_cluster();
-		emit_clusters(subg);
+		/* when drawing, lay down clusters before sub_clusters */
+		if (!(flags & EMIT_CLUSTERS_LAST)) {
+			emit_clusters(subg,flags);
+		}
 	}
 }
 
 void emit_node(node_t* n)
 {
-	if (n->u.shape == NULL) return;
-	if (node_in_layer(n->graph,n) && node_in_CB(n) && (n->u.state != Page)) {
-		n->u.shape->codefn(n);
-		n->u.state = Page;
+	if (ND_shape(n) == NULL) return;
+	if (node_in_layer(n->graph,n) && node_in_CB(n) && (ND_state(n) != Page)) {
+		ND_shape(n)->codefn(n);
+		ND_state(n) = Page;
 	}
 }
 
-void emit_arrowhead (point p, double theta, double scale, int flag)
-/* allowable are NORM|INV + DOT|ODOT */
-{
-  point a[3];
-  pointf u,v;
-  int r;
-
-  theta = RADIANS(theta);
-
-  /* FIXME: does this have enough precision for ps? */
-  if (flag & ARR_NORM) {
-    u.x = p.x + ARROW_LENGTH*scale*cos(theta);
-    u.y = p.y + ARROW_LENGTH*scale*sin(theta);
-    v.x = ARROW_WIDTH/2.*scale*cos(theta+PI/2);
-    v.y = ARROW_WIDTH/2.*scale*sin(theta+PI/2);
-    a[0].x = ROUND(u.x+v.x); a[0].y = ROUND(u.y+v.y);
-    a[1] = p;
-    a[2].x = ROUND(u.x-v.x); a[2].y = ROUND(u.y-v.y);
-    CodeGen->polygon(a,3,1);
-    p.x = ROUND(u.x); p.y = ROUND(u.y);
-  } else if (flag & ARR_INV) {
-    u.x = p.x + ARROW_INV_LENGTH*scale*cos(theta);
-    u.y = p.y + ARROW_INV_LENGTH*scale*sin(theta);
-    v.x = ARROW_INV_WIDTH/2.*scale*cos(theta+PI/2);
-    v.y = ARROW_INV_WIDTH/2.*scale*sin(theta+PI/2);
-    a[0].x = p.x + ROUND(v.x);  a[0].y = p.y + ROUND(v.y);
-    a[1].x = ROUND(u.x);        a[1].y = ROUND(u.y);
-    a[2].x = p.x + ROUND(-v.x); a[2].y = p.y + ROUND(-v.y);
-    CodeGen->polygon(a,3,1);
-    p.x = ROUND(u.x); p.y = ROUND(u.y);
-  }
-  if (flag & (ARR_DOT|ARR_ODOT)) {
-    r = ROUND (ARROW_DOT_RADIUS*scale);
-    CodeGen->ellipse(p,r,r, flag & ARR_DOT);
-  }
-}
-
 void emit_edge(edge_t* e)
 {
 	int		i;
 	char	*color,*style;
+	char	**styles = 0;
+	char	**sp;
 	bezier	bz;
 	boolean	saved = FALSE;
 	double	scale;
+	double theta;
+	char	*p;
 
 	if ((edge_in_CB(e) == FALSE) || (edge_in_layer(e->head->graph,e) == FALSE))
 		return;
 
 	CodeGen->begin_edge(e);
 	style = late_string(e,E_style,"");
+		/* We shortcircuit drawing an invisible edge because the arrowhead
+         * code resets the style to solid, and most of the code generators
+		 * (except PostScript) won't honor a previous style of invis.
+         */
+	if (style[0]) {
+      styles = parse_style(style);
+      sp = styles;
+	  while ((p = *sp++)) {
+		if (streq(p, "invis")) {
+			CodeGen->end_edge();
+			return;
+        }
+      }
+    }
 	color = late_string(e,E_color,"");
-	scale = late_float(e,E_arrowsz,1.0,0.0);
-	if (color[0] || style [0]) {
+	scale = late_double(e,E_arrowsz,1.0,0.0);
+	if (color[0] || styles) {
 		CodeGen->begin_context();
-		if (style[0]) CodeGen->set_style(parse_style(style));
+		if (styles) CodeGen->set_style(styles);
 		if (color[0]) {
 			CodeGen->set_pencolor(color);
 			CodeGen->set_fillcolor(color);
 		}
 		saved = TRUE;
 	}
-	if (e->u.spl) {
-		for (i = 0; i < e->u.spl->size; i++) {
-			bz = e->u.spl->list[i];
+	if (ED_spl(e)) {
+		for (i = 0; i < ED_spl(e)->size; i++) {
+			bz = ED_spl(e)->list[i];
 			if (NOT(codegen_bezier_has_arrows())) {
 				CodeGen->beziercurve(bz.list,bz.size,FALSE,FALSE);
                 /* vladimir: added sflag/eflag, arrowheads are made here */
-				if (bz.sflag) 
-                  emit_arrowhead
-                    (bz.sp, DEGREES(atan2pt(bz.list[0],bz.sp)),
-                     scale, bz.sflag);
-				if (bz.eflag) 
-                  emit_arrowhead 
-                    (bz.ep, DEGREES(atan2pt(bz.list[bz.size-1],bz.ep)),
-                     scale, bz.eflag);
-			}
-			else CodeGen->beziercurve(bz.list,bz.size,bz.sflag,bz.eflag);
+				if (bz.sflag) {
+					/* dds: open style has sp == list[0] */
+					if ((bz.sflag == ARR_OPEN) || (bz.sflag == ARR_HALFOPEN))
+						theta = atan2pt(bz.list[1],bz.list[0]);
+					else
+						theta = atan2pt(bz.list[0],bz.sp);
+					arrow_gen(bz.sp, DEGREES(theta), scale, bz.sflag);
+				}
+				if (bz.eflag) {
+					/* dds: open style has ep == list[size-1] */
+					if ((bz.eflag == ARR_OPEN) || (bz.eflag == ARR_HALFOPEN))
+						theta = atan2pt(bz.list[bz.size-2],bz.list[bz.size-1]);
+					else
+						theta = atan2pt(bz.list[bz.size-1],bz.ep);
+					arrow_gen(bz.ep, DEGREES(theta), scale, bz.eflag);
+				}
+			} else 
+				CodeGen->beziercurve(bz.list,bz.size,bz.sflag,bz.eflag);
 		}
 	}
-	if (e->u.label) {
-		emit_label(e->u.label,e->tail->graph);
-		if (mapbool(late_string(e,E_decorate,"false")))
-			emit_attachment(e->u.label,e->u.spl);
+	if (ED_label(e)) {
+		emit_label(ED_label(e),e->tail->graph);
+		if (mapbool(late_string(e,E_decorate,"false")) && ED_spl(e))
+			emit_attachment(ED_label(e),ED_spl(e));
 	}
-	if (e->u.head_label) emit_label(e->u.head_label,e->tail->graph); /* vladimir */
-	if (e->u.tail_label) emit_label(e->u.tail_label,e->tail->graph); /* vladimir */
+	if (ED_head_label(e)) emit_label(ED_head_label(e),e->tail->graph); /* vladimir */
+	if (ED_tail_label(e)) emit_label(ED_tail_label(e),e->tail->graph); /* vladimir */
 
 	if (saved) CodeGen->end_context();
 	CodeGen->end_edge();
@@ -283,10 +308,10 @@ int node_in_CB(node_t* n)
 	box	nb;
 
 	if (N_pages == 1) return TRUE;
-	nb.LL.x = n->u.coord.x - n->u.lw;
-	nb.LL.y = n->u.coord.y - n->u.ht/2;
-	nb.UR.x = n->u.coord.x + n->u.rw;
-	nb.UR.y = n->u.coord.y + n->u.ht/2;
+	nb.LL.x = ND_coord(n).x - ND_lw(n);
+	nb.LL.y = ND_coord(n).y - ND_ht(n)/2;
+	nb.UR.x = ND_coord(n).x + ND_rw(n);
+	nb.UR.y = ND_coord(n).y + ND_ht(n)/2;
 	return rect_overlap(CB,nb);
 }
 
@@ -298,7 +323,7 @@ int node_in_layer(graph_t* g, node_t* n)
 	if (Nlayers <= 0) return TRUE;
 	pn = late_string(n,N_layer,"");
 	if (selectedlayer(pn)) return TRUE;
-	if (pn[0]) return FALSE;
+	if (pn[0]) return FALSE;  /* Only check edges if pn = "" */
 	if ((e = agfstedge(g,n)) == NULL) return TRUE;
 	for (e = agfstedge(g,n); e; e = agnxtedge(g,e,n)) {
 		pe = late_string(e,E_layer,"");
@@ -346,9 +371,9 @@ int edge_in_CB(edge_t* e)
 	textlabel_t	*lp;
 
 	if (N_pages == 1) return TRUE;
-	if (e->u.spl == NULL) return FALSE;
-	for (i = 0; i < e->u.spl->size; i++) {
-		bz = e->u.spl->list[i];
+	if (ED_spl(e) == NULL) return FALSE;
+	for (i = 0; i < ED_spl(e)->size; i++) {
+		bz = ED_spl(e)->list[i];
 		np = bz.size;
 		p = bz.list;
 		pp = p[0];
@@ -357,7 +382,7 @@ int edge_in_CB(edge_t* e)
 			pp = p[j];
 		}
 	}
-	if ((lp = e->u.label) == NULL) return FALSE;
+	if ((lp = ED_label(e)) == NULL) return FALSE;
 	sz = cvt2pt(lp->dimen);
 	b.LL.x = lp->p.x - sz.x / 2; b.UR.x = lp->p.x + sz.x / 2;
 	b.LL.y = lp->p.y - sz.y / 2; b.UR.y = lp->p.y + sz.y / 2;
@@ -381,11 +406,11 @@ void emit_attachment(textlabel_t* lp, splines* spl)
 
 void emit_header(graph_t* g)
 {
-	char	*user,buf[SMALLBUF];
+	char	*user;
 
 	setup_graph(g);
 	if (Page == 0) {
-		user = username(buf);
+		user = username();
 		CodeGen->begin_job(Output_file,g,Lib,user,Info,Pages);
 	}
 	CodeGen->begin_graph(g,PB,PFC);
@@ -462,21 +487,36 @@ int layer_index(char* str, int all)
 int selectedlayer(char* spec)
 {
 	int		n0,n1;
-	char	buf[SMALLBUF], *w0, *w1;
+	unsigned char	buf[SMALLBUF];
+	char	*w0, *w1;
+    xbuf    xb;
+    int     rval = FALSE;
 
-	strcpy(buf,spec);
-	w1 = w0 = strtok(buf,Layerdelims);
+    xbinit (&xb, SMALLBUF, buf);
+	xbput(&xb,spec);
+	w1 = w0 = strtok(xbuse(&xb),Layerdelims);
 	if (w0) w1 = strtok(NULL,Layerdelims);
 	switch((w0 != NULL) + (w1 != NULL)) {
-	case 0: return FALSE;
-	case 1: n0 = layer_index(w0,Layer);
-		return (n0 == Layer);
-	case 2: n0 = layer_index(w0,0);  n1 = layer_index(w1,Nlayers);
-		if ((n0 < 0) || (n1 < 0)) return TRUE;
-		if (n0 > n1) {int t = n0; n0 = n1; n1 = t;}
-		return BETWEEN(n0,Layer,n1);
+	case 0: 
+      rval = FALSE;
+      break;
+	case 1: 
+      n0 = layer_index(w0,Layer);
+	  rval = (n0 == Layer);
+      break;
+	case 2: 
+      n0 = layer_index(w0,0);  
+      n1 = layer_index(w1,Nlayers);
+	  if ((n0 < 0) || (n1 < 0)) rval = TRUE;
+	  else if (n0 > n1) {
+        int t = n0; 
+        n0 = n1; 
+        n1 = t;}
+		rval = BETWEEN(n0,Layer,n1);
+      break;
 	}
-	return FALSE;
+    xbfree (&xb);
+	return rval;
 }
 
 point
@@ -513,7 +553,7 @@ void setup_graph(graph_t* g)
 	point		PFCLM;	/* page for centering less margins */
 	point		DS;		/* device drawable region for a page of the graph */
 
-	assert((g->u.bb.LL.x == 0) && (g->u.bb.LL.y == 0));
+	assert((GD_bb(g).LL.x == 0) && (GD_bb(g).LL.y == 0));
 
 	if (LayerID) free(LayerID);
 	if ((p = agget(g,"layers")) != 0) Nlayers = parse_layers(p);
@@ -522,57 +562,57 @@ void setup_graph(graph_t* g)
 	/* determine final drawing size and scale to apply. */
 	/* N.B. magnification could be allowed someday in the next conditional */
 	/* N.B. size given by user is not rotated by landscape mode */
-	if ((g->u.drawing->size.x > 0)	/* was given by user... */
-		&& ((g->u.drawing->size.x < g->u.bb.UR.x) /* drawing is too big... */
-			|| (g->u.drawing->size.y < g->u.bb.UR.y))) {
-		xscale = ((double)g->u.drawing->size.x) / g->u.bb.UR.x;
-		yscale = ((double)g->u.drawing->size.y) / g->u.bb.UR.y;
+	if ((GD_drawing(g)->size.x > 0)	/* was given by user... */
+		&& ((GD_drawing(g)->size.x < GD_bb(g).UR.x) /* drawing is too big... */
+			|| (GD_drawing(g)->size.y < GD_bb(g).UR.y))) {
+		xscale = ((double)GD_drawing(g)->size.x) / GD_bb(g).UR.x;
+		yscale = ((double)GD_drawing(g)->size.y) / GD_bb(g).UR.y;
 		scale = MIN(xscale,yscale);
-		g->u.drawing->scale = scale;
-		g->u.drawing->size.x = scale * g->u.bb.UR.x;
-		g->u.drawing->size.y = scale * g->u.bb.UR.y;
+		GD_drawing(g)->scale = scale;
+		GD_drawing(g)->size.x = scale * GD_bb(g).UR.x;
+		GD_drawing(g)->size.y = scale * GD_bb(g).UR.y;
 	}
 	else {	/* go with "natural" size of layout */
-		g->u.drawing->size = g->u.bb.UR;
-		scale = g->u.drawing->scale = 1.0;
+		GD_drawing(g)->size = GD_bb(g).UR;
+		scale = GD_drawing(g)->scale = 1.0;
 	}
 
 	/* determine pagination */
-	PB.LL = g->u.drawing->margin;
-	if ((g->u.drawing->page.x > 0) && (g->u.drawing->page.y > 0)) {
+	PB.LL = GD_drawing(g)->margin;
+	if ((GD_drawing(g)->page.x > 0) && (GD_drawing(g)->page.y > 0)) {
 			/* page was set by user */
 		point	tp;
-		PFC = g->u.drawing->page;
+		PFC = GD_drawing(g)->page;
 		PFCLM.x = PFC.x - 2*PB.LL.x; PFCLM.y = PFC.y - 2*PB.LL.y;
 		GP.x = PFCLM.x ; GP.y = PFCLM.y;	/* convert to double */
-		if (g->u.drawing->landscape) GP = exch_xyf(GP);
+		if (GD_drawing(g)->landscape) GP = exch_xyf(GP);
 		GP.x = GP.x / scale; GP.y = GP.y / scale;
 			/* we don't want graph page to exceed its bounding box */
-		GP.x = MIN(GP.x,g->u.bb.UR.x); GP.y = MIN(GP.y,g->u.bb.UR.y);
-		Pages.x = (GP.x > 0) ? ceil( ((double)g->u.bb.UR.x) / GP.x) : 1;
-		Pages.y = (GP.y > 0) ? ceil( ((double)g->u.bb.UR.y) / GP.y) : 1;
+		GP.x = MIN(GP.x,GD_bb(g).UR.x); GP.y = MIN(GP.y,GD_bb(g).UR.y);
+		Pages.x = (GP.x > 0) ? ceil( ((double)GD_bb(g).UR.x) / GP.x) : 1;
+		Pages.y = (GP.y > 0) ? ceil( ((double)GD_bb(g).UR.y) / GP.y) : 1;
 		N_pages = Pages.x * Pages.y;
 
 			/* find the drawable size in device coords */
-		tp = g->u.drawing->size;
-		if (g->u.drawing->landscape) tp = exch_xy(tp);
+		tp = GD_drawing(g)->size;
+		if (GD_drawing(g)->landscape) tp = exch_xy(tp);
 		DS.x = MIN(tp.x,PFCLM.x);
 		DS.y = MIN(tp.y,PFCLM.y);
 	}
 	else {
 			/* page not set by user, assume default when centering,
 				but allow infinite page for any other interpretation */
-		GP.x = g->u.bb.UR.x; GP.y = g->u.bb.UR.y;
+		GP.x = GD_bb(g).UR.x; GP.y = GD_bb(g).UR.y;
 		PFC.x = DEFAULT_PAGEWD; PFC.y = DEFAULT_PAGEHT;
 		PFCLM.x = PFC.x - 2*PB.LL.x; PFCLM.y = PFC.y - 2*PB.LL.y;
-		DS = g->u.drawing->size;
-		if (g->u.drawing->landscape) DS = exch_xy(DS);
+		DS = GD_drawing(g)->size;
+		if (GD_drawing(g)->landscape) DS = exch_xy(DS);
 		Pages.x = Pages.y = N_pages = 1;
 	}
 
 	set_pagedir(g);
 	/* determine page box including centering */
-	if (g->u.drawing->centered) {
+	if (GD_drawing(g)->centered) {
 		point	extra;
 		if ((extra.x = PFCLM.x - DS.x) < 0) extra.x = 0;
 		if ((extra.y = PFCLM.y - DS.y) < 0) extra.y = 0;
@@ -580,7 +620,7 @@ void setup_graph(graph_t* g)
 	}
 	PB.UR = add_points(PB.LL,DS);
 	Deffontname = late_nnstring(g->proto->n,N_fontname,DEFAULT_FONTNAME);
-	Deffontsize = late_float(g->proto->n,N_fontsize,DEFAULT_FONTSIZE,MIN_FONTSIZE);
+	Deffontsize = late_double(g->proto->n,N_fontsize,DEFAULT_FONTSIZE,MIN_FONTSIZE);
 }
 
 void emit_background(graph_t* g, point LL, point UR)
@@ -589,10 +629,10 @@ void emit_background(graph_t* g, point LL, point UR)
 	point	A[4];
 
 	if (((str = agget(g,"bgcolor")) != 0) && str[0]) {
-		A[0] = A[1] = LL;
-		A[2] = A[3] = UR;
-		A[1].y = A[2].y;
-		A[3].y = A[0].y;
+		A[0].x = A[1].x = LL.x - GD_drawing(g)->margin.x;
+		A[2].x = A[3].x = UR.x + GD_drawing(g)->margin.x;
+		A[1].y = A[2].y = UR.y + GD_drawing(g)->margin.y;
+		A[3].y = A[0].y = LL.y - GD_drawing(g)->margin.y;
 		CodeGen->set_fillcolor(str);
 		CodeGen->set_pencolor(str);
 		CodeGen->polygon(A,4,TRUE);	/* filled */
@@ -606,16 +646,16 @@ void setup_page(graph_t* g, point page)
 	int			theta;
 
 	Page++;
-	theta = (g->u.drawing->landscape ? 90 : 0);
+	theta = (GD_drawing(g)->landscape ? 90 : 0);
 
 	/* establish current box in graph coordinates */
 	CB.LL.x = page.x  * GP.x;	CB.LL.y = page.y * GP.y;
 	CB.UR.x = CB.LL.x + GP.x;	CB.UR.y = CB.LL.y + GP.y;
 
 	/* establish offset to be applied, in graph coordinates */
-	if (g->u.drawing->landscape == FALSE) offset = pointof(-CB.LL.x,-CB.LL.y);
+	if (GD_drawing(g)->landscape == FALSE) offset = pointof(-CB.LL.x,-CB.LL.y);
 	else { offset.x = (page.y + 1) * GP.y; offset.y = -page.x * GP.x; }
-	CodeGen->begin_page(page,g->u.drawing->scale,theta,offset);
+	CodeGen->begin_page(g,page,GD_drawing(g)->scale,theta,offset);
 	emit_background(g, CB.LL, CB.UR);
 	emit_defaults(g);
 }
@@ -646,7 +686,7 @@ void emit_label(textlabel_t* lp, Agraph_t *g)
 
 	CodeGen->begin_context();
 	CodeGen->set_pencolor(lp->fontcolor);
-	CodeGen->set_font(lp->fontname,lp->fontsize*g->u.drawing->font_scale_adj);
+	CodeGen->set_font(lp->fontname,lp->fontsize*GD_drawing(g)->font_scale_adj);
 
 	for (i = 0; i < lp->nlines; i++) {
 		switch(lp->line[i].just) {
@@ -714,38 +754,67 @@ static int style_delim(int c)
 	}
 }
 
-static char *
-style_token(char * p, char *out)
+#define SID 1
+
+static int
+style_token(char** s, xbuf *xb)
 {
+    char* p = *s;
+    int token;
+    char  c;
+
 	while (*p && (isspace(*p) || (*p ==','))) p++;
 	switch (*p) {
-		case '\0': return NULL;
-		case '(': case ')': *out++ = *p++; break;
-		default: while (style_delim(*p) == FALSE) *out++ = *p++;
+	case '\0': token = 0; break;
+	case '(': case ')': token = *p++; break;
+	default: 
+        token = SID;
+        while (!style_delim(c = *p)) {
+          xbputc(xb,c);
+          p++;
+        }
 	}
-	*out = '\0';
-	return p;
+    *s = p;
+	return token;
+}
+
+#define FUNLIMIT 64
+static unsigned char outbuf[SMALLBUF];
+static xbuf ps_xb;
+
+static void
+cleanup()
+{
+  xbfree(&ps_xb);
 }
 
 char **
 parse_style(char* s)
 {
-	static char	*parse[64];
-	static char	outbuf[SMALLBUF];
-
-	int			fun = 0;
-	boolean		in_parens = FALSE;
-	char		buf[SMALLBUF],*p,*q,*out;
+	static char*   parse[FUNLIMIT];
+    static int     first = 1;
+	int			   fun = 0;
+	boolean		   in_parens = FALSE;
+	unsigned char  buf[SMALLBUF];
+    char*          p;
+    int            c;
+    xbuf           xb;
 
+    if (first) {
+      xbinit (&ps_xb, SMALLBUF, outbuf);
+      atexit(cleanup);
+      first = 0;
+    }
 
+    xbinit (&xb, SMALLBUF, buf);
 	p = s;
-	out = outbuf;
-	while ((p = style_token(p,buf)) != 0) {
-		switch (buf[0]) {
+	while ((c = style_token(&p,&xb)) != 0) {
+		switch (c) {
 		case '(':
 			if (in_parens) {
 				fprintf(stderr,"nesting not allowed in style: %s\n",s);
 				parse[0] = (char*)0;
+                xbfree (&xb);
 				return parse;
 			}
 			in_parens = TRUE;
@@ -755,6 +824,7 @@ parse_style(char* s)
 			if (in_parens == FALSE) {
 				fprintf(stderr,"unmatched ')' in style: %s\n",s);
 				parse[0] = (char*)0;
+                xbfree (&xb);
 				return parse;
 			}
 			in_parens = FALSE; 
@@ -762,21 +832,29 @@ parse_style(char* s)
 
 		default:
 			if (in_parens == FALSE) {
-				*out++ = '\0';	/* terminate previous */
-				parse[fun++] = out;
+                if (fun == FUNLIMIT-1) {
+                    fprintf (stderr, "Warning: truncating style '%s'\n", s);
+	                parse[fun] = (char*)0;
+                    xbfree (&xb);
+                    return parse;
+                }
+                xbputc(&ps_xb, '\0'); /* terminate previous */
+				parse[fun++] = xbnext(&ps_xb);
 			}
-			q = buf;
-			while ((*out++ = *q++) != 0);
+			xbput (&ps_xb, xbuse(&xb));
+            xbputc(&ps_xb, '\0');
 		}
 	}
-	*out = '\0';
 
 	if (in_parens) {
 		fprintf(stderr,"unmatched '(' in style: %s\n",s);
 		parse[0] = (char*)0;
+        xbfree (&xb);
 		return parse;
 	}
 	parse[fun] = (char*)0;
+    xbfree (&xb);
+    xbuse (&ps_xb);  /* adds final '\0' to buffer */
 	return parse;
 }
 
@@ -787,12 +865,15 @@ static struct cg_s {
     }
     gens[] = {
         {&PS_CodeGen,"ps",POSTSCRIPT},
+        {&PS_CodeGen,"ps2",PDF},
         {&HPGL_CodeGen,"hpgl",HPGL},
         {&HPGL_CodeGen,"pcl",PCL},
         {&MIF_CodeGen,"mif",MIF},
         {&PIC_CodeGen,"pic",PIC_format},
         {&GD_CodeGen,"gd",GD},
+#ifdef HAVE_LIBZ
         {&GD_CodeGen,"gd2",GD2},
+#endif
         {&GD_CodeGen,"gif",GIF},
 #ifdef HAVE_LIBJPEG
         {&GD_CodeGen,"jpg",JPEG},
@@ -809,6 +890,7 @@ static struct cg_s {
 #endif
         {&ISMAP_CodeGen,"ismap",ISMAP},
         {&IMAP_CodeGen,"imap",IMAP},
+        {&CMAP_CodeGen,"cmap",CMAP},
 #ifdef HAVE_LIBPNG
         {&VRML_CodeGen,"vrml",VRML},
 #endif
@@ -823,6 +905,7 @@ static struct cg_s {
         {(codegen_t*)0,"canon",CANONICAL_DOT},
         {(codegen_t*)0,"plain",PLAIN},
         {(codegen_t*)0,"plain-ext",PLAIN_EXT},
+        {&XDot_CodeGen,"xdot",EXTENDED_DOT},
         {(codegen_t*)0,(char*)0,0}
     };
 
