@@ -9,20 +9,38 @@
 */
 #pragma prototyped
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifdef _UWIN
+#ifndef DEFAULT_FONTPATH
+#define		DEFAULT_FONTPATH	"/win/fonts"
+#endif  /* DEFAULT_FONTPATH */
+#else
+#ifndef MSWIN32
+#ifndef DEFAULT_FONTPATH
+#define		DEFAULT_FONTPATH	"/usr/share/ttf:/usr/local/share/ttf:/usr/share/fonts/ttf:/usr/local/share/fonts/ttf:/usr/lib/fonts:/usr/local/lib/fonts:/usr/lib/fonts/ttf:/usr/local/lib/fonts/ttf:/usr/common/graphviz/lib/fonts/ttf:/windows/fonts:/dos/windows/fonts:/usr/add-on/share/ttf:."
+#endif /* DEFAULT_FONTPATH */
+#else
+#ifndef DEFAULT_FONTPATH
+#define		DEFAULT_FONTPATH	"C:/WINDOWS/FONTS;C:/WINNT/Fonts;C:/winnt/fonts"
+#endif /* DEFAULT_FONTPATH */
+#endif /* MSWIN32 */
+#endif /* _UWIN */
+
 #include		"render.h"
 #include		"gd.h"
 #include		"utils.h"
 #include		<fcntl.h>
+#ifdef MSWIN32
+#include <io.h>
+#endif
 
 #ifdef DMALLOC
 #include "dmalloc.h"
 #endif
 
-#define		NONE		0
-#define		NODE		1
-#define		EDGE		2
-#define		CLST		3
-
 #define SCALE (GD_RESOLUTION/72.0)
 
 #define BEZIERSUBDIVISION 10
@@ -34,43 +52,41 @@
 
 /* patterns */
 #define P_SOLID		0
-#define P_NONE  15
-#define P_DOTTED 4				/* i wasn't sure about this */
-#define P_DASHED 11				/* or this */
+#define P_DOTTED	4
+#define P_DASHED	11
+#define P_NONE		15
 
 /* bold line constant */
-#define WIDTH_NORMAL 1
-#define WIDTH_BOLD 3
+#define WIDTH_NORMAL	1
+#define WIDTH_BOLD	3
 
 typedef struct {
 	unsigned char r, g, b;
 } Color;
 
 
-static FILE	*Outfile;
-/* static int	  Obj; */
-#if 0
-static int	 N_pages;
+static int	N_pages;
 static point	Pages;
-#endif
 static double   Scale;
 static pointf	Offset;
-static double   ArgScale;
-static int	  Rot;
-static box	  PB;
-static int	  onetime = TRUE;
+static double	ArgScale;
+static int	Rot;
+static box	PB;
+static int	onetime = TRUE;
 
 static gdImagePtr im;
+static Dict_t	*ImageDict;
 
 typedef struct context_t {
 	unsigned char	pencolor_ix, fillcolor_ix;
         char	*fontfam, fontopt, font_was_set, pen, fill, penwidth;
-	float   fontsz;
+	double   fontsz;
 } context_t;
 
-#define MAXNEST 4
+#define MAXNEST	4
 static context_t cstk[MAXNEST];
-static int	  SP;
+static int	SP;
+static node_t	*Curnode;
 
 static void gd_reset(void)
 {
@@ -83,16 +99,21 @@ static void init_gd(void)
 
 	SP = 0;
 	/* must create default background color first... */
-	white = gdImageColorResolve(im, 255, 255, 255);
-	if ((transparent = gdImageGetTransparent(im)) == -1) {
-		/* transparent uses an rgb value very close to white
-		   so that formats like GIF that don't support
-		   transparency show a white background */
-		transparent = gdImageColorResolve(im, 255, 255, 254);
-		gdImageColorTransparent(im, transparent);
-	}
+	white = gdImageColorResolve(im, 255,255,255);
+	/* we have to force the background to be filled for some reason */
+	gdImageFilledRectangle(im, 0, 0, im->sx-1, im->sy-1, white);
+
+	/* transparent uses an rgb value very close to white
+	   so that formats like GIF that don't support
+	   transparency show a white background */
+	/*transparent = gdImageColorResolve(im, 255, 255, 254);*/
+	transparent = gdImageColorResolveAlpha(im, 255, 255, 255, 127);
+
+	gdImageColorTransparent(im, transparent);
+
 	/* ...then any other colors that we need */
 	black = gdImageColorResolve(im, 0, 0, 0);
+	
 	cstk[0].pencolor_ix = black;		/* set pen black*/
 	cstk[0].fillcolor_ix = black;		/* set fill black*/
 	cstk[0].fontfam = "times";		/* font family name */
@@ -107,11 +128,11 @@ pointf gdpt(pointf p)
 	pointf		  rv;
 
 	if (Rot == 0) {
-		rv.x = p.x * Scale + Offset.x;
-		rv.y = PB.UR.y - PB.LL.y - p.y * Scale + Offset.y;
+		rv.x = PB.LL.x + p.x * Scale + Offset.x;
+		rv.y = PB.UR.y - 1 - p.y * Scale - Offset.y;
 	} else {
-		rv.x = PB.UR.x - PB.LL.x - p.y * Scale + Offset.x;
-		rv.y = PB.UR.y - PB.LL.y - p.x * Scale + Offset.y;
+		rv.x = PB.UR.x - 1 - p.y * Scale - Offset.x;
+		rv.y = PB.UR.y - 1 - p.x * Scale - Offset.y;
 	}
 	return rv;
 }
@@ -136,48 +157,60 @@ void gd_font(context_t* cp)
 static void gd_begin_job(FILE *ofp, graph_t *g, char **lib, char *user,
 char *info[], point pages)
 {
-	Outfile = ofp;
-	/* Pages = pages; */
-	/* N_pages = pages.x * pages.y; */
+	Pages = pages;
+	N_pages = pages.x * pages.y;
+#ifdef MYTRACE
+fprintf(stderr,"gd_begin_job\n");
+#endif
 }
 
-static void gd_end_job(void) { }
+static void gd_end_job(void)
+{
+#ifdef MYTRACE
+fprintf(stderr,"gd_end_job\n");
+#endif
+}
+
+void gd_begin_graph(graph_t* g, box bb, point pb)
+{
+	PB.LL.x = PB.LL.y = 0;
+	PB.UR.x = (bb.UR.x - bb.LL.x + 2*GD_drawing(g)->margin.x) * SCALE;
+	PB.UR.y = (bb.UR.y - bb.LL.y + 2*GD_drawing(g)->margin.y) * SCALE;
+	Offset.x = GD_drawing(g)->margin.x * SCALE;
+	Offset.y = GD_drawing(g)->margin.y * SCALE;
+}
 
 static void gd_begin_graph_to_file(graph_t* g, box bb, point pb)
 {
-	g = g;
-	PB.LL.x = bb.LL.x * SCALE;
-	PB.LL.y = bb.LL.y * SCALE;
-	PB.UR.x = bb.UR.x * SCALE;
-	PB.UR.y = bb.UR.y * SCALE;
-	Offset.x = PB.LL.x + 1;
-	Offset.y = PB.LL.y + 1;
+	gd_begin_graph(g, bb, pb);
 	if (Verbose)
-		fprintf(stderr,"dot: allocating a %dK GD image\n",
-			(PB.UR.x + PB.LL.x + 2) * (PB.UR.y + PB.LL.y + 2) / 1024);
-	im = gdImageCreate((PB.UR.x + PB.LL.x + 2), (PB.UR.y + PB.LL.y + 2));
+		fprintf(stderr,"%s: allocating a %dK GD image\n",
+			CmdName, (PB.UR.x - PB.LL.x - 1) * (PB.UR.y - PB.LL.y - 1) / 1024);
+	if (mapbool(agget(g,"truecolor")))
+	  im = gdImageCreateTrueColor((PB.UR.x - PB.LL.x - 1), (PB.UR.y - PB.LL.y - 1));
+	else im = gdImageCreate((PB.UR.x - PB.LL.x - 1), (PB.UR.y - PB.LL.y - 1));
 	if (onetime) {
 		init_gd();
 		onetime = FALSE;
 	}
+#ifdef MYTRACE
+fprintf(stderr,"gd_begin_graph_to_file\n");
+#endif
 }
 
 static void gd_begin_graph_to_memory(graph_t* g, box bb, point pb)
 {
-	g = g;
-	PB.LL.x = bb.LL.x * SCALE;
-	PB.LL.y = bb.LL.y * SCALE;
-	PB.UR.x = bb.UR.x * SCALE;
-	PB.UR.y = bb.UR.y * SCALE;
-	Offset.x = PB.LL.x + 1;
-	Offset.y = PB.LL.y + 1;
+	gd_begin_graph(g, bb, pb);
 	if (Verbose)
-		fprintf(stderr,"dot: using existing GD image\n");
+		fprintf(stderr,"%s: using existing GD image\n", CmdName);
 	im = *(gdImagePtr *)Output_file;
 	if (onetime) {
 		init_gd();
 		onetime = FALSE;
 	}
+#ifdef MYTRACE
+fprintf(stderr,"gd_begin_graph_to_memory\n");
+#endif
 }
 
 static void gd_end_graph_to_file(void)
@@ -187,7 +220,7 @@ static void gd_end_graph_to_file(void)
  */
 #ifdef HAVE_SETMODE
 #ifdef O_BINARY
-	setmode(fileno(Outfile), O_BINARY);
+	setmode(fileno(Output_file), O_BINARY);
 #endif
 #endif
 
@@ -202,120 +235,96 @@ static void gd_end_graph_to_file(void)
 #define JPEG_QUALITY -1
 
 	if (Output_lang == GIF) {
-		gdImageGif(im, Outfile);
+		gdImageGif(im, Output_file);
 #ifdef HAVE_LIBPNG
 #ifdef HAVE_LIBZ
 	} else if (Output_lang == PNG) {
-		gdImagePng(im, Outfile);
+		gdImagePng(im, Output_file);
 #endif
 #endif
 #ifdef HAVE_LIBJPEG
 	} else if (Output_lang == JPEG) {
-		gdImageJpeg(im, Outfile, JPEG_QUALITY);
+		gdImageJpeg(im, Output_file, JPEG_QUALITY);
 #endif
 	} else if (Output_lang == GD) {
-		gdImageGd(im, Outfile);
+		gdImageGd(im, Output_file);
+#ifdef HAVE_LIBZ
 	} else if (Output_lang == GD2) {
 #define GD2_CHUNKSIZE 128
 #define GD2_RAW 1
 #define GD2_COMPRESSED 2
-		gdImageGd2(im, Outfile, GD2_CHUNKSIZE, GD2_COMPRESSED);
+		gdImageGd2(im, Output_file, GD2_CHUNKSIZE, GD2_COMPRESSED);
+#endif
 	} else if (Output_lang == WBMP) {
         	/* Use black for the foreground color for the B&W wbmp image. */
         	int foreground = gdImageColorResolve(im, 0, 0, 0);
-		gdImageWBMP(im, foreground, Outfile);
+		gdImageWBMP(im, foreground, Output_file);
 #ifdef HAVE_LIBXPM
 	} else if (Output_lang == XBM) {
-		gdImageXbm(im, Outfile);
+		gdImageXbm(im, Output_file);
 #endif
 	}
+	if (ImageDict) {
+		dtclose(ImageDict); ImageDict = 0;
+	}
 	gdImageDestroy(im);
+#ifdef MYTRACE
+fprintf(stderr,"gd_end_graph_to_file\n");
+#endif
 }
 
 static void gd_end_graph_to_memory(void)
 {
 /* leave image in memory to be handled by Gdtclft output routines */
+#ifdef MYTRACE
+fprintf(stderr,"gd_end_graph_to_memory\n");
+#endif
 }
 
-static void
-gd_begin_page(point page, double scale, int rot, point offset)
+void
+gd_begin_page(graph_t *g, point page, double scale, int rot, point offset)
 {
-#if 0
-	int			 page_number;
+	int		page_number;
 	point		sz;
-#endif
 
 	ArgScale = scale;
 	Scale = scale * SCALE;
 	Rot = rot;
-	/* i guess we would use this if we were displaying page numbers */
-#if 0
+
 	page_number = page.x + page.y * Pages.x + 1;
 	sz = sub_points(PB.UR, PB.LL);
+#ifdef MYTRACE
+fprintf(stderr,"gd_begin_page\n");
+fprintf(stderr," page=%d,%d offset=%d,%d\n",page.x,page.y,offset.x,offset.y);
+fprintf(stderr," page_number=%d\n",page_number);
 #endif
 }
 
-static void gd_end_page(void) { }
-
-static void
-gd_begin_cluster(graph_t* g)
-{
-	/* Obj = CLST; */
+void gd_end_page(void) {
+#ifdef MYTRACE
+fprintf(stderr,"gd_end_page\n");
+#endif
 }
 
-static void
-gd_end_cluster(void)
-{
-	/* Obj = NONE; */
-}
+static void gd_begin_cluster(graph_t* g) { }
 
-static void
-gd_begin_nodes(void)
-{
-	/* Obj = NONE; */
-}
+static void gd_end_cluster(void) { } 
 
-static void
-gd_end_nodes(void)
-{
-	/* Obj = NONE; */
-}
+static void gd_begin_nodes(void) { }
 
-static void
-gd_begin_edges(void)
-{
-	/* Obj = NONE; */
-}
+static void gd_end_nodes(void) { }
 
-static void
-gd_end_edges(void)
-{
-	/* Obj = NONE; */
-}
+static void gd_begin_edges(void) { }
 
-static void
-gd_begin_node(node_t* n)
-{
-	/* Obj = NODE; */
-}
+static void gd_end_edges(void) { }
 
-static void
-gd_end_node(void)
-{
-	/* Obj = NONE; */
-}
+static void gd_begin_node(node_t* n) { Curnode = n; }
 
-static  void
-gd_begin_edge(edge_t* e)
-{
-	/* Obj = EDGE; */
-}
+static void gd_end_node(void) { }
 
-static  void
-gd_end_edge(void)
-{
-	/* Obj = NONE; */
-}
+static  void gd_begin_edge(edge_t* e) { }
+
+static  void gd_end_edge(void) { }
 
 static  void
 gd_begin_context(void)
@@ -362,7 +371,7 @@ static unsigned char gd_resolve_color(char* name)
  
 	if (!(strcmp(name,"transparent"))) {
 	    	/* special case for "transparent" color */
-		return gdImageColorResolve(im,255,255,254);
+		return gdImageGetTransparent(im);
 	} 
 	else {
 		colorxlate(name,&color,RGBA_BYTE);
@@ -388,23 +397,18 @@ gd_set_style(char** s)
 
 	cp = &(cstk[SP]);
 	while ((p = line = *s++)) {
-		if (streq(line, "solid")) { /* no-op */ }
-		else if (streq(line, "dashed"))
-			cp->pen = P_DASHED;
-                else if (streq(line, "dotted"))
- 			cp->pen = P_DOTTED;
-		else if (streq(line, "invis"))
-			cp->pen = P_NONE;
- 		else if (streq(line, "bold"))
- 			cp->penwidth = WIDTH_BOLD;
+		if (streq(line, "solid")) cp->pen = P_SOLID;
+		else if (streq(line, "dashed")) cp->pen = P_DASHED;
+                else if (streq(line, "dotted")) cp->pen = P_DOTTED;
+		else if (streq(line, "invis")) cp->pen = P_NONE;
+ 		else if (streq(line, "bold")) cp->penwidth = WIDTH_BOLD;
 		else if (streq(line, "setlinewidth")) {
 			while (*p) p++;
 			p++;
 			cp->penwidth = atol(p);
 		}
-		else if (streq(line, "filled"))
-			cp->fill = P_SOLID;
-		else if (streq(line, "unfilled")) { /* no-op */ }
+		else if (streq(line, "filled")) cp->fill = P_SOLID;
+		else if (streq(line, "unfilled")) cp->fill = P_NONE;
 		else fprintf(stderr,
 			"gd_set_style: unsupported style %s - ignoring\n",
 			line);
@@ -432,6 +436,10 @@ gd_alternate_fontlist(char *font)
 		fontlist = "Helvetica arial";
 	else if (strcasecmp(font,"Arial")==0)
 		fontlist = "Arial arial";
+	else if (strcasecmp(font,"arialb")==0)
+		fontlist = "arialb Arial-Bold";
+	else if (strcasecmp(font, "ariali")==0)
+		fontlist = "ariali Arial-Italic";
 	else if (strcasecmp(font,"Courier")==0)
 		fontlist = "Courier cour Courier-New Courier_New";
 	else if (strcasecmp(font,"Courier-New")==0)
@@ -458,13 +466,14 @@ void gd_missingfont(char *err, char *fontreq)
 	}
 }
 
+extern gdFontPtr gdFontTiny, gdFontSmall, gdFontMediumBold, gdFontLarge, gdFontGiant;
+
 static  void
 gd_textline(point p, textline_t *line)
 {
 	char		*fontlist, *err;
 	pointf		mp;
 	int		brect[8];
-	extern          gdFontPtr       gdFontSmall;
 	char		*str = line->str;
 	double		fontsz = cstk[SP].fontsz;
 
@@ -492,8 +501,27 @@ gd_textline(point p, textline_t *line)
 	if (err) {
 		/* revert to builtin fonts */
 	    	gd_missingfont (err, cstk[SP].fontfam);
-		gdImageString(im, gdFontSmall, ROUND(mp.x), ROUND(mp.y),
-			(unsigned char *)str, cstk[SP].pencolor_ix);
+		if (fontsz <= 8.5) {
+			gdImageString(im, gdFontTiny,
+				ROUND(mp.x), ROUND(mp.y-9.),
+				(unsigned char *)str, cstk[SP].pencolor_ix);
+		} else if (fontsz <= 9.5) {
+			gdImageString(im, gdFontSmall,
+				ROUND(mp.x), ROUND(mp.y-12.),
+				(unsigned char *)str, cstk[SP].pencolor_ix);
+		} else if (fontsz <= 10.5) {
+			gdImageString(im, gdFontMediumBold,
+				ROUND(mp.x), ROUND(mp.y-13.),
+				(unsigned char *)str, cstk[SP].pencolor_ix);
+		} else if (fontsz <= 11.5) {
+			gdImageString(im, gdFontLarge,
+				ROUND(mp.x), ROUND(mp.y-14.),
+				(unsigned char *)str, cstk[SP].pencolor_ix);
+		} else {
+			gdImageString(im, gdFontGiant,
+				ROUND(mp.x), ROUND(mp.y-15.),
+				(unsigned char *)str, cstk[SP].pencolor_ix);
+		}
 	}
 }
 
@@ -605,7 +633,7 @@ gd_polygon(point *A, int n, int filled)
 			pen = cstk[SP].pencolor_ix;
 		}
                 if (cstk[SP].penwidth != WIDTH_NORMAL) {
-			width=cstk[SP].penwidth;
+			width=cstk[SP].penwidth * Scale;
                         brush = gdImageCreate(width,width);
                         gdImagePaletteCopy(brush, im);
                         gdImageFilledRectangle(brush,
@@ -669,11 +697,9 @@ gd_ellipse(point p, int rx, int ry, int filled)
 		mp.x = p.x; mp.y = p.y;
 		mp = gdpt(mp);
 		if (filled) {
-			gdImageArc(im, ROUND(mp.x), ROUND(mp.y),
+			gdImageFilledEllipse(im, ROUND(mp.x), ROUND(mp.y),
 				ROUND(Scale*(rx + rx)), ROUND(Scale*(ry + ry)),
-				0, 360, cstk[SP].fillcolor_ix);
-			gdImageFillToBorder(im, ROUND(mp.x), ROUND(mp.y),
-				cstk[SP].fillcolor_ix, cstk[SP].fillcolor_ix);
+				cstk[SP].fillcolor_ix);
 		}
 		gdImageArc(im, ROUND(mp.x), ROUND(mp.y),
 			ROUND(Scale*(rx + rx)), ROUND(Scale*(ry + ry)), 0, 360, pen);
@@ -736,16 +762,135 @@ gd_polyline(point* A, int n)
 	}
 }
 
+static gdImagePtr loadimage(char *name)
+{
+	gdImagePtr	rv = 0;
+	char	*imagefile,*suffix;
+	FILE	*in = NULL;
+	char		*cmd = 0;
+#ifdef HAVE_CURL
+	char		*curlopts;
+#endif
+
+	imagefile = name;
+	if (!imagefile) return 0;
+	if (!strncmp(imagefile,"file:",5)) {
+		imagefile = &imagefile[5];
+		if (!strncmp(imagefile,"//",2)) imagefile = &imagefile[2];
+		in = fopen (imagefile, "r");
+		cmd = malloc(16);
+		strcpy(cmd,"fopen()");
+	}
+#ifdef HAVE_CURL
+	else {
+		cmd = malloc(strlen(imagefile) + strlen(CURLCMD) + 16);
+		/* the following is a feeble attempt to protect webdot */
+		if (getenv("WEBDOTSECURITY") || !(curlopts = agget(Curnode,"curlopts"))) curlopts = CURLOPTS;
+		sprintf(cmd,"%s %s \"%s\"",CURLCMD,curlopts,imagefile);
+		in = popen(cmd,"r");
+	}
+#endif
+	if (!in) 
+		fprintf(stderr,"%s: couldn't open image file %s via %s\n",CmdName,name,cmd);
+	else {
+	suffix = strrchr(imagefile,'.');
+	if (!suffix) suffix = imagefile; else suffix++;
+	if (!strcasecmp(suffix,"gif")) rv = gdImageCreateFromGif(in);
+#ifdef HAVE_LIBPNG
+	else if (!strcasecmp(suffix,"png")) rv = gdImageCreateFromPng(in);
+#endif
+#ifdef HAVE_LIBJPEG
+	else if (!strcasecmp(suffix,"jpeg")||!strcasecmp(suffix,"jpg")) rv = gdImageCreateFromJpeg(in);
+#endif
+	else if (!strcasecmp(suffix,"wbmp")) rv = gdImageCreateFromWBMP(in);
+	else if (!strcasecmp(suffix,"xbm")) rv = gdImageCreateFromXbm(in);
+	else fprintf(stderr,"%s: image file %s suffix not recognized\n",CmdName,name);
+	fclose(in);
+	if (!rv) fprintf(stderr,"%s: image file %s contents were not recognized\n",CmdName,name);
+	}
+	if (cmd) free(cmd);
+	return rv;
+}
+
+typedef struct imagerec_s {
+    Dtlink_t        link;
+    char            *name;
+	gdImagePtr		im;
+} imagerec_t;
+
+
+static void imagerec_free(Dict_t *dict, Void_t *p, Dtdisc_t *disc)
+{
+	gdImagePtr im = ((imagerec_t*)p)->im;
+	if (im) gdImageDestroy(im);
+}
+
+static Dtdisc_t ImageDictDisc = {
+    offsetof(imagerec_t,name),   /* key */
+    -1,                     /* size */
+    0,                      /* link offset */
+    NIL(Dtmake_f),
+	imagerec_free,
+    NIL(Dtcompar_f),
+    NIL(Dthash_f),
+    NIL(Dtmemory_f),
+    NIL(Dtevent_f)
+};
+
+static gdImagePtr getimage(char *name)
+{
+	imagerec_t	probe, *val;
+    if (!name) return 0;  /* cdt does not like NULL keys */
+	if (!ImageDict) ImageDict = dtopen(&ImageDictDisc,Dttree);
+	probe.name = name;
+	val = dtsearch(ImageDict,&probe);
+	if (!val) {
+		val = malloc(sizeof(imagerec_t));
+		val->name = name;
+		val->im = loadimage(name);
+		dtinsert(ImageDict,val);
+	}
+	return val->im;
+}
+
 static  void
 gd_user_shape(char *name, point *A, int n, int filled)
 {
-	static boolean  onetime = TRUE;
-	if (onetime) {
-		fprintf(stderr, "custom shapes not available with this driver\n");
-		onetime = FALSE;
+	gdImagePtr im2 = 0;
+	pointf	destul, destlr;
+	pointf	ul, lr;		/* upper left, lower right */
+	int		i;
+
+	im2 = getimage(agget(Curnode,"shapefile"));
+	if (im2) {
+		/* compute dest origin and size */
+		ul.x = lr.x = A[0].x; ul.y = lr.y = A[0].y;
+		for (i = 1; i < n; i++) {
+			if (ul.x > A[i].x) ul.x = A[i].x;
+			if (ul.y < A[i].y) ul.y = A[i].y;
+			if (lr.y > A[i].y) lr.y = A[i].y;
+			if (lr.x < A[i].x) lr.x = A[i].x;
+		}
+		destul = gdpt(ul);
+		destlr = gdpt(lr);
+		gdImageCopyResized(im,im2,ROUND(destul.x),ROUND(destul.y),0,0,ROUND(destlr.x - destul.x),ROUND(destlr.y - destul.y),im2->sx,im2->sy);
 	}
-	gd_polygon(A, n, filled);
+	gd_polygon(A, n, 0);
 }
+
+point gd_user_shape_size(node_t *n, char *imagefile)
+{
+	point		rv;
+	gdImagePtr	im;
+
+	Curnode = n;
+	im = getimage(imagefile);
+	if (im) {rv.x = im->sx / SCALE; rv.y = im->sy / SCALE; }
+	else rv.x = rv.y = 0;
+	return rv;
+}
+
+
 codegen_t	GD_CodeGen = {
 	gd_reset,
 	gd_begin_job, gd_end_job,
@@ -762,8 +907,10 @@ codegen_t	GD_CodeGen = {
 	gd_ellipse, gd_polygon,
 	gd_bezier, gd_polyline,
 	0 /* gd_arrowhead */, gd_user_shape,
-	0 /* gd_comment */, gd_textsize
+	0 /* gd_comment */, gd_textsize,
+	gd_user_shape_size
 };
+
 codegen_t	memGD_CodeGen = {		/* see tcldot */
 	gd_reset,
 	gd_begin_job, gd_end_job,
@@ -780,6 +927,6 @@ codegen_t	memGD_CodeGen = {		/* see tcldot */
 	gd_ellipse, gd_polygon,
 	gd_bezier, gd_polyline,
 	0 /* gd_arrowhead */, gd_user_shape,
-	0 /* gd_comment */, gd_textsize
+	0 /* gd_comment */, gd_textsize,
+	gd_user_shape_size
 };
-
