@@ -45,11 +45,9 @@ static  char    *clr_suffix = "\033%-12345X\n";
 static  char    *pcl_prefix = "\033E\n\033%%0B\n";
 static  char    *pcl_suffix = "\033%%0A\n";
 
-static	FILE	*Outfile;
 static	int	    N_pages;
 /* static 	point	Pages; */
 static	double	Scale;
-static  double	CurrentFontSize;
 static 	point	Origin;
 static	box		PB;
 static  int     CurrentPen;
@@ -71,10 +69,10 @@ static void output (char* str)
   while (*ptr != '\0') ptr++;
   len = ptr-str;
   if (bufcnt+len > MAXLINELEN) {
-    fputs("\n", Outfile);
+    fputs("\n", Output_file);
     bufcnt = 0;
   }
-  fputs(str,Outfile);
+  fputs(str,Output_file);
   if ((len > 0) && (*(ptr-1) == '\n')) bufcnt = 0;
   else bufcnt += len;
 }
@@ -89,18 +87,18 @@ static void output_text (char* str)
   while (*ptr != '\0') ptr++;
   len = (ptr-str) + strlen(text_tail) + strlen(text_hdr);
   if (bufcnt+len > MAXLINELEN) {
-    fputs("\n", Outfile);
+    fputs("\n", Output_file);
   }
-  fputs(text_hdr,Outfile);
-  fputs(str,Outfile);
-  fputs(text_tail,Outfile);
+  fputs(text_hdr,Output_file);
+  fputs(str,Output_file);
+  fputs(text_tail,Output_file);
   bufcnt = 0;
 }
 
 #ifdef SMILE
 void doSmile (void)
 {
-  fprintf(Outfile,
+  fprintf(Output_file,
     "SP1SD1,341,2,1,4,14,5,0,6,0,7,5SSLO7PA%d,0LB\001\003IN\n",
     (int)PT2UNIT(PageWid-2*HP_OX));
     
@@ -340,6 +338,7 @@ static int isInvis(void)
   return(curGC->style == INVIS);
 }
 
+#if 0 /* not used */
 static double _Xalign;
 #define getTextAlign() (_Xalign)
 static void
@@ -351,7 +350,6 @@ initTextAlign (void)
   output(buffer);
 }
 
-#if 0 /* not used */
 static int
 setTextAlign (double al)
 {
@@ -379,7 +377,6 @@ hpgl_reset(void)
 static void
 hpgl_begin_job(FILE *ofp, graph_t *g, char **lib, char *user, char *info[], point pages)
 {
-	Outfile = ofp;
 	/* Pages = pages; */
 	N_pages = pages.x * pages.y;
     if (Output_lang == PCL) {
@@ -392,10 +389,7 @@ hpgl_begin_job(FILE *ofp, graph_t *g, char **lib, char *user, char *info[], poin
     }
 }
 
-static  void
-hpgl_end_job(void)
-{
-}
+static  void hpgl_end_job(void) { }
 
 static void
 hpgl_begin_graph(graph_t* g, box bb, point pb)
@@ -404,10 +398,7 @@ hpgl_begin_graph(graph_t* g, box bb, point pb)
     PageWidth = pb.x;
 }
 
-static void
-hpgl_end_graph(void)
-{
-}
+static void hpgl_end_graph(void) { }
 
 static void hpgl_set_scale (double scx,double scy)
 {
@@ -417,7 +408,7 @@ static void hpgl_set_scale (double scx,double scy)
     output(buffer);
 }
 
-static void hpgl_begin_page(point page, double scale, int rot, point offset)
+static void hpgl_begin_page(graph_t *g, point page, double scale, int rot, point offset)
 {
     char   buffer[64];
     box    clipWin;
@@ -431,7 +422,9 @@ static void hpgl_begin_page(point page, double scale, int rot, point offset)
 #ifdef SMILE
     doSmile();
 #endif
+#if 0 /* not used */
     initTextAlign();
+#endif
     initGC();
 
     if (N_pages > 1) {
@@ -598,11 +591,10 @@ static void mkFontInfo(char* name, double size, FontInfo* fip)
 
 static void hpgl_set_font(char* name, double size)
 {
-    FontInfo fi;
+    static FontInfo fi;
 
     mkFontInfo(name,size,&fi);
     setFont(&fi);
-    CurrentFontSize = size;
 }
 
 static void hpgl_set_color(char* name)
@@ -638,11 +630,23 @@ static void hpgl_set_style(char** s)
 static void hpgl_textline(point p, textline_t *line)
 {
     char   buffer[128];
-	char   *str = line->str;
+    char   *str = line->str;
 
     if (isInvis()) return;
 
-    sprintf(buffer,"PA%d,%d%s",CX(p.x),CY(p.y-.3*CurrentFontSize), Sep);
+    switch(line->just) {
+	case 'l':
+	    break;
+	case 'r':
+	    p.x -= line->width;
+	    break;
+	default:
+	case 'n':
+	    p.x -= line->width / 2;
+	    break;
+    }
+
+    sprintf(buffer,"PA%d,%d%s",CX(p.x),CY(p.y), Sep);
     output(buffer);
     output_text(str);
 
