@@ -15,15 +15,8 @@
 #include "dmalloc.h"
 #endif
 
-#define	NONE	0
-#define	NODE	1
-#define	EDGE	2
-#define	CLST	3
-
 #define PIC_COORDS_PER_LINE (16)		/* to avoid stdio BUF overflow */
 
-static	FILE	*Outfile;
-/* static	int		Obj; */
 static	box		BB;
 static  int		BezierSubdivision = 10;
 static  int	Rot;
@@ -45,18 +38,9 @@ static int SP = 0;
 
 static char		picgen_msghdr[] = "dot picgen: ";
 static void unsupported(char* s)
-{ fprintf(stderr,"%s%s unsupported\n",picgen_msghdr,s); }
+{ fprintf(stderr,"warning - %s%s unsupported\n",picgen_msghdr,s); }
 static void warn(char* s)
 { fprintf(stderr,"%s%s warning\n",picgen_msghdr,s); }
-#undef DEBUG
-static void debug(char *s)
-{
-#ifdef	DEBUG
-fprintf(stderr, "%s%s\n",picgen_msghdr,s);
-fflush(stderr);
-fflush(Outfile);
-#endif
-}
 
 #undef	MAX
 #ifndef	MAX
@@ -135,7 +119,6 @@ static char *picfontname(char* psname)
 	char	*rv;
 	fontinfo *p;
 
-	debug("picfontname");
 	for (p = fonttab; p->psname; p++)
 		if (streq(p->psname,psname)) break;
 	if (p->psname) rv = p->trname;
@@ -153,61 +136,52 @@ static char *picfontname(char* psname)
 
 static char *pic_fcoord(char* buf, pointf pf)
 {
-	debug("pic_fcoord");
-	sprintf(buf,"(%.5lf,%.5lf)",Scale*pf.x,Scale*pf.y);
+	sprintf(buf,"(%.5f,%.5f)",Scale*pf.x,Scale*pf.y);
 	return buf;
 }
 
 static char *pic_coord(char* buf, point p)
 {
-	debug("pic_coord");
 	return pic_fcoord(buf,cvt2ptf(p));
 }
 
 static void pic_reset(void)
 {
-	debug("pic_reset");
 	onetime = TRUE;
 }
 
 static void pic_begin_job(FILE *ofp, graph_t *g, char **lib, char *user,
 char *info[], point pages)
 {
-	debug("pic_begin_job");
-	Outfile = ofp;
 	/* U_lib = lib; */
 	if (onetime && (pages.x * pages.y > 1)) {
 		unsupported("pagination");
 		onetime = FALSE;
 	}
-        fprintf(Outfile,"%s Creator: %s version %s (%s)\n",
+        fprintf(Output_file,"%s Creator: %s version %s (%s)\n",
                 EscComment, info[0], info[1],info[2]);
-        fprintf(Outfile,"%s For: %s\n",EscComment, user);
-        fprintf(Outfile,"%s Title: %s\n",EscComment, g->name);
+        fprintf(Output_file,"%s For: %s\n",EscComment, user);
+        fprintf(Output_file,"%s Title: %s\n",EscComment, g->name);
 }
 
-static void pic_end_job(void) { debug("pic_end_job"); }
+static void pic_end_job(void) { };
 
 static void pic_begin_graph(graph_t* g, box bb, point pb)
 {
-	debug("pic_begin_graph");
-	/*g = g;*/
 	BB = bb;
 
-	fprintf(Outfile,"%s save point size and font\n.nr .S \\n(.s\n.nr DF \\n(.f\n", EscComment);
+	fprintf(Output_file,"%s save point size and font\n.nr .S \\n(.s\n.nr DF \\n(.f\n", EscComment);
 }
 
 static void pic_end_graph(void)
 {
-	debug("pic_end_graph");
-	fprintf(Outfile,"%s restore point size and font\n.ps \\n(.S\n.ft \\n(DF\n", EscComment);
+	fprintf(Output_file,"%s restore point size and font\n.ps \\n(.S\n.ft \\n(DF\n", EscComment);
 }
 
-static void pic_begin_page(point page, double scale, int rot, point offset)
+static void pic_begin_page(graph_t *g, point page, double scale, int rot, point offset)
 {
 	double	height, width;
 
-	debug("pic_begin_page");
 	if (onetime && rot && (rot != 90)) {
 		unsupported("rotation");
 		onetime = FALSE;
@@ -217,156 +191,117 @@ static void pic_begin_page(point page, double scale, int rot, point offset)
 	width = PS2INCH((double)(BB.UR.x)-(double)(BB.LL.x));
 	Scale = scale;
 	if (Rot == 90) { double temp = width; width = height; height = temp; }
-	fprintf(Outfile,".PS %.5lf %.5lf\n", width, height);
+	fprintf(Output_file,".PS %.5f %.5f\n", width, height);
 	EscComment = "#";	/* pic comment */
-	fprintf(Outfile,"%s to change drawing size, multiply the width and height on the .PS line above and the number on the two lines below (rounded to the nearest integer) by a scale factor\n", EscComment);
+	fprintf(Output_file,"%s to change drawing size, multiply the width and height on the .PS line above and the number on the two lines below (rounded to the nearest integer) by a scale factor\n", EscComment);
 	if (width > 0.0) {
 		Fontscale = log10(width);
 		Fontscale += 3.0 - (int)Fontscale;	/* between 3.0 and 4.0 */
 	} else
 		Fontscale = 3.0;
 	Fontscale = pow(10.0, Fontscale);	/* a power of 10 times width, between 1000 and 10000 */
-	fprintf(Outfile,".nr SF %.0lf\nscalethickness = %.0lf\n", Fontscale, Fontscale);
-	fprintf(Outfile,"%s don't change anything below this line in this drawing\n", EscComment);
-	fprintf(Outfile,"scale=1.0 %s required for boxwid, etc. comparisons\n", EscComment);
-	fprintf(Outfile,"%s dashwid is 0.1 in 10th Edition, 0.05 in DWB 2 and in gpic\n", EscComment);
-	fprintf(Outfile,"%s fillval is 0.3 in 10th Edition (fill 0 means black), 0.5 in gpic (fill 0 means white), undefined in DWB 2\n", EscComment);
-	fprintf(Outfile,"%s arrowhead is undefined in DWB 2.0, initially 1 in gpic, 2 in 10th Edition\n", EscComment);
-	fprintf(Outfile,"%s fill has no meaning in DWB 2, gpic can use fill or filled, 10th Edition uses fill only\n", EscComment);
-	fprintf(Outfile,"%s boxwid is 0.75 in DWB 2 and 10th Edition(? check!), ever so slightly (approx. 1e-16) less in gpic [ditto for ellipsewid]\n", EscComment);
-	fprintf(Outfile,"%s DWB 2 doesn't use fill and doesn't define fillval\n", EscComment);
-	fprintf(Outfile,"%s reset works in gpic and 10th edition, but isn't defined in DWB 2\n", EscComment);
-	fprintf(Outfile,"%s if boxwid or dashwid have been changed before .PS, the following may fail\n", EscComment);
-	fprintf(Outfile,"if boxwid == 0.75 && dashwid == 0.05 then X fillval = 1; define fill Y Y; define solid Y Y; define reset Y scale=1.0 Y X\n");
-	fprintf(Outfile,"reset %s set to known state\n", EscComment);
-	fprintf(Outfile,"%s GNU pic vs. 10th Edition d\\(e'tente\n", EscComment);
-	fprintf(Outfile,"if fillval == 0.5 then X define setfillval Y fillval = 1 - Y; define bold Y thickness 2 Y; define solid Y Y X else Z define setfillval Y fillval = Y; define bold Y Y; define filled Y fill Y Z\n");
-	fprintf(Outfile,"%s arrowhead has no meaning in DWB 2.0, arrowhead = 7 makes filled arrowheads in gpic and in 10th Edition\n", EscComment);
-	fprintf(Outfile,"arrowhead = 7\n");
-	fprintf(Outfile,"%s GNU pic supports a boxrad variable to draw boxes with rounded corners; DWB and 10th Ed. do not\n", EscComment);
-	fprintf(Outfile,"boxrad = 0\n");
-	fprintf(Outfile,"%s GNU pic supports a linethick variable to set line thickness; DWB and 10th Ed. do not\n", EscComment);
-	fprintf(Outfile,"linethick = 0; oldlinethick = linethick\n");
-	fprintf(Outfile,"%s .PS w/o args causes GNU pic to scale drawing to fit 8.5x11 paper; DWB does not\n", EscComment);
-	fprintf(Outfile,"%s maxpsht and maxpswid have no meaning in DWB 2.0, set page boundaries in gpic and in 10th Edition\n", EscComment);
-	fprintf(Outfile,"%s maxpsht and maxpswid are predefined to 11.0 and 8.5 in gpic\n", EscComment);
-	fprintf(Outfile,"maxpsht = %f\nmaxpswid = %f\n", height, width);
-	fprintf(Outfile,"Dot: [\n");
-	fprintf(Outfile,"define attrs0 %% %%; define unfilled %% %%; define rounded %% %%; define diagonals %% %%\n");
+	fprintf(Output_file,".nr SF %.0f\nscalethickness = %.0f\n", Fontscale, Fontscale);
+	fprintf(Output_file,"%s don't change anything below this line in this drawing\n", EscComment);
+	fprintf(Output_file,"%s non-fatal run-time pic version determination, version 2\n", EscComment);
+	fprintf(Output_file,"boxrad=2.0 %s will be reset to 0.0 by gpic only\n", EscComment);
+	fprintf(Output_file,"scale=1.0 %s required for comparisons\n", EscComment);
+	fprintf(Output_file,"%s boxrad is now 0.0 in gpic, else it remains 2.0\n", EscComment);
+	fprintf(Output_file,"%s dashwid is 0.1 in 10th Edition, 0.05 in DWB 2 and in gpic\n", EscComment);
+	fprintf(Output_file,"%s fillval is 0.3 in 10th Edition (fill 0 means black), 0.5 in gpic (fill 0 means white), undefined in DWB 2\n", EscComment);
+	fprintf(Output_file,"%s fill has no meaning in DWB 2, gpic can use fill or filled, 10th Edition uses fill only\n", EscComment);
+	fprintf(Output_file,"%s DWB 2 doesn't use fill and doesn't define fillval\n", EscComment);
+	fprintf(Output_file,"%s reset works in gpic and 10th edition, but isn't defined in DWB 2\n", EscComment);
+	fprintf(Output_file,"%s DWB 2 compatibility definitions\n", EscComment);
+	fprintf(Output_file,"if boxrad > 1.0 && dashwid < 0.075 then X\n\tfillval = 1;\n\tdefine fill Y Y;\n\tdefine solid Y Y;\n\tdefine reset Y scale=1.0 Y;\nX\n");
+	fprintf(Output_file,"reset %s set to known state\n", EscComment);
+	fprintf(Output_file,"%s GNU pic vs. 10th Edition d\\(e'tente\n", EscComment);
+	fprintf(Output_file,"if fillval > 0.4 then X\n\tdefine setfillval Y fillval = 1 - Y;\n\tdefine bold Y thickness 2 Y;\n");
+	fprintf(Output_file,"\t%s if you use gpic and it barfs on encountering \"solid\",\n", EscComment);
+	fprintf(Output_file,"\t%s\tinstall a more recent version of gpic or switch to DWB or 10th Edition pic;\n", EscComment);
+	fprintf(Output_file,"\t%s\tsorry, the groff folks changed gpic; send any complaint to them;\n", EscComment);
+	fprintf(Output_file,"X else Z\n\tdefine setfillval Y fillval = Y;\n\tdefine bold Y Y;\n\tdefine filled Y fill Y;\nZ\n");
+	fprintf(Output_file,"%s arrowhead has no meaning in DWB 2, arrowhead = 7 makes filled arrowheads in gpic and in 10th Edition\n", EscComment);
+	fprintf(Output_file,"%s arrowhead is undefined in DWB 2, initially 1 in gpic, 2 in 10th Edition\n", EscComment);
+	fprintf(Output_file,"arrowhead = 7 %s not used by graphviz\n", EscComment);
+	fprintf(Output_file,"%s GNU pic supports a boxrad variable to draw boxes with rounded corners; DWB and 10th Ed. do not\n", EscComment);
+	fprintf(Output_file,"boxrad = 0 %s no rounded corners in graphviz\n", EscComment);
+	fprintf(Output_file,"%s GNU pic supports a linethick variable to set line thickness; DWB and 10th Ed. do not\n", EscComment);
+	fprintf(Output_file,"linethick = 0; oldlinethick = linethick\n");
+	fprintf(Output_file,"%s .PS w/o args causes GNU pic to scale drawing to fit 8.5x11 paper; DWB does not\n", EscComment);
+	fprintf(Output_file,"%s maxpsht and maxpswid have no meaning in DWB 2.0, set page boundaries in gpic and in 10th Edition\n", EscComment);
+	fprintf(Output_file,"%s maxpsht and maxpswid are predefined to 11.0 and 8.5 in gpic\n", EscComment);
+	fprintf(Output_file,"maxpsht = %f\nmaxpswid = %f\n", height, width);
+	fprintf(Output_file,"Dot: [\n");
+	fprintf(Output_file,"define attrs0 %% %%; define unfilled %% %%; define rounded %% %%; define diagonals %% %%\n");
 }
 
 static void pic_end_page(void)
 {
-	debug("pic_end_page");
-	fprintf(Outfile,"]\n.PE\n");
+	fprintf(Output_file,"]\n.PE\n");
 	EscComment = ".\\\" ";	/* troff comment */
 	assert(SP == 0);
 }
 
-static void pic_begin_cluster(graph_t* g)
-{
-	debug("pic_begin_cluster");
-	/* Obj = CLST; */
-}
+static void pic_begin_cluster(graph_t* g) { }
 
-static void pic_end_cluster(void)
-{
-	debug("pic_end_cluster");
-	/* Obj = NONE; */
-}
+static void pic_end_cluster(void) { }
 
-static void
-pic_begin_nodes(void)        
-{
-	debug("pic_begin_nodes");
-	/* Obj = NONE; */
-}
+static void pic_begin_nodes(void) { }
     
-static void
-pic_end_nodes(void)  
-{
-	debug("pic_end_nodes");
-	/* Obj = NONE; */
+static void pic_end_nodes(void) { }
+
+static void pic_begin_edges(void) { }            
+
+static void pic_end_edges(void) { }            
+
+static void pic_begin_node(node_t* n) {
+	fprintf(Output_file,"%s\t%s\n",EscComment,n->name);
 }
 
-static void 
-pic_begin_edges(void)        
-{
-	debug("pic_begin_edges");
-	/* Obj = NONE; */
-}            
-
-static void
-pic_end_edges(void)  
-{            
-	debug("pic_end_edges");
-	/* Obj = NONE; */
-}            
-
-static void pic_begin_node(node_t* n)
-{
-	debug("pic_begin_node");
-	/* Obj = NODE; */
-	fprintf(Outfile,"%s\t%s\n",EscComment,n->name);
-}
-
-static void pic_end_node (void)
-{
-	debug("pic_end_node");
-	/* Obj = NONE; */
-}
+static void pic_end_node (void) { }
 
 static void pic_begin_edge (edge_t* e)
 {
-	debug("pic_begin_edge");
-	/* Obj = EDGE; */
-	fprintf(Outfile,"%s\t%s -> %s\n",EscComment,e->tail->name,e->head->name);
+	fprintf(Output_file,"%s\t%s -> %s\n",EscComment,e->tail->name,e->head->name);
 }
 
-static void pic_end_edge (void)
-{
-	debug("pic_end_edge");
-	/* Obj = NONE; */
-}
+static void pic_end_edge (void) { }
 
 static void pic_begin_context(void)
 {
-	debug("pic_begin_context");
-	fprintf(Outfile,"{\n");
+	fprintf(Output_file,"{\n");
 	if (SP == STACKSIZE - 1) warn("stk ovfl");
 	else {
 		SP++; S[SP] = S[SP-1];
-		fprintf(Outfile,"define attrs%d %% %%\n", SP); /* ensure plain (no attributes) style at start of context */
+		fprintf(Output_file,"define attrs%d %% %%\n", SP); /* ensure plain (no attributes) style at start of context */
 	}
 }
 
 static void pic_end_context(void)
 {
-	debug("pic_end_context");
 	if (SP == 0) warn("stk undfl");
 	else {
 		SP--;
-		fprintf(Outfile,"}\n");	/* end context group */
+		fprintf(Output_file,"}\n");	/* end context group */
 		/* restore correct font and size for context */
 		if (S[SP+1].font && (!(S[SP].font) || strcmp(S[SP+1].font,S[SP].font)))
-			fprintf(Outfile, ".ft %s\n", picfontname(S[SP].font));
+			fprintf(Output_file, ".ft %s\n", picfontname(S[SP].font));
 		if (S[SP+1].size != S[SP].size) {
 			int	sz;
 
 			if ((sz = (int)(S[SP].size * Scale)) < 1)
 				sz = 1;
-			fprintf(Outfile,".ps %d*\\n(SFu/%.0lfu\n",sz, Fontscale);
+			fprintf(Output_file,".ps %d*\\n(SFu/%.0fu\n",sz, Fontscale);
 		}
-	fprintf(Outfile,"linethick = oldlinethick\n");
+	fprintf(Output_file,"linethick = oldlinethick\n");
 	}
 }
 
 static void pic_set_font(char* name, double size)
 {
-	debug("pic_set_font");
 	if (name && (!(S[SP].font) || strcmp(S[SP].font,name))) {
 		S[SP].font = name;
-		fprintf(Outfile,".ft %s\n",picfontname(name));
+		fprintf(Output_file,".ft %s\n",picfontname(name));
 	}
 	if (size != S[SP].size) {
 		int	sz;
@@ -374,7 +309,7 @@ static void pic_set_font(char* name, double size)
 		S[SP].size = size;
 		if ((sz = (int)(size * Scale)) < 1)
 			sz = 1;
-		fprintf(Outfile,".ps %d*\\n(SFu/%.0lfu\n",sz, Fontscale);
+		fprintf(Output_file,".ps %d*\\n(SFu/%.0fu\n",sz, Fontscale);
 	}
 }
 
@@ -401,10 +336,13 @@ pic_string(char *s)
 		if (*s == '\015')	/* GACK, PTUI! Fire up the teletype, boys;
 					   somebody's sending an old-fashioned mechanical
 					   "carriage return" control character. */
-			continue;
+			{s++; continue;}
 		if (*s == '\\') {
-			*p++ = '\\';
-			pos++;
+			strcpy(p,"\\(rs");	/* e.g. man 5 groff_char from mkssoftware.com*/
+			p += 4;
+			pos += 4;
+			s++;
+			continue;
 		}
 		*p++ = *s++;
 		pos++;
@@ -419,13 +357,25 @@ static void pic_textline(point p, textline_t *line)
 	short	flag = 0;
 	double	fontsz = S[SP].size;
 
-	debug("pic_textline");
+	switch(line->just) {
+		case 'l':
+			p.x = p.x;
+			break;
+		case 'r':
+			p.x = p.x - line->width;
+			break;
+		default:
+		case 'n':
+			p.x = p.x - line->width / 2;
+		break;
+	}
 	pf = cvt2ptf(p);
 #ifdef NOTDEF
 	/* Why on earth would we want this? SCN  11/29/2001 */
 	pf.y -= fontsz / (5.0 * POINTS_PER_INCH);
 #endif
-	pf.y += fontsz / (5.0 * POINTS_PER_INCH);
+	/* Why on earth would we do this either. But it works. SCN 2/26/2002 */
+	pf.y += fontsz / (3.0 * POINTS_PER_INCH);
 	pf.x += line->width/(2.0 * POINTS_PER_INCH);
 	if (!(S[SP].size)) {	/* size was never set in this or hierarchically higher context */
 		pic_set_font(S[SP].font, fontsz);	/* primarily to output font and/or size directives */
@@ -438,7 +388,7 @@ static void pic_textline(point p, textline_t *line)
 		pic_begin_context();
 		pic_set_font(S[SP-1].font, fontsz);
 	}
-	fprintf(Outfile,"\"%s\" at (%.5lf,%.5lf);\n",
+	fprintf(Output_file,"\"%s\" at (%.5f,%.5f);\n",
 			pic_string(line->str),Scale*pf.x,Scale*pf.y);
 	if (flag)
 		pic_end_context();
@@ -448,11 +398,10 @@ static void pic_set_color(char* name)
 {
 	color_t	color;
 
-	debug("pic_set_color");
 	S[SP].color = name;
 	colorxlate(name,&color,HSV_DOUBLE);
 	/* just v used to set grayscale value */
-	fprintf(Outfile,"setfillval %f\n", color.u.HSV[2]);
+	fprintf(Output_file,"setfillval %f\n", color.u.HSV[2]);
 }
 
 static void pic_set_style(char** s) {
@@ -460,36 +409,34 @@ static void pic_set_style(char** s) {
 	char	skip = 0;
 	char	buf[BUFSIZ];
 
-	debug("pic_set_style");
 	buf[0]='\0';
-	fprintf(Outfile, "define attrs%d %%", SP);
+	fprintf(Output_file, "define attrs%d %%", SP);
 	while ((p = line = *s++)) {
 		while (*p) p++; p++;
 		while (*p) {
 			if (!strcmp(line, "setlinewidth")) {	/* a hack to handle the user-defined (PS) style spec in proc3d.dot */
 				long n = atol(p);
 
-				sprintf(buf,"oldlinethick = linethick;linethick = %ld * scalethickness / %.0lf\n", n, Fontscale / Scale);
+				sprintf(buf,"oldlinethick = linethick;linethick = %ld * scalethickness / %.0f\n", n, Fontscale / Scale);
 				skip = 1;
 			} else
-				fprintf(Outfile," %s",p);
+				fprintf(Output_file," %s",p);
 			while (*p) p++; p++;
 		}
 		if (!skip)
-			fprintf(Outfile," %s", line);
+			fprintf(Output_file," %s", line);
 		skip = 0;
 	}
-	fprintf(Outfile," %%\n");
-	fprintf(Outfile,"%s", buf);
+	fprintf(Output_file," %%\n");
+	fprintf(Output_file,"%s", buf);
 }
 
 static void pic_ellipse(point p, int rx, int ry, int filled)
 {
 	pointf	pf;
 
-	debug("pic_ellipse");
 	pf = cvt2ptf(p);
-	fprintf(Outfile,"ellipse attrs%d %swid %.5lf ht %.5lf at (%.5lf,%.5lf);\n",
+	fprintf(Output_file,"ellipse attrs%d %swid %.5f ht %.5f at (%.5f,%.5f);\n",
 		SP, filled? "fill ": "",
 		Scale*PS2INCH(2*rx), Scale*PS2INCH(2*ry), Scale*pf.x, Scale*pf.y);
 }
@@ -499,18 +446,16 @@ static void point_list_out(point* A, int n, int close)
 	int		j;
 	char	buf[SMALLBUF];
 
-	debug("pic_point_list_out");
 	for (j = 0; j < n; j ++)
-		fprintf(Outfile,"P%d: %s\n", j, pic_coord(buf,A[j]));
+		fprintf(Output_file,"P%d: %s\n", j, pic_coord(buf,A[j]));
 	for (j = 0; j+1 < n; j ++)
-		fprintf(Outfile,"move to P%d; line attrs%d to P%d\n", j, SP, j+1);
+		fprintf(Output_file,"move to P%d; line attrs%d to P%d\n", j, SP, j+1);
 	if (close)
-		fprintf(Outfile,"move to P%d; line attrs%d to P0\n", n-1, SP);
+		fprintf(Output_file,"move to P%d; line attrs%d to P0\n", n-1, SP);
 }
 
 static void pic_polygon(point *A, int n, int filled)
 {
-	debug("pic_polygon");
 	/* test for special case: rectangle oriented with page */
 	if ((n==4) && (
 	((A[0].x == A[1].x) && (A[0].y == A[3].y) && (A[1].y == A[2].y) && (A[2].x == A[3].x))
@@ -525,9 +470,9 @@ static void pic_polygon(point *A, int n, int filled)
 			color_t		color;
 
 			colorxlate(S[SP].color,&color,HSV_DOUBLE);
-			fprintf(Outfile,"setfillval %f\n", color.u.HSV[2]);
+			fprintf(Output_file,"setfillval %f\n", color.u.HSV[2]);
 		}
-		fprintf(Outfile,"box attrs%d %swid %.5lf ht %.5lf at (%.5lf,%.5lf);\n",
+		fprintf(Output_file,"box attrs%d %swid %.5f ht %.5f at (%.5f,%.5f);\n",
 			SP, filled ? "fill " : "",
 			Scale*fabs(pf1.x-pf2.x), Scale*fabs(pf1.y-pf2.y),	/* width, height */
 			Scale*(pf1.x+pf2.x)/2.0, Scale*(pf1.y+pf2.y)/2.0);	/* center coordinates */
@@ -542,18 +487,16 @@ static void pic_polygon(point *A, int n, int filled)
 
 static void pic_polyline(point* A, int n)
 {
-	debug("pic_polyline");
 	point_list_out(A,n,FALSE);
 }
 
 static void pic_user_shape(char *name, point *A, int sides, int filled)
 {
-	debug("pic_user_shape");
 	/* it's not at all clear what xxx_user_shape is supposed to do; in most xxxgen.c files it emits a message or draws a polygon */
 	/* this defines the shape as a macro and then invokes the macro */
-	fprintf(Outfile, "define %s {\n", name);
+	fprintf(Output_file, "define %s {\n", name);
 	pic_polygon(A, sides, filled);
-	fprintf(Outfile, "}\n%s\n", name);
+	fprintf(Output_file, "}\n%s\n", name);
 }
 
 static void pic_bezier(point *A, int n, int arrow_at_start, int arrow_at_end)
@@ -562,7 +505,6 @@ static void pic_bezier(point *A, int n, int arrow_at_start, int arrow_at_end)
 	int		i,j,m,step;
 	char	buf[SMALLBUF];
 
-	debug("pic_bezier");
 	if (arrow_at_start || arrow_at_end)
 		warn("not supposed to be making arrows here!");
 	V[3] = cvt2ptf(A[0]);	/* initial cond */
@@ -572,15 +514,15 @@ static void pic_bezier(point *A, int n, int arrow_at_start, int arrow_at_end)
 			V[j] = cvt2ptf(A[i+j]);
 		p = Bezier(V, 3, 0.0, NULL, NULL);
 		if (!i)
-			fprintf(Outfile,"P0: %s\n", pic_fcoord(buf,p));
+			fprintf(Output_file,"P0: %s\n", pic_fcoord(buf,p));
 		for (step = 1; step <= BezierSubdivision; step++) {
-			p = Bezier(V, 3, (float)step/BezierSubdivision, NULL, NULL);
+			p = Bezier(V, 3, (double)step/BezierSubdivision, NULL, NULL);
 			++m;
-			fprintf(Outfile,"P%d: %s\n", m, pic_fcoord(buf,p));
+			fprintf(Output_file,"P%d: %s\n", m, pic_fcoord(buf,p));
 		}
 	}
 	for (i=0; i+2 <= m; i+=2)	/* DWB 2 pic suffers from severe roundoff errors if too many steps are plotted at once */
-		fprintf(Outfile,"move to P%d; line attrs%d to P%d then to P%d\n", i, SP, i+1, i+2); /* use line, as splines can't be dotted or dashed */
+		fprintf(Output_file,"move to P%d; line attrs%d to P%d then to P%d\n", i, SP, i+1, i+2); /* use line, as splines can't be dotted or dashed */
 }
 
 static void
@@ -588,9 +530,8 @@ pic_comment(void* obj, attrsym_t* sym)
 {
 	char	*str;
 
-	debug("pic_comment");
 	str = late_string(obj,sym,"");
-	if (str[0]) fprintf(Outfile,"'\\\" %s\n",str);
+	if (str[0]) fprintf(Output_file,"'\\\" %s\n",str);
 }
 
 codegen_t	PIC_CodeGen = {
