@@ -34,6 +34,10 @@
 
  /* this needs an overhaul */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include	"render.h"
 
 #ifdef DMALLOC
@@ -73,14 +77,18 @@ static void
 storeline(textlabel_t *lp, char *line, char terminator,graph_t *g)
 {
 	double	width = 0.0;
+	int	iwidth = 0;
 
 	lp->line = ALLOC(lp->nlines+2,lp->line,textline_t);
 	lp->line[lp->nlines].str = line;
 	if (line) {
 		if (CodeGen && CodeGen->textsize)
-			width = (CodeGen->textsize(line, lp->fontname,lp->fontsize).x);
-		else
-			width = estimate_textsize(line, lp->fontname,lp->fontsize).x;
+			iwidth = (CodeGen->textsize(line, lp->fontname,lp->fontsize).x);
+		if (!iwidth)
+			iwidth = estimate_textsize(line, lp->fontname,lp->fontsize).x;
+
+		width = (double)iwidth;
+
 	}
 	lp->line[lp->nlines].width = width;
 	lp->line[lp->nlines].just = terminator;
@@ -99,32 +107,51 @@ storeline(textlabel_t *lp, char *line, char terminator,graph_t *g)
 pointf label_size(char *str, textlabel_t *lp, graph_t *g)
 {
 	char		c,*p,*line,*lineptr;
+	unsigned char byte = 0x00;
+
+	if (*str == '\0') return lp->dimen;
 
 	line = lineptr = NULL;
 	p = str;
 	line = lineptr = (char*) malloc(strlen(p) + 1);
 	*line = 0;
 	while ((c = *p++)) {
-		if (c & ~0x7f) g->u.has_Latin1char = TRUE;
-		if (c == '\\') {
-			switch (*p) {
-				case 'n': case 'l': case 'r':
-					*lineptr++ = '\0';
-					storeline(lp,line,*p,g);
-					line = lineptr;
-					break;
-				default:
-					*lineptr++ = *p;
+		byte = (unsigned int )c;
+		if (c & ~0x7f) GD_has_Latin1char(g) = TRUE;
+		//Fix some Double Character error for Big5 (Start)
+		if (0xA1 <= byte && byte <= 0xFE) {		
+			*lineptr++ = c;
+			c = *p++;
+			*lineptr++ = c;
+		//Fix some Double Character error for Big5 (End)
+		}else{
+			if (c == '\\') {
+				switch (*p) {
+					case 'n': case 'l': case 'r':
+						*lineptr++ = '\0';
+						storeline(lp,line,*p,g);
+						line = lineptr;
+						break;
+					default:
+						*lineptr++ = *p;
+				}
+				if (*p) p++;
+			/* tcldot can enter real linend characters */
+			} else if (c == '\n') {
+				*lineptr++ = '\0';
+				storeline(lp,line,'n',g);
+				line = lineptr;
+			}else{
+				*lineptr++ = c;
 			}
-			if (*p) p++;
 		}
-		else *lineptr++ = c;
 	}
 
 	if (line != lineptr) {
 		*lineptr++ = '\0';
 		storeline(lp,line,'n',g);
 	}
+
 	return lp->dimen;
 }
 
@@ -139,19 +166,20 @@ unrecognized(node_t* n, char* p)
 void	poly_init(node_t *);
 void	poly_free(node_t *);
 port_t	poly_port(node_t*, char *);
-int		poly_inside(node_t*, pointf, edge_t *);
+int	poly_inside(node_t*, pointf, edge_t *);
 void	poly_gencode(node_t*);
+void	point_init(node_t *);
 
 void	record_init(node_t *);
 void	record_free(node_t *);
 port_t	record_port(node_t*, char *);
-int		record_inside(node_t*, pointf, edge_t *);
-int		record_path(node_t* n, edge_t* e, int pt, box rv[], int* kptr);
+int	record_inside(node_t*, pointf, edge_t *);
+int	record_path(node_t* n, edge_t* e, int pt, box rv[], int* kptr);
 void	record_gencode(node_t*);
 
 void	epsf_init(node_t *);
 void	epsf_free(node_t *);
-int		epsf_inside(node_t*, pointf, edge_t *);
+int	epsf_inside(node_t*, pointf, edge_t *);
 void	epsf_gencode(node_t*);
 
 /* polygon descriptions.  "polygon" with 0 sides takes all user control */
@@ -169,7 +197,9 @@ static polygon_t p_diamond	  = { FALSE,  1,    4,    45.,  0.,   0. };
 static polygon_t p_trapezium	  = { FALSE,  1,    4,    0.,   -.4,  0. };
 static polygon_t p_parallelogram  = { FALSE,  1,    4,    0.,   0.,   .6 };
 static polygon_t p_house	  = { FALSE,  1,    5,    0.,   -.64, 0. };
+static polygon_t p_pentagon	  = { FALSE,  1,    5,    0.,   0.,   0. };
 static polygon_t p_hexagon	  = { FALSE,  1,    6,    0.,   0.,   0. };
+static polygon_t p_septagon	  = { FALSE,  1,    7,    0.,   0.,   0. };
 static polygon_t p_octagon	  = { FALSE,  1,    8,    0.,   0.,   0. };
 /* redundant and undocumented builtin polygons */
 static polygon_t p_doublecircle   = { TRUE,   2,    1,    0.,   0.,   0. };
@@ -187,6 +217,7 @@ static shape_desc Shapes[]  = {	/* first entry is default for no such shape */
 {"polygon"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_polygon	},
 {"ellipse"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_ellipse	},
 {"circle"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_circle	},
+{"point"	,point_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_circle	},
 {"egg"		,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_egg		},
 {"triangle"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_triangle	},
 {"plaintext"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_plaintext	},
@@ -194,9 +225,13 @@ static shape_desc Shapes[]  = {	/* first entry is default for no such shape */
 {"trapezium"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_trapezium	},
 {"parallelogram",poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_parallelogram	},
 {"house"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_house		},
+{"pentagon"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_pentagon	},
 {"hexagon"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_hexagon	},
+{"septagon"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_septagon	},
 {"octagon"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_octagon	},
 /* redundant and undocumented builtin polygons */
+{"rect"		,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_box		},
+{"rectangle"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_box		},
 {"doublecircle"	,poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_doublecircle	},
 {"doubleoctagon",poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_doubleoctagon	},
 {"tripleoctagon",poly_init,poly_free,poly_port,poly_inside,NULL,poly_gencode,&p_tripleoctagon	},
@@ -221,6 +256,57 @@ static double quant(double val, double q)
 	return i * q;
 }
 
+
+#define ATTR_SET(a,n) ((a) && (*(agxget(n,a->index)) != '\0'))
+#define DEF_POINT 0.05
+  /* extra null character needed to avoid style emitter from thinking
+   * there are arguments.
+   */
+static char* point_style[2] = {"filled\0",0}; 
+static shape_desc* point_desc;
+
+/* point_init:
+ * shorthand for shape=circle, style=filled, width=0.05, label=""
+ */
+void point_init(node_t* n)
+{
+	textlabel_t* p;
+
+	if (!point_desc) {
+		shape_desc	*ptr;
+		for (ptr = Shapes; ptr->name; ptr++)
+			if (!strcmp(ptr->name,"point")) {point_desc = ptr; break;}
+		assert(point_desc);
+	}
+
+		/* adjust label to "" */
+	p = ND_label(n);
+	if (p->nlines != 0) free(p->line[0].str);
+	free(p->line);
+    p->nlines = 0;
+	p->text = "";
+	p->dimen.x = 0;
+	p->dimen.y = 0;
+      
+		/* set width and height, and make them equal
+		 * if user has set weight or height, use it.
+		 * if both are set, use smallest.
+		 * if neither, use default
+		 */
+    if (ATTR_SET(N_width,n)) {
+    	if (ATTR_SET(N_height,n)) {
+			ND_width(n) = ND_height(n) = MIN(ND_width(n),ND_height(n));
+    	}
+		else ND_height(n) = ND_width(n);
+    }
+    else if (ATTR_SET(N_height,n)) {
+		ND_width(n) = ND_height(n);
+    }
+	else ND_width(n) = ND_height(n) = DEF_POINT;
+
+	poly_init (n);
+}
+
 void poly_init(node_t* n)
 {
 	pointf	dimen;
@@ -234,34 +320,48 @@ void poly_init(node_t* n)
 	int	i,j,outp;
 	polygon_t *poly=NEW(polygon_t);
 
-	regular = n->u.shape->polygon->regular;
-	peripheries = n->u.shape->polygon->peripheries;
-	sides = n->u.shape->polygon->sides;
-	orientation = n->u.shape->polygon->orientation;
-	skew = n->u.shape->polygon->skew;
-	distortion = n->u.shape->polygon->distortion;
+	regular = ND_shape(n)->polygon->regular;
+	peripheries = ND_shape(n)->polygon->peripheries;
+	sides = ND_shape(n)->polygon->sides;
+	orientation = ND_shape(n)->polygon->orientation;
+	skew = ND_shape(n)->polygon->skew;
+	distortion = ND_shape(n)->polygon->distortion;
 
 	regular |= mapbool(agget(n,"regular"));
 	peripheries = late_int(n,N_peripheries,peripheries,0);
-	orientation += late_float(n,N_orientation,0.0,-360.0);
+	orientation += late_double(n,N_orientation,0.0,-360.0);
 	if (sides==0) { /* not for builtins */
-		skew = late_float(n,N_skew,0.0,-100.0);
+		skew = late_double(n,N_skew,0.0,-100.0);
 		sides = late_int(n,N_sides,4,0);
-		distortion = late_float(n,N_distortion,0.0,-100.0);
+		distortion = late_double(n,N_distortion,0.0,-100.0);
 	}
 
 	/* get label dimensions */
-	dimen = n->u.label->dimen;
+	dimen = ND_label(n)->dimen;
 
 	if (mapbool(late_string(n,N_fixed,"false"))) {
-		if ((n->u.width < dimen.x) || (n->u.height < dimen.y))
-			fprintf(stderr,"dot: warning, node '%s' size too small for label\n",
-				n->name);
+		if ((ND_width(n) < dimen.x) || (ND_height(n) < dimen.y))
+			fprintf(stderr,"%s: warning, node '%s' size too small for label\n",
+				CmdName, n->name);
 		dimen.x = dimen.y = 0;
 	}
+	else {	
+		if (ND_shape(n)->usershape && CodeGen && CodeGen->usershapesize) {
+			point	imagesize;
+			char*   sfile = agget(n,"shapefile");
+
+			imagesize = CodeGen->usershapesize(n,sfile);
+            if ((imagesize.x == 0) && (imagesize.y == 0)) {
+				fprintf (stderr, "No or improper shapefile=\"%s\" for user-defined shape=\"%s\" on node \"%s\"\n", 
+					(sfile ? sfile : "<nil>"), agget(n,"shape"), n->name);
+ 			}
+			dimen.x = MAX(dimen.x,PS2INCH(imagesize.x));
+			dimen.y = MAX(dimen.y,PS2INCH(imagesize.y));
+		}
+	}
 
 	/* quantization */
-	if ((temp = n->graph->u.drawing->quantum) > 0.0) {
+	if ((temp = GD_drawing(n->graph)->quantum) > 0.0) {
 		dimen.x = quant(dimen.x,temp);
 		dimen.y = quant(dimen.y,temp);
 	}
@@ -269,7 +369,7 @@ void poly_init(node_t* n)
 	/* make square if necessary */
 	if (regular) {
 		/* make x and y dimensions equal */
-		n->u.width = n->u.height = MIN(n->u.width,n->u.height);
+		ND_width(n) = ND_height(n) = MIN(ND_width(n),ND_height(n));
 		xb = yb = MAX(dimen.x,dimen.y);
 	} else {
 		xb = dimen.x; yb = dimen.y;
@@ -298,7 +398,7 @@ void poly_init(node_t* n)
 		else temp = yb * (sqrt(2.) - 1.);
 		xb += temp; yb += temp;
         }
-	xb=MAX(n->u.width,xb); yb=MAX(n->u.height,yb);
+	xb=MAX(ND_width(n),xb); yb=MAX(ND_height(n),yb);
 	outp=peripheries;
 	if (peripheries<1) outp=1;
 	if (sides<3) { /* ellipses */
@@ -354,7 +454,7 @@ void poly_init(node_t* n)
 
 		/* apply minimum dimensions */
 		xmax *=2.; ymax *=2.;
-		xb=MAX(n->u.width,xmax); yb=MAX(n->u.height,ymax);
+		xb=MAX(ND_width(n),xmax); yb=MAX(ND_height(n),ymax);
 		scalex=xb/xmax; scaley=yb/ymax;
 	
 		for (i=0; i<sides; i++) {
@@ -405,14 +505,14 @@ void poly_init(node_t* n)
 	poly->distortion = distortion;
 	poly->vertices = vertices;
 
-	n->u.width = xb;
-	n->u.height = yb;
-	n->u.shape_info = (void*) poly;
+	ND_width(n) = xb;
+	ND_height(n) = yb;
+	ND_shape_info(n) = (void*) poly;
 }
 
 void poly_free(node_t *n)
 {
-	polygon_t* p = n->u.shape_info;
+	polygon_t* p = ND_shape_info(n);
 
 	if (p) {
 		free(p->vertices);
@@ -433,26 +533,26 @@ int poly_inside(node_t* n, pointf p, edge_t* e)
 	pointf		P,Q,R;
 
 	e = e;
-	P = (n->graph->u.left_to_right? flip_ptf(p) : p);
+	P = (GD_left_to_right(n->graph)? flip_ptf(p) : p);
 	if (n != lastn) {
-		poly = (polygon_t*) n->u.shape_info;
+		poly = (polygon_t*) ND_shape_info(n);
 		vertex = poly->vertices;
 		sides = poly->sides;
 		lastn = n;
 
 		/* get point and node size adjusted for rankdir=LR */
-		if (n->graph->u.left_to_right) {
-			ysize = n->u.lw + n->u.rw; xsize = n->u.ht;
+		if (GD_left_to_right(n->graph)) {
+			ysize = ND_lw(n) + ND_rw(n); xsize = ND_ht(n);
 		}
 		else {
-			xsize = n->u.lw + n->u.rw; ysize = n->u.ht;
+			xsize = ND_lw(n) + ND_rw(n); ysize = ND_ht(n);
 		}
 
         	/* scale */
 		if (xsize == 0.0) xsize = 1.0;
 		if (ysize == 0.0) ysize = 1.0;
-		scalex = n->u.width/xsize; scaley = n->u.height/ysize;
-		box_URx = n->u.width/2.0; box_URy = n->u.height/2.0;
+		scalex = ND_width(n)/xsize; scaley = ND_height(n)/ysize;
+		box_URx = ND_width(n)/2.0; box_URy = ND_height(n)/2.0;
 
 		/* index to outer-periphery */
 		outp=(poly->peripheries-1)*sides;
@@ -510,7 +610,7 @@ poly_port(node_t* n, char* pname)
 {
 	static char *points_of_compass[] =
 		{"n","ne","e","se","s","sw","w","nw",NULL};
-static struct {char x,y;} a[] = {{0,1},{1,1},{1,0},{1,-1},{0,-1},{-1,-1},{-1,0},{-1,1}};
+static struct {signed char x,y;} a[] = {{0,1},{1,1},{1,0},{1,-1},{0,-1},{-1,-1},{-1,0},{-1,1}};
 
 	int		i,ht,wd;
 	port_t	rv;
@@ -526,11 +626,11 @@ static struct {char x,y;} a[] = {{0,1},{1,1},{1,0},{1,-1},{0,-1},{-1,-1},{-1,0},
 		rv = Center;
 	}
 	else {
-		ht = n->u.ht / 2; 
-		wd = n->u.lw;
+		ht = ND_ht(n) / 2; 
+		wd = ND_lw(n);
 		rv.p.x = a[i].x * wd;
 		rv.p.y = a[i].y * ht;
-		rv.order = (MC_SCALE * (n->u.lw + rv.p.x)) / (n->u.lw + n->u.rw);
+		rv.order = (MC_SCALE * (ND_lw(n) + rv.p.x)) / (ND_lw(n) + ND_rw(n));
 		rv.constrained = FALSE;
 		rv.defined = TRUE;
 	}
@@ -554,8 +654,14 @@ static void fillcolor(node_t *n)
 	if (! color[0]) {
 		/* for backward compatibilty, default fill is same as pen */
 		color = late_nnstring(n,N_color,"");
-		if (! color[0])
-			color = (Output_lang == MIF ? "black" : DEFAULT_FILL);
+		if (! color[0]) {
+			if (ND_shape(n) == point_desc) {
+				color = "black";
+			}
+			else {
+				color = (Output_lang == MIF ? "black" : DEFAULT_FILL);
+			}
+		}
 	}
 	CodeGen->set_fillcolor(color);
 }
@@ -577,7 +683,7 @@ static int stylenode(node_t* n)
 			if (strcmp(pstyle[i],"diagonals") == 0) {istyle |= DIAGONALS; continue;}
 		}
 	}
-	if ((poly = n->u.shape->polygon)) istyle |= poly->option;
+	if ((poly = ND_shape(n)->polygon)) istyle |= poly->option;
 	return istyle;
 }
 
@@ -593,7 +699,7 @@ void poly_gencode(node_t* n)
 	static int		A_size;
 	int			filled;
 	
-	poly = (polygon_t*) n->u.shape_info;
+	poly = (polygon_t*) ND_shape_info(n);
 	vertices = poly->vertices;
 	sides = poly->sides;
 	peripheries = poly->peripheries;
@@ -602,8 +708,8 @@ void poly_gencode(node_t* n)
 	CodeGen->begin_node(n);
 	CodeGen->begin_context();
 /* prescale by 16.0 to help rounding trick below */
-	xsize = ((n->u.lw + n->u.rw) / n->u.width) * 16.0;
-	ysize = ((n->u.ht) / n->u.height) * 16.0;
+	xsize = ((ND_lw(n) + ND_rw(n)) / ND_width(n)) * 16.0;
+	ysize = ((ND_ht(n)) / ND_height(n)) * 16.0;
 
 	/* this is bad, but it's because of how the VRML driver works */
 #ifdef HAVE_LIBPNG
@@ -612,7 +718,11 @@ void poly_gencode(node_t* n)
 	}
 #endif
 
-	style = stylenode(n); 
+	if (ND_shape(n) == point_desc) {
+		CodeGen->set_style(point_style);
+		style = FILLED;
+    }
+	else style = stylenode(n); 
 	pencolor(n); /* emit pen color */
 	if (style & FILLED) fillcolor(n); /* emit fill color */
 	for (j = 0; j < peripheries; j++) {
@@ -623,7 +733,7 @@ void poly_gencode(node_t* n)
  * (note xsize/ysize prescaled by 16.0 above) */
 			A[i].x = ROUND(P.x * xsize) / 16;
 			A[i].y = ROUND(P.y * ysize) / 16;
-			if (sides > 2) {A[i].x += n->u.coord.x; A[i].y += n->u.coord.y;}
+			if (sides > 2) {A[i].x += ND_coord(n).x; A[i].y += ND_coord(n).y;}
 		}
 		if (!j && (style & FILLED)) {
 			/* fill innermost periphery only */
@@ -632,11 +742,11 @@ void poly_gencode(node_t* n)
 		else {
 			filled = 0;
 		}
-		if (find_user_shape(n->u.shape->name)) {
-			CodeGen->user_shape(n->u.shape->name,A,sides,filled);
+		if (ND_shape(n)->usershape && CodeGen->user_shape) {
+			CodeGen->user_shape(ND_shape(n)->name,A,sides,filled);
 		}
 		else if (sides <= 2) {
-			CodeGen->ellipse(n->u.coord,A[0].x,A[0].y,filled);
+			CodeGen->ellipse(ND_coord(n),A[0].x,A[0].y,filled);
 			if (style & DIAGONALS) {
 				Mcircle_hack(n);
 			}
@@ -650,8 +760,8 @@ void poly_gencode(node_t* n)
 	}
 
 	if (style & AUXLABELS) Mlabel_hack(n);
-    	n->u.label->p = n->u.coord;
-	emit_label(n->u.label,n->graph);
+    	ND_label(n)->p = ND_coord(n);
+	emit_label(ND_label(n),n->graph);
 	CodeGen->end_context();
 	CodeGen->end_node();
 }
@@ -663,12 +773,12 @@ hack1(node_t* n, char* str, int k)
 	double		fontsize;
 	textline_t	fake;
 
-	fontsize = n->u.label->fontsize*.8;	/* magic? */
+	fontsize = ND_label(n)->fontsize*.8;	/* magic? */
 
-	p.x = n->u.coord.x - (strlen(str) * n->u.label->fontsize) / 2;
-	p.y = n->u.coord.y + (k * (n->u.ht - n->u.label->fontsize - 2)) / 2;
+	p.x = ND_coord(n).x - (strlen(str) * ND_label(n)->fontsize) / 2;
+	p.y = ND_coord(n).y + (k * (ND_ht(n) - ND_label(n)->fontsize - 2)) / 2;
 	CodeGen->begin_context();
-	CodeGen->set_font(n->u.label->fontname,fontsize);
+	CodeGen->set_font(ND_label(n)->fontname,fontsize);
 	fake.str = str;
 	fake.width = strlen(str) * fontsize; /* ugh */
 	fake.just = 0; /* does this field do anything? SCN */
@@ -685,15 +795,15 @@ void Mlabel_hack(node_t* n)
 
 void Mcircle_hack(node_t* n)
 {
-	float	x,y;
+	double	x,y;
 	point	A[2],p;
 
 	y = .7500;
 	x = .6614;	/* x^2 + y^2 = 1.0 */
-	p.y = y * n->u.ht / 2.0;
-	p.x = n->u.rw * x;	/* assume node is symmetric */
+	p.y = y * ND_ht(n) / 2.0;
+	p.x = ND_rw(n) * x;	/* assume node is symmetric */
 
-	A[0] = add_points(p,n->u.coord);
+	A[0] = add_points(p,ND_coord(n));
 	A[1].y = A[0].y; A[1].x = A[0].x - 2*p.x;
 	CodeGen->polyline(A,2);
 	A[0].y -= 2*p.y; A[1].y = A[0].y;
@@ -791,7 +901,7 @@ parse_reclbl(node_t *n, int LR, int flag, char* text)
 						*(tsp - 1) == ' ')
 					tsp--;
 				*tsp = '\000';
-				fp->lp = make_label (strdup (text), n->u.label->fontsize, n->u.label->fontname, n->u.label->fontcolor,n->graph);
+				fp->lp = make_label (strdup (text), ND_label(n)->fontsize, ND_label(n)->fontname, ND_label(n)->fontcolor,n->graph);
 				fp->LR = TRUE;
 				hstsp = tsp = text;
 			}
@@ -909,30 +1019,41 @@ record_init(node_t* n)
     int     len;
     char*   textbuf;   /* temp buffer for storing labels */
 
-	reclblp = n->u.label->text;
+	reclblp = ND_label(n)->text;
     len = strlen (reclblp);
     textbuf = N_NEW(len+1,char);
-	if (!(info = parse_reclbl(n,NOT(n->graph->u.left_to_right), TRUE, textbuf))) {
-		fprintf (stderr, "bad label format %s\n", n->u.label->text);
+	if (!(info = parse_reclbl(n,NOT(GD_left_to_right(n->graph)), TRUE, textbuf))) {
+		fprintf (stderr, "bad label format %s\n", ND_label(n)->text);
 		reclblp = "\\N";
-		info = parse_reclbl(n,NOT(n->graph->u.left_to_right), TRUE, textbuf);
+		info = parse_reclbl(n,NOT(GD_left_to_right(n->graph)), TRUE, textbuf);
 	}
     free (textbuf);
+
 	size_reclbl(n,info);
-	sz.x = POINTS(n->u.width);  sz.y = POINTS(n->u.height);
-	sz.x = MAX(info->size.x,sz.x); sz.y = MAX(info->size.y,sz.y);
+	sz.x = POINTS(ND_width(n));  sz.y = POINTS(ND_height(n));
+    if (mapbool(late_string(n,N_fixed,"false"))) {
+		if ((sz.x < info->size.x) || (sz.y < info->size.y)) {
+/* should check that the record really won't fit, e.g., there may be no text.
+			fprintf(stderr,"%s: warning, node '%s' size may be too small\n",
+				CmdName, n->name);
+ */
+		}
+    }
+    else {
+		sz.x = MAX(info->size.x,sz.x); sz.y = MAX(info->size.y,sz.y);
+    }
 	resize_reclbl(info,sz);
 	ul = pointof(-sz.x/2,sz.y/2);
 	pos_reclbl(info,ul);
-	n->u.width = PS2INCH(info->size.x);
-	n->u.height = PS2INCH(info->size.y);
-	n->u.shape_info = (void*) info;
+	ND_width(n) = PS2INCH(info->size.x);
+	ND_height(n) = PS2INCH(info->size.y);
+	ND_shape_info(n) = (void*) info;
 }
 
 void
 record_free(node_t* n)
 {
-	field_t* p = n->u.shape_info;
+	field_t* p = ND_shape_info(n);
 
 	free(p);
 }
@@ -960,15 +1081,15 @@ record_port(node_t* n, char* pname)
 	port_t	rv;
 
 	if (pname[0] != ':') return Center;		/*could be '\000' */
-	if ((f = map_rec_port((field_t*) n->u.shape_info,pname+1)) == NULL) {
+	if ((f = map_rec_port((field_t*) ND_shape_info(n),pname+1)) == NULL) {
 		unrecognized(n,pname);
 		return Center;
 	}
 
 	b = f->b;
 	rv.p = pointof((b.LL.x+b.UR.x)/2,(b.LL.y+b.UR.y)/2);
-	if (n->graph->u.left_to_right) rv.p = invflip_pt(rv.p);
-	rv.order = (MC_SCALE * (n->u.lw + rv.p.x)) / (n->u.lw + n->u.rw);
+	if (GD_left_to_right(n->graph)) rv.p = invflip_pt(rv.p);
+	rv.order = (MC_SCALE * (ND_lw(n) + rv.p.x)) / (ND_lw(n) + ND_rw(n));
 	rv.constrained = FALSE;
 	rv.defined = TRUE;
 	return rv;
@@ -986,17 +1107,17 @@ record_inside(node_t* n, pointf p, edge_t* e)
 	static node_t	*last_n;
 	static field_t	*fld;
 
-	if (n->graph->u.left_to_right) p = flip_ptf(p);
-	for (f = e; f->u.edge_type != NORMAL; f = f->u.to_orig);
+	if (GD_left_to_right(n->graph)) p = flip_ptf(p);
+	for (f = e; ED_edge_type(f) != NORMAL; f = ED_to_orig(f));
 	e = f;
 	if ((e != last_e) || (n != last_n)) {
 		last_e = e; last_n = n;
 		pname = agget(e,(n == f->head ? "headport" : "tailport"));
-		fld = map_rec_port((field_t*)n->u.shape_info,pname+1);
+		fld = map_rec_port((field_t*)ND_shape_info(n),pname+1);
 	}
 
 	if (fld == NULL) {
-		fld0 = (field_t*) n->u.shape_info;
+		fld0 = (field_t*) ND_shape_info(n);
 		UR.x = fld0->size.x / 2.0; LL.x = -UR.x;
 		UR.y = fld0->size.y / 2.0; LL.y = -UR.y;
 	}
@@ -1027,34 +1148,34 @@ record_path(node_t* n, edge_t* e, int pt, box rv[], int* kptr)
 	point		p;
 	field_t		*info;
 
-	if (pt == 1) p = e->u.tail_port.p;
-	else p = e->u.head_port.p;
-	info = (field_t*) n->u.shape_info;
+	if (pt == 1) p = ED_tail_port(e).p;
+	else p = ED_head_port(e).p;
+	info = (field_t*) ND_shape_info(n);
 
 	for (i = 0; i < info->n_flds; i++) {
-		if (n->graph->u.left_to_right == FALSE)
+		if (GD_left_to_right(n->graph) == FALSE)
 			{ ls = info->fld[i]->b.LL.x; rs = info->fld[i]->b.UR.x; }
 		else
 			{ ls = info->fld[i]->b.LL.y; rs = info->fld[i]->b.UR.y; }
 		if (BETWEEN(ls,p.x,rs)) {
 			/* FIXME: I don't understand this code */
-			if (n->graph->u.left_to_right) {
-				rv[0] = flip_rec_box(info->fld[i]->b,n->u.coord);
+			if (GD_left_to_right(n->graph)) {
+				rv[0] = flip_rec_box(info->fld[i]->b,ND_coord(n));
 			}
 			else {
-				rv[0].LL.x = n->u.coord.x + ls;
-				rv[0].LL.y = n->u.coord.y - n->u.ht/2;
-				rv[0].UR.x = n->u.coord.x + rs;
+				rv[0].LL.x = ND_coord(n).x + ls;
+				rv[0].LL.y = ND_coord(n).y - ND_ht(n)/2;
+				rv[0].UR.x = ND_coord(n).x + rs;
 			}
 #if 0
 			s0 = (rv[0].UR.x - rv[0].LL.x)/6;
-			s0 = MIN(s0,n->graph->u.nodesep);
+			s0 = MIN(s0,n->GD_nodesep(graph));
 			s1 = MIN(p.x - rv[0].LL.x,rv[0].UR.x - p.x)/2;
 			sep = MIN(s0,s1);
 			rv[0].LL.x += sep;
 			rv[0].UR.x -= sep;
 #endif
-			rv[0].UR.y = n->u.coord.y + n->u.ht/2;
+			rv[0].UR.y = ND_coord(n).y + ND_ht(n)/2;
 			*kptr = 1;
 			break;
 		}
@@ -1071,8 +1192,8 @@ gen_fields(node_t* n, field_t* f)
 	point		A[2];
 
 	if (f->lp) {
-		cx = (f->b.LL.x + f->b.UR.x)/2.0 + n->u.coord.x;
-		cy = (f->b.LL.y + f->b.UR.y)/2.0 + n->u.coord.y;
+		cx = (f->b.LL.x + f->b.UR.x)/2.0 + ND_coord(n).x;
+		cy = (f->b.LL.y + f->b.UR.y)/2.0 + ND_coord(n).y;
 		f->lp->p =  pointof((int)cx,(int)cy);
 		emit_label(f->lp,n->graph);
 	}
@@ -1095,8 +1216,8 @@ gen_fields(node_t* n, field_t* f)
 				A[0].x = f->fld[i]->b.LL.x;
 				A[0].y = A[1].y;
 			}
-			A[0] = add_points(A[0],n->u.coord);
-			A[1] = add_points(A[1],n->u.coord);
+			A[0] = add_points(A[0],ND_coord(n));
+			A[1] = add_points(A[1],ND_coord(n));
 			CodeGen->polyline(A,2);
 		}
 		gen_fields(n,f->fld[i]);
@@ -1110,18 +1231,18 @@ record_gencode(node_t* n)
 	int		i,style;
 	field_t	*f;
 
-	f = (field_t*) n->u.shape_info;
+	f = (field_t*) ND_shape_info(n);
 	A[0] = f->b.LL;
 	A[2] = f->b.UR;
 	A[1].x = A[2].x; A[1].y = A[0].y;
 	A[3].x = A[0].x; A[3].y = A[2].y;
-	for (i = 0; i < 4; i++) A[i] = add_points(A[i],n->u.coord);
+	for (i = 0; i < 4; i++) A[i] = add_points(A[i],ND_coord(n));
 	CodeGen->begin_node(n);
 	CodeGen->begin_context();
 	style = stylenode(n);
 	pencolor(n);
 	if (style & FILLED) fillcolor(n); /* emit fill color */
-	if (streq(n->u.shape->name,"Mrecord")) style |= ROUNDED;
+	if (streq(ND_shape(n)->name,"Mrecord")) style |= ROUNDED;
 	if (style & (ROUNDED | DIAGONALS)) round_corners(n,A,4,ROUNDED);
 	else CodeGen->polygon(A,4,style&FILLED);
 	gen_fields(n,f);
@@ -1156,7 +1277,10 @@ user_shape(char* name)
 	p = UserShape[i] = NEW(shape_desc);
 	*p = Shapes[0];
 	p->name = name;
-	if (Lib == NULL) fprintf(stderr, "warning: using %s for unknown shape %s\n", Shapes[0].name,p->name);
+	p->usershape = TRUE;
+	if ((Lib == NULL) && (!CodeGen || (CodeGen->usershapesize == NULL))) 
+      fprintf(stderr, "warning: using %s for unknown shape %s\n", 
+        Shapes[0].name,p->name);
 	return p;
 }
 
@@ -1243,7 +1367,7 @@ epsf_inside(node_t* n, pointf p, edge_t* e)
 	pointf	P;
 	double	x2;
 
-	P = (n->graph->u.left_to_right? flip_ptf(p) : p);
-	x2 = n->u.ht / 2;
-	return ((P.y >= -x2) && (P.y <= x2) && (P.x >= -n->u.lw) && (P.x <= n->u.rw));
+	P = (GD_left_to_right(n->graph)? flip_ptf(p) : p);
+	x2 = ND_ht(n) / 2;
+	return ((P.y >= -x2) && (P.y <= x2) && (P.x >= -ND_lw(n)) && (P.x <= ND_rw(n)));
 }
