@@ -1,4 +1,5 @@
-#include "gvconfig.h"
+#include <utils.h>
+
 #ifndef HAVE_STRCASECMP
 
 #include <string.h>
@@ -11,7 +12,8 @@
 int
 strcasecmp(char *s1, char *s2)
 {
-	while (*s1 != '\0' && tolower(*s1) == tolower(*s2))
+	while ((*s1 != '\0') 
+        && (tolower(*(unsigned char *)s1) == tolower(*(unsigned char *)s2)))
 	{
 		s1++;
 		s2++;
