@@ -1,4 +1,5 @@
-#include "gvconfig.h"
+#include <utils.h>
+
 #ifndef HAVE_STRNCASECMP
 
 #include <string.h>
@@ -9,15 +10,16 @@
 #endif
 
 int 
-strncasecmp(char *s1, char *s2, size_t n)
+strncasecmp(char *s1, char *s2, unsigned int n)
 {
 	if (n == 0)
 		return 0;
 
-	while (n-- != 0 && tolower(*s1) == tolower(*s2))
-		{
+	while ((n-- != 0)
+            && (tolower(*(unsigned char *)s1) == tolower(*(unsigned char *)s2)))
+	{
 		if (n == 0 || *s1 == '\0' || *s2 == '\0')
-			break;
+			return 0;
 		s1++;
 		s2++;
 	}
