@@ -15,10 +15,10 @@
 
 #include	"dot.h"
 #ifdef HAVE_CONFIG_H
-#include "gvconfig.h"
+#include "config.h"
 #endif
 #include	<time.h>
-#ifndef MSWIN32
+#ifdef HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
 
@@ -29,7 +29,7 @@
 char *Info[] = {
     "dot",              /* Program */
     VERSION,            /* Version */
-    DATE                /* Build Date */
+    BUILDDATE           /* Build Date */
 };
 
 static graph_t *G;
@@ -54,7 +54,10 @@ int main(int argc, char** argv)
 #endif
 
 	while ((G = next_input_graph())) {
-		if (prev) dot_cleanup(prev);
+		if (prev) {
+          dot_cleanup(prev);
+          agclose(prev);
+		}
 		prev = G;
 		dot_layout(G);
 		dotneato_write(G);
