@@ -24,10 +24,10 @@ void acyclic(graph_t* g)
 	int		c;
 	node_t	*n;
 
-	for (c = 0; c < g->u.comp.size; c++) {
-		g->u.nlist = g->u.comp.list[c];
-		for (n = g->u.nlist; n; n = n->u.next) n->u.mark = FALSE;
-		for (n = g->u.nlist; n; n = n->u.next) dfs(n);
+	for (c = 0; c < GD_comp(g).size; c++) {
+		GD_nlist(g) = GD_comp(g).list[c];
+		for (n = GD_nlist(g); n; n = ND_next(n)) ND_mark(n) = FALSE;
+		for (n = GD_nlist(g); n; n = ND_next(n)) dfs(n);
 	}
 }
 
@@ -37,15 +37,15 @@ void dfs(node_t* n)
 	edge_t	*e;
 	node_t	*w;
 	
-	if (n->u.mark) return;
-	n->u.mark = TRUE;
-	n->u.onstack = TRUE;
-	for (i = 0; (e = n->u.out.list[i]); i++) {
+	if (ND_mark(n)) return;
+	ND_mark(n) = TRUE;
+	ND_onstack(n) = TRUE;
+	for (i = 0; (e = ND_out(n).list[i]); i++) {
 		w = e->head;
-		if (w->u.onstack) { reverse_edge(e); i--; }
-		else { if (w->u.mark == FALSE) dfs(w); }
+		if (ND_onstack(w)) { reverse_edge(e); i--; }
+		else { if (ND_mark(w) == FALSE) dfs(w); }
 	}
-	n->u.onstack = FALSE;
+	ND_onstack(n) = FALSE;
 }
 
 void reverse_edge(edge_t* e)
