@@ -10,7 +10,7 @@
 #pragma prototyped
 
 /*
- * position(g): set n->u.coord (x and y) for all nodes n of g, using g->u.rank.
+ * position(g): set ND_coord(n) (x and y) for all nodes n of g, using GD_rank(g).
  * (the graph may be modified by merging certain edges with a common endpoint.)
  * the coordinates are computed by constructing and ranking an auxiliary graph.
  * then leaf nodes are inserted in the fast graph.  cluster boundary nodes are
@@ -25,14 +25,14 @@
 
 void dot_position(graph_t* g)
 {
-	if (g->u.nlist == NULL) return;		/* ignore empty graph */
+	if (GD_nlist(g) == NULL) return;		/* ignore empty graph */
 	mark_lowclusters(g);				/* we could remove from splines.c now */
 	set_ycoords(g);
 	if (Concentrate) dot_concentrate(g);
 	expand_leaves(g);
 	if (flat_edges(g)) set_ycoords(g);
 	create_aux_edges(g);
-	rank(g,FALSE,nsiter2(g));
+	rank(g,2,nsiter2(g));				/* LR balance == 2*/
 	set_xcoords(g);
 	remove_aux_edges(g);
 	set_aspect(g);
@@ -55,7 +55,7 @@ static int go(node_t* u, node_t* v)
 	edge_t	*e;
 
 	if (u == v) return TRUE;
-	for (i = 0; (e = u->u.out.list[i]); i++) {
+	for (i = 0; (e = ND_out(u).list[i]); i++) {
 		if (go(e->head,v))
 			return TRUE;
 	}
@@ -76,8 +76,8 @@ make_aux_edge(node_t *u, node_t *v, int len, int wt)
 	e = NEW(edge_t);
 	e->tail = u;
 	e->head = v;
-	e->u.minlen = len;
-	e->u.weight = wt;
+	ED_minlen(e) = len;
+	ED_weight(e) = wt;
 	fast_edge(e);
 	return e;
 }
@@ -89,14 +89,14 @@ void allocate_aux_edges(graph_t* g)
 	node_t	*n;
 
 	/* allocate space for aux edge lists */
-	for (n = g->u.nlist; n; n = n->u.next) {
-		n->u.save_in = n->u.in;
-		n->u.save_out = n->u.out;
-		for (i = 0; n->u.out.list[i]; i++);
-		for (j = 0; n->u.in.list[j]; j++);
+	for (n = GD_nlist(g); n; n = ND_next(n)) {
+		ND_save_in(n) = ND_in(n);
+		ND_save_out(n) = ND_out(n);
+		for (i = 0; ND_out(n).list[i]; i++);
+		for (j = 0; ND_in(n).list[j]; j++);
 		n_in = i + j;
-		alloc_elist(n_in + 3,n->u.in);
-		alloc_elist(3,n->u.out);
+		alloc_elist(n_in + 3,ND_in(n));
+		alloc_elist(3,ND_out(n));
 	}
 }
 
@@ -108,72 +108,72 @@ void make_LR_constraints(graph_t* g)
 	int		width;
 	edge_t	*e, *e0, *e1, *f, *ff;
 	node_t	*u,*v, *t0, *h0;
-	rank_t	*rank = g->u.rank;
+	rank_t	*rank = GD_rank(g);
 
 	/* make edges to constrain left-to-right ordering */
-	for (i = g->u.minrank; i <= g->u.maxrank; i++) {
+	for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
 		int		last;
 		last = rank[i].v[0]->u.rank = 0;
 		for (j = 0; j < rank[i].n; j++) {
 			u = rank[i].v[j];
-			u->u.mval = u->u.rw;	/* keep it somewhere safe */
-			if (u->u.other.size > 0) {	/* compute self size */
+			ND_mval(u) = ND_rw(u);	/* keep it somewhere safe */
+			if (ND_other(u).size > 0) {	/* compute self size */
 				sw = 0;
-				for (k = 0; (e = u->u.other.list[k]); k++) {
+				for (k = 0; (e = ND_other(u).list[k]); k++) {
 					if (e->tail == e->head) {
 						sw += SELF_EDGE_SIZE;
-						if (e->u.label) {
+						if (ED_label(e)) {
 							double	label_width;
-							label_width = g->u.left_to_right? e->u.label->dimen.y : e->u.label->dimen.x;
+							label_width = GD_left_to_right(g)? ED_label(e)->dimen.y : ED_label(e)->dimen.x;
 							sw += POINTS(label_width);
 						}
 					}
 				}
-				u->u.rw += sw;			/* increment to include self edges */
+				ND_rw(u) += sw;			/* increment to include self edges */
 			}
 			v = rank[i].v[j+1];
 			if (v) {
-				width = u->u.rw + v->u.lw + g->u.nodesep;
+				width = ND_rw(u) + ND_lw(v) + GD_nodesep(g);
 				e0 = make_aux_edge(u,v,width,0);
-				last = (v->u.rank = last + width);
+				last = (ND_rank(v) = last + width);
 			}
 
 				/* position flat edge endpoints */
-			for (k = 0; k < u->u.flat_out.size; k++) {
-				e = u->u.flat_out.list[k];
+			for (k = 0; k < ND_flat_out(u).size; k++) {
+				e = ND_flat_out(u).list[k];
 				v = e->head;
-				if (e->tail->u.order < e->head->u.order)
+				if (ND_order(e->tail) < ND_order(e->head))
 					{ t0 = e->tail; h0 = e->head; }
 				else
 					{ t0 = e->head; h0 = e->tail; }
 
 				/* case 1: flat edge with a label */
-				if ((f = e->u.to_virt)) {
-					while (f->u.to_virt) f = f->u.to_virt;
-					e0 = f->tail->u.save_out.list[0];
-					e1 = f->tail->u.save_out.list[1];
-					if (e0->head->u.order > e1->head->u.order)
+				if ((f = ED_to_virt(e))) {
+					while (ED_to_virt(f)) f = ED_to_virt(f);
+					e0 = ND_save_out(f->tail).list[0];
+					e1 = ND_save_out(f->tail).list[1];
+					if (ND_order(e0->head) > ND_order(e1->head))
 						{ ff = e0; e0 = e1; e1 = ff; }
-					m0 = (e->u.minlen *g->u.nodesep)/2;
-					m1 = m0 + e0->head->u.rw + e0->tail->u.lw;
+					m0 = (ED_minlen(e) *GD_nodesep(g))/2;
+					m1 = m0 + ND_rw(e0->head) + ND_lw(e0->tail);
 					/* these guards are needed because the flat edges
 						work very poorly with cluster layout */
 					if (canreach(e0->tail,e0->head) == FALSE)
-						make_aux_edge(e0->head,e0->tail,m1,e->u.weight);
-					m1 = m0 + e1->tail->u.rw + e1->head->u.lw;
+						make_aux_edge(e0->head,e0->tail,m1,ED_weight(e));
+					m1 = m0 + ND_rw(e1->tail) + ND_lw(e1->head);
 					if (canreach(e1->head,e1->tail) == FALSE)
-						make_aux_edge(e1->tail,e1->head,m1,e->u.weight);
+						make_aux_edge(e1->tail,e1->head,m1,ED_weight(e));
 					continue;
 				}
 
-				m0 = e->u.minlen *g->u.nodesep + t0->u.rw + h0->u.lw;
+				m0 = ED_minlen(e) *GD_nodesep(g) + ND_rw(t0) + ND_lw(h0);
 
 				if ((e0 = find_fast_edge(t0,h0)))
 					/* case 2: flat edge between neighbors */
-					e0->u.minlen = MAX(e0->u.minlen,m0);
+					ED_minlen(e0) = MAX(ED_minlen(e0),m0);
 				else
 					/* case 3: flat edge between non-neighbors */
-					make_aux_edge(t0,h0,m0,e->u.weight);
+					make_aux_edge(t0,h0,m0,ED_weight(e));
 			}
 		}
 	}
@@ -186,16 +186,25 @@ void make_edge_pairs(graph_t* g)
 	node_t		*n,*sn;
 	edge_t		*e;
 
-	for (n = g->u.nlist; n; n = n->u.next) {
-		if (n->u.save_out.list) for (i = 0; (e = n->u.save_out.list[i]); i++) {
+	for (n = GD_nlist(g); n; n = ND_next(n)) {
+		if (ND_save_out(n).list) for (i = 0; (e = ND_save_out(n).list[i]); i++) {
 			sn = virtual_node(g);
-			sn->u.node_type = SLACKNODE;
-			m0 = (e->u.head_port.p.x - e->u.tail_port.p.x);
+			ND_node_type(sn) = SLACKNODE;
+			m0 = (ED_head_port(e).p.x - ED_tail_port(e).p.x);
 			if (m0 > 0) m1 = 0;
 			else {m1 = -m0; m0 = 0;}
-			make_aux_edge(sn,e->tail,m0+1,e->u.weight);
-			make_aux_edge(sn,e->head,m1+1,e->u.weight);
-			sn->u.rank = MIN(e->tail->u.rank - m0 -1, e->head->u.rank - m1 - 1);
+#ifdef NOTDEF	
+/* was trying to improve LR balance */
+if ((ND_save_out(n).size % 2 == 0) && (i == ND_save_out(n).size / 2 - 1)) {
+node_t *u = ND_save_out(n).list[i]->head;
+node_t *v = ND_save_out(n).list[i+1]->head;
+int width = ND_rw(u) + ND_lw(v) + GD_nodesep(g);
+m0 = width / 2 - 1;
+}
+#endif
+			make_aux_edge(sn,e->tail,m0+1,ED_weight(e));
+			make_aux_edge(sn,e->head,m1+1,ED_weight(e));
+			ND_rank(sn) = MIN(ND_rank(e->tail) - m0 -1, ND_rank(e->head) - m1 - 1);
 		}
 	}
 }
@@ -207,7 +216,7 @@ void make_edge_pairs(graph_t* g)
  */
 void pos_clusters(graph_t* g)
 {
-	if (g->u.n_cluster > 0) {
+	if (GD_n_cluster(g) > 0) {
 		contain_clustnodes(g);
 		keepout_othernodes(g);
 		contain_subclust(g);
@@ -221,18 +230,18 @@ void contain_clustnodes(graph_t* g)
 
 	if (g != g->root) {
 		contain_nodes(g);
-		make_aux_edge(g->u.ln, g->u.rn, 1, 128);	/* clust compaction edge */
+		make_aux_edge(GD_ln(g), GD_rn(g), 1, 128);	/* clust compaction edge */
 	}
-	for (c = 1; c <= g->u.n_cluster; c++)
-		contain_clustnodes(g->u.clust[c]);
+	for (c = 1; c <= GD_n_cluster(g); c++)
+		contain_clustnodes(GD_clust(g)[c]);
 }
 
 int vnode_not_related_to(graph_t* g, node_t* v)
 {
 	edge_t	*e;
 
-	if (v->u.node_type != VIRTUAL) return FALSE;
-	for (e = v->u.save_out.list[0]; e->u.to_orig; e = e->u.to_orig);
+	if (ND_node_type(v) != VIRTUAL) return FALSE;
+	for (e = ND_save_out(v).list[0]; ED_to_orig(e); e = ED_to_orig(e));
 	if (agcontains(g,e->tail)) return FALSE;
 	if (agcontains(g,e->head)) return FALSE;
 	return TRUE;
@@ -243,29 +252,29 @@ void keepout_othernodes(graph_t* g)
 	int		i,c,r;
 	node_t	*u,*v;
 
-	for (r = g->u.minrank; r <= g->u.maxrank; r++) {
-		if (g->u.rank[r].n == 0) continue;
-		v = g->u.rank[r].v[0];
+	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+		if (GD_rank(g)[r].n == 0) continue;
+		v = GD_rank(g)[r].v[0];
 		if (v == NULL) continue;
-		for (i = v->u.order - 1; i >= 0; i--) {
-			u = g->root->u.rank[r].v[i];
+		for (i = ND_order(v) - 1; i >= 0; i--) {
+			u = GD_rank(g->root)[r].v[i];
 				/* can't use "is_a_vnode_of" because elists are swapped */
-			if ((u->u.node_type == NORMAL) || vnode_not_related_to(g,u)) {
-				make_aux_edge(u,g->u.ln,CL_OFFSET+u->u.rw+g->u.border[LEFT_IX].x,0);
+			if ((ND_node_type(u) == NORMAL) || vnode_not_related_to(g,u)) {
+				make_aux_edge(u,GD_ln(g),CL_OFFSET+ND_rw(u)+GD_border(g)[LEFT_IX].x,0);
 				break;
 			}
 		}
-		for (i = v->u.order + g->u.rank[r].n; i < g->root->u.rank[r].n; i++) {
-			u = g->root->u.rank[r].v[i];
-			if ((u->u.node_type == NORMAL) || vnode_not_related_to(g,u)) {
-				make_aux_edge(g->u.rn,u,CL_OFFSET+u->u.lw+g->u.border[RIGHT_IX].x,0);
+		for (i = ND_order(v) + GD_rank(g)[r].n; i < GD_rank(g->root)[r].n; i++) {
+			u = ND_rank(g->root)[r].v[i];
+			if ((ND_node_type(u) == NORMAL) || vnode_not_related_to(g,u)) {
+				make_aux_edge(GD_rn(g),u,CL_OFFSET+ND_lw(u)+GD_border(g)[RIGHT_IX].x,0);
 				break;
 			}
 		}
 	}
 	
-	for (c = 1; c <= g->u.n_cluster; c++)
-		keepout_othernodes(g->u.clust[c]);
+	for (c = 1; c <= GD_n_cluster(g); c++)
+		keepout_othernodes(GD_clust(g)[c]);
 }
 
 void contain_subclust(graph_t* g)
@@ -274,11 +283,11 @@ void contain_subclust(graph_t* g)
 	graph_t	*subg;
 
 	make_lrvn(g);
-	for (c = 1; c <= g->u.n_cluster; c++) {
-		subg = g->u.clust[c];
+	for (c = 1; c <= GD_n_cluster(g); c++) {
+		subg = GD_clust(g)[c];
 		make_lrvn(subg);
-		make_aux_edge(g->u.ln, subg->u.ln, CL_OFFSET + subg->u.border[LEFT_IX].x, 0);
-		make_aux_edge(subg->u.rn, g->u.rn, CL_OFFSET + subg->u.border[RIGHT_IX].x, 0);
+		make_aux_edge(GD_ln(g), GD_ln(subg), CL_OFFSET + GD_border(subg)[LEFT_IX].x, 0);
+		make_aux_edge(GD_rn(subg), GD_rn(g), CL_OFFSET + GD_border(subg)[RIGHT_IX].x, 0);
 		contain_subclust(subg);
 	}
 }
@@ -289,22 +298,22 @@ void separate_subclust(graph_t* g)
 	graph_t		*low,*high;
 	graph_t		*left,*right;
 
-	for (i = 1; i <= g->u.n_cluster; i++) make_lrvn(g->u.clust[i]);
-	for (i = 1; i <= g->u.n_cluster; i++) {
-		for (j = i + 1; j <= g->u.n_cluster; j++) {
-			low = g->u.clust[i]; high = g->u.clust[j];
-			if (low->u.minrank > high->u.minrank)
+	for (i = 1; i <= GD_n_cluster(g); i++) make_lrvn(GD_clust(g)[i]);
+	for (i = 1; i <= GD_n_cluster(g); i++) {
+		for (j = i + 1; j <= GD_n_cluster(g); j++) {
+			low = GD_clust(g)[i]; high = GD_clust(g)[j];
+			if (GD_minrank(low) > GD_minrank(high))
 				{ graph_t	*temp = low; low = high; high= temp; }
-			if (low->u.maxrank < high->u.minrank) continue;
-			if ((low->u.rank[high->u.minrank].v[0]->u.order)
-				< (high->u.rank[high->u.minrank].v[0]->u.order))
+			if (GD_maxrank(low) < GD_minrank(high)) continue;
+			if (ND_order(GD_rank(low)[GD_minrank(high)].v[0])
+				< ND_order(GD_rank(high)[GD_minrank(high)].v[0]))
 					{ left = low; right = high; }
 				else
 					{ left = high; right = low; }
-			make_aux_edge(left->u.rn, right->u.ln,
-				CL_OFFSET+left->u.border[RIGHT_IX].x+right->u.border[LEFT_IX].x,0);
+			make_aux_edge(ND_rn(left), ND_ln(right),
+				CL_OFFSET+ND_border(left)[RIGHT_IX].x+ND_border(right)[LEFT_IX].x,0);
 		}
-		separate_subclust(g->u.clust[i]);
+		separate_subclust(GD_clust(g)[i]);
 	}
 }
 
@@ -323,38 +332,38 @@ void remove_aux_edges(graph_t* g)
 	node_t	*n,*nnext,*nprev;
 	edge_t	*e;
 
-	for (n = g->u.nlist; n; n = n->u.next) {
-		for (i = 0; (e = n->u.out.list[i]); i++) free(e);
-		free_list(n->u.out);
-		free_list(n->u.in);
-		n->u.out = n->u.save_out;
-		n->u.in = n->u.save_in;
+	for (n = GD_nlist(g); n; n = ND_next(n)) {
+		for (i = 0; (e = ND_out(n).list[i]); i++) free(e);
+		free_list(ND_out(n));
+		free_list(ND_in(n));
+		ND_out(n) = ND_save_out(n);
+		ND_in(n) = ND_save_in(n);
 	}
 	/* cannot be merged with previous loop */
 	nprev = NULL;
-	for (n = g->u.nlist; n; n = nnext) {
-		nnext = n->u.next;
-		if (n->u.node_type == SLACKNODE) {
-			if (nprev) nprev->u.next = nnext;
-			else g->u.nlist = nnext;
+	for (n = GD_nlist(g); n; n = nnext) {
+		nnext = ND_next(n);
+		if (ND_node_type(n) == SLACKNODE) {
+			if (nprev) ND_next(nprev) = nnext;
+			else GD_nlist(g) = nnext;
 			free(n);
 		}
 		else nprev = n;
 	}
-	g->u.nlist->u.prev = NULL;
+	GD_nlist(g)->u.prev = NULL;
 }
 
 void set_xcoords(graph_t* g)
 {
 	int		i,j;
 	node_t	*v;
-	rank_t	*rank = g->u.rank;
+	rank_t	*rank = GD_rank(g);
 
-	for (i = g->u.minrank;  i <= g->u.maxrank; i++) {
+	for (i = GD_minrank(g);  i <= GD_maxrank(g); i++) {
 		for (j = 0; j < rank[i].n; j++) {
 			v = rank[i].v[j];
-			v->u.coord.x = v->u.rank;
-			v->u.rank = i;
+			ND_coord(v).x = ND_rank(v);
+			ND_rank(v) = i;
 		}
 	}
 }
@@ -364,30 +373,30 @@ void set_ycoords(graph_t* g)
 {
 	int		i,r,ht2,maxht,delta,d0,d1;
 	node_t	*n;
-	rank_t	*rank = g->u.rank;
+	rank_t	*rank = GD_rank(g);
 	graph_t	*clust;
 	static void clust_ht(graph_t *g);
 
 	ht2 = maxht = 0;
 
 	/* scan ranks for tallest nodes.  */
-	for (r = g->u.minrank; r <=  g->u.maxrank; r++) {
+	for (r = GD_minrank(g); r <=  GD_maxrank(g); r++) {
 		for (i = 0; i < rank[r].n; i++) {
 			n = rank[r].v[i];
 
 			/* assumes symmetry, ht1 = ht2 */
-			ht2 = (n->u.ht + 1)/2;
+			ht2 = (ND_ht(n) + 1)/2;
 
 			/* update global rank ht */
 			if (rank[r].pht2 < ht2) rank[r].pht2 = rank[r].ht2 = ht2;
 			if (rank[r].pht1 < ht2) rank[r].pht1 = rank[r].ht1 = ht2;
 
 			/* update nearest enclosing cluster rank ht */
-			if ((clust = n->u.clust)) {
-				if (n->u.rank == clust->u.minrank)
-					clust->u.ht2 = MAX(clust->u.ht2,ht2 + CL_OFFSET);
-				if (n->u.rank == clust->u.maxrank)
-					clust->u.ht1 = MAX(clust->u.ht1,ht2 + CL_OFFSET);
+			if ((clust = ND_clust(n))) {
+				if (ND_rank(n) == GD_minrank(clust))
+					GD_ht2(clust) = MAX(GD_ht2(clust),ht2 + CL_OFFSET);
+				if (ND_rank(n) == GD_maxrank(clust))
+					GD_ht1(clust) = MAX(GD_ht1(clust),ht2 + CL_OFFSET);
 			}
 		}
 	}
@@ -397,10 +406,10 @@ void set_ycoords(graph_t* g)
 
 	/* make the initial assignment of ycoords to leftmost nodes by ranks */
 	maxht = 0;
-	r = g->u.maxrank;
+	r = GD_maxrank(g);
 	rank[r].v[0]->u.coord.y = rank[r].ht1;
-	while (--r >= g->u.minrank) {
-		d0 = rank[r+1].pht2 + rank[r].pht1 + g->u.ranksep; /* prim node sep */
+	while (--r >= GD_minrank(g)) {
+		d0 = rank[r+1].pht2 + rank[r].pht1 + GD_ranksep(g); /* prim node sep */
 		d1 = rank[r+1].ht2 + rank[r].ht1 + CL_OFFSET;	/* cluster sep */
 		delta = MAX(d0,d1);
 		if (rank[r].n > 0) /* this may reflect some problem */
@@ -412,14 +421,14 @@ void set_ycoords(graph_t* g)
 	}
 
 	/* re-assign if ranks are equally spaced */
-	if (g->u.exact_ranksep)
-		for (r = g->u.maxrank - 1; r <=  g->u.minrank; r--)
+	if (GD_exact_ranksep(g))
+		for (r = GD_maxrank(g) - 1; r >=  GD_minrank(g); r--)
 			if (rank[r].n > 0) /* this may reflect the same problem :-() */
 				rank[r].v[0]->u.coord.y = rank[r + 1].v[0]->u.coord.y + maxht;
 
 	/* copy ycoord assignment from leftmost nodes to others */
-	for (n = g->u.nlist; n; n = n->u.next)
-		n->u.coord.y = rank[n->u.rank].v[0]->u.coord.y;
+	for (n = GD_nlist(g); n; n = ND_next(n))
+		ND_coord(n).y = rank[ND_rank(n)].v[0]->u.coord.y;
 }
 
 void compute_bb(graph_t* g, graph_t* root)
@@ -430,42 +439,42 @@ void compute_bb(graph_t* g, graph_t* root)
 	
 	LL.x = LL.y = MAXINT;
 	UR.x = UR.y = -MAXINT;
-	for (r = g->u.minrank; r <= g->u.maxrank; r++) {
-		if (g->u.rank[r].n == 0) continue;
-		if ((v = g->u.rank[r].v[0]) == NULL) continue;
-		x = v->u.coord.x - v->u.lw;
+	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+		if (GD_rank(g)[r].n == 0) continue;
+		if ((v = GD_rank(g)[r].v[0]) == NULL) continue;
+		x = ND_coord(v).x - ND_lw(v);
 		if (g != g->root) x-= CL_OFFSET;
 		LL.x = MIN(LL.x,x);
-		v = g->u.rank[r].v[g->u.rank[r].n-1];
-		x = v->u.coord.x + v->u.rw;
+		v = GD_rank(g)[r].v[GD_rank(g)[r].n-1];
+		x = ND_coord(v).x + ND_rw(v);
 		if (g != g->root) x+= CL_OFFSET;
 		UR.x = MAX(UR.x,x);
 	}
 	offset.x = offset.y = CL_OFFSET;
-	for (c = 1; c <= g->u.n_cluster; c++) {
-		p = sub_points(g->u.clust[c]->u.bb.LL,offset);
+	for (c = 1; c <= GD_n_cluster(g); c++) {
+		p = sub_points(GD_clust(g)[c]->u.bb.LL,offset);
 		if (LL.x > p.x) LL.x = p.x;
-		p = add_points(g->u.clust[c]->u.bb.UR,offset);
+		p = add_points(GD_clust(g)[c]->u.bb.UR,offset);
 		if (UR.x < p.x) UR.x = p.x;
 	}
-	LL.y = root->u.rank[g->u.maxrank].v[0]->u.coord.y - g->u.ht1;
-	UR.y = root->u.rank[g->u.minrank].v[0]->u.coord.y + g->u.ht2;
-	g->u.bb.LL = LL; g->u.bb.UR = UR;
+	LL.y = ND_rank(root)[GD_maxrank(g)].v[0]->u.coord.y - GD_ht1(g);
+	UR.y = ND_rank(root)[GD_minrank(g)].v[0]->u.coord.y + GD_ht2(g);
+	GD_bb(g).LL = LL; GD_bb(g).UR = UR;
 }
 
 void update_bb(graph_t* g, point pt)
 {
-	if (pt.x > g->u.bb.UR.x)  g->u.bb.UR.x = pt.x;
-	if (pt.y > g->u.bb.UR.y)  g->u.bb.UR.y = pt.y;
-	if (pt.x < g->u.bb.LL.x)  g->u.bb.LL.x = pt.x;
-	if (pt.y < g->u.bb.LL.y)  g->u.bb.LL.y = pt.y;
+	if (pt.x > GD_bb(g).UR.x)  GD_bb(g).UR.x = pt.x;
+	if (pt.y > GD_bb(g).UR.y)  GD_bb(g).UR.y = pt.y;
+	if (pt.x < GD_bb(g).LL.x)  GD_bb(g).LL.x = pt.x;
+	if (pt.y < GD_bb(g).LL.y)  GD_bb(g).LL.y = pt.y;
 }
 
 void rec_bb(graph_t *g, graph_t *root)
 {
 	int		c;
-	for (c = 1; c <= g->u.n_cluster; c++)
-		rec_bb(g->u.clust[c],root);
+	for (c = 1; c <= GD_n_cluster(g); c++)
+		rec_bb(GD_clust(g)[c],root);
 	compute_bb(g,root);
 }
 
@@ -477,20 +486,20 @@ void set_aspect(graph_t* g)
 	boolean	scale_it,filled;
 
 	rec_bb(g,g);
-	if ((g->u.maxrank > 0) && (str = agget(g,"ratio"))) {
-		g->u.bb.UR.x -= g->u.bb.LL.x;
-		g->u.bb.UR.y -= g->u.bb.LL.y;	/* normalize */
-		if (g->u.left_to_right)
-			{int t = g->u.bb.UR.x; g->u.bb.UR.x = g->u.bb.UR.y; g->u.bb.UR.y = t;}
+	if ((GD_maxrank(g) > 0) && (str = agget(g,"ratio"))) {
+		GD_bb(g).UR.x -= GD_bb(g).LL.x;
+		GD_bb(g).UR.y -= GD_bb(g).LL.y;	/* normalize */
+		if (GD_left_to_right(g))
+			{int t = GD_bb(g).UR.x; GD_bb(g).UR.x = GD_bb(g).UR.y; GD_bb(g).UR.y = t;}
 		scale_it = TRUE;
 		if (streq(str,"auto")) filled = idealsize(g,.5);
 		else filled = (streq(str,"fill"));
 		if (filled) {
 			/* fill is weird because both X and Y can stretch */
-			if (g->u.drawing->size.x <= 0) scale_it = FALSE;
+			if (GD_drawing(g)->size.x <= 0) scale_it = FALSE;
 			else {
-				xf = (double)g->u.drawing->size.x / (double)g->u.bb.UR.x;
-				yf = (double)g->u.drawing->size.y / (double)g->u.bb.UR.y;
+				xf = (double)GD_drawing(g)->size.x / (double)GD_bb(g).UR.x;
+				yf = (double)GD_drawing(g)->size.y / (double)GD_bb(g).UR.y;
 				if ((xf < 1.0) || (yf < 1.0)) {
 					if (xf < yf) {yf = yf / xf; xf = 1.0;}
 					else {xf = xf / yf; yf = 1.0;}
@@ -501,16 +510,16 @@ void set_aspect(graph_t* g)
 			desired = atof(str);
 			if (desired == 0.0) scale_it = FALSE;
 			else {
-				actual = ((float)g->u.bb.UR.y)/((float)g->u.bb.UR.x);
+				actual = ((double)GD_bb(g).UR.y)/((double)GD_bb(g).UR.x);
 				if (actual < desired) {yf = desired/actual; xf = 1.0;}
 				else {xf = actual/desired; yf = 1.0;}
 			}
 		}
 		if (scale_it) {
-			if (g->u.left_to_right) {float t = xf; xf = yf; yf = t;}
-			for (n = g->u.nlist; n; n = n->u.next) {
-				n->u.coord.x = n->u.coord.x * xf;
-				n->u.coord.y = n->u.coord.y * yf;
+			if (GD_left_to_right(g)) {double t = xf; xf = yf; yf = t;}
+			for (n = GD_nlist(g); n; n = ND_next(n)) {
+				ND_coord(n).x = ND_coord(n).x * xf;
+				ND_coord(n).y = ND_coord(n).y * yf;
 			}
 		}
 	}
@@ -520,10 +529,10 @@ void set_aspect(graph_t* g)
 point
 resize_leaf(node_t* leaf, point lbound)
 {
-	dot_nodesize(leaf,leaf->graph->u.left_to_right);
-	leaf->u.coord.y = lbound.y;
-	leaf->u.coord.x = lbound.x + leaf->u.lw;
-	lbound.x = lbound.x + leaf->u.lw + leaf->u.rw + leaf->graph->u.nodesep;
+	dot_nodesize(leaf,GD_left_to_right(leaf->graph));
+	ND_coord(leaf).y = lbound.y;
+	ND_coord(leaf).x = lbound.x + ND_lw(leaf);
+	lbound.x = lbound.x + ND_lw(leaf) + ND_rw(leaf) + GD_nodesep(leaf->graph);
 	return lbound;
 }
 
@@ -535,9 +544,9 @@ place_leaf(node_t* leaf, point lbound, int order)
 
 	leader = UF_find(leaf);
 	if (leaf != leader) fast_nodeapp(leader,leaf);
-	leaf->u.order = order;
-	leaf->u.rank = leader->u.rank;
-	g->u.rank[leaf->u.rank].v[leaf->u.order] = leaf;
+	ND_order(leaf) = order;
+	ND_rank(leaf) = ND_rank(leader);
+	GD_rank(g)[ND_rank(leaf)].v[ND_order(leaf)] = leaf;
 	return resize_leaf(leaf,lbound);
 }
 
@@ -547,22 +556,22 @@ void make_leafslots(graph_t* g)
 	int		i,j,r;
 	node_t	*v;
 
-	for (r = g->u.minrank; r <= g->u.maxrank; r++) {
+	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 		j = 0;
-		for (i = 0; i < g->u.rank[r].n; i++) {
-			v = g->u.rank[r].v[i];
-			v->u.order = j;
-			if (v->u.ranktype == LEAFSET) j = j + v->u.UF_size;
+		for (i = 0; i < GD_rank(g)[r].n; i++) {
+			v = GD_rank(g)[r].v[i];
+			ND_order(v) = j;
+			if (ND_ranktype(v) == LEAFSET) j = j + ND_UF_size(v);
 			else j++;
 		}
-		if (j <= g->u.rank[r].n) continue;
-		g->u.rank[r].v = ALLOC(j+1,g->u.rank[r].v,node_t*);
-		for (i = g->u.rank[r].n - 1; i >= 0; i--) {
-			v = g->u.rank[r].v[i];
-			g->u.rank[r].v[v->u.order] = v;
+		if (j <= GD_rank(g)[r].n) continue;
+		GD_rank(g)[r].v = ALLOC(j+1,GD_rank(g)[r].v,node_t*);
+		for (i = GD_rank(g)[r].n - 1; i >= 0; i--) {
+			v = GD_rank(g)[r].v[i];
+			GD_rank(g)[r].v[ND_order(v)] = v;
 		}
-		g->u.rank[r].n = j;
-		g->u.rank[r].v[j] = NULL;
+		GD_rank(g)[r].n = j;
+		GD_rank(g)[r].v[j] = NULL;
 	}
 }
 
@@ -573,29 +582,29 @@ void do_leaves(graph_t* g, node_t* leader)
 	node_t	*n;
 	edge_t	*e;
 
-	if (leader->u.UF_size <= 1) return;
-	lbound.x = leader->u.coord.x - leader->u.lw;
-	lbound.y = leader->u.coord.y;
+	if (ND_UF_size(leader) <= 1) return;
+	lbound.x = ND_coord(leader).x - ND_lw(leader);
+	lbound.y = ND_coord(leader).y;
 	lbound = resize_leaf(leader,lbound);
-	if (leader->u.out.size > 0) {		/* in-edge leaves */
-		n = leader->u.out.list[0]->head;
-		j = leader->u.order + 1;
+	if (ND_out(leader).size > 0) {		/* in-edge leaves */
+		n = ND_out(leader).list[0]->head;
+		j = ND_order(leader) + 1;
 		for (e = agfstin(g,n); e; e = agnxtin(g,e)) {
 			if ((e->tail != leader) && (UF_find(e->tail) == leader)) {
 				lbound = place_leaf(e->tail,lbound,j++);
 				unmerge_oneway(e);
-				elist_append(e,e->head->u.in);
+				elist_append(e,ND_in(e->head));
 			}
 		}
 	}
 	else {							/* out edge leaves */
-		n = leader->u.in.list[0]->tail;
-		j = leader->u.order + 1;
+		n = ND_in(leader).list[0]->tail;
+		j = ND_order(leader) + 1;
 		for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
 			if ((e->head != leader) && (UF_find(e->head) == leader)) {
 				lbound = place_leaf(e->head,lbound,j++);
 				unmerge_oneway(e);
-				elist_append(e,e->tail->u.out);
+				elist_append(e,ND_out(e->tail));
 			}
 		}
 	}
@@ -604,13 +613,13 @@ void do_leaves(graph_t* g, node_t* leader)
 int ports_eq(edge_t *e,edge_t *f)
 {
 	return (
-		   (e->u.head_port.defined == f->u.head_port.defined)
-		&& ( ((e->u.head_port.p.x == f->u.head_port.p.x) &&
-			 (e->u.head_port.p.y == f->u.head_port.p.y))
-			|| (e->u.head_port.defined == FALSE))
-		&& ( ((e->u.tail_port.p.x == f->u.tail_port.p.x) &&
-			 (e->u.tail_port.p.y == f->u.tail_port.p.y))
-			|| (e->u.tail_port.defined == FALSE))
+		   (ED_head_port(e).defined == ED_head_port(f).defined)
+		&& ( ((ED_head_port(e).p.x == ED_head_port(f).p.x) &&
+			 (ED_head_port(e).p.y == ED_head_port(f).p.y))
+			|| (ED_head_port(e).defined == FALSE))
+		&& ( ((ED_tail_port(e).p.x == ED_tail_port(f).p.x) &&
+			 (ED_tail_port(e).p.y == ED_tail_port(f).p.y))
+			|| (ED_tail_port(e).defined == FALSE))
 	);
 }
 
@@ -621,14 +630,14 @@ void expand_leaves(graph_t* g)
 	edge_t	*e,*f;
 
 	make_leafslots(g);
-	for (n = g->u.nlist; n; n = n->u.next) {
-		if (n->u.inleaf) do_leaves(g,n->u.inleaf);
-		if (n->u.outleaf) do_leaves(g,n->u.outleaf);
-		if (n->u.other.list) for (i = 0; (e = n->u.other.list[i]); i++) {
-			if ((d = e->head->u.rank - e->head->u.rank) == 0) continue;
-			f = e->u.to_orig;
+	for (n = GD_nlist(g); n; n = ND_next(n)) {
+		if (ND_inleaf(n)) do_leaves(g,ND_inleaf(n));
+		if (ND_outleaf(n)) do_leaves(g,ND_outleaf(n));
+		if (ND_other(n).list) for (i = 0; (e = ND_other(n).list[i]); i++) {
+			if ((d = ND_rank(e->head) - ND_rank(e->head)) == 0) continue;
+			f = ED_to_orig(e);
 			if (ports_eq(e,f) == FALSE) {
-				zapinlist(&(n->u.other),e);
+				zapinlist(&(ND_other(n)),e);
 				if (d == 1) fast_edge(e);
 				/*else unitize(e); ### */
 				i--;
@@ -643,23 +652,23 @@ void compress_graph(graph_t* g)
 	double		x;
 	point		p;
 
-	p = g->u.drawing->size;
+	p = GD_drawing(g)->size;
 	if ((str = agget(g,"ratio")) == NULL) return;
 	if (strcmp(str,"compress")) return;
 	if (p.x * p.y <= 1) return;
 	contain_nodes(g);
-	if (g->u.left_to_right == FALSE) x = p.x; else x = p.y;
-	make_aux_edge(g->u.ln,g->u.rn,(int)x,1000);
+	if (GD_left_to_right(g) == FALSE) x = p.x; else x = p.y;
+	make_aux_edge(GD_ln(g),GD_rn(g),(int)x,1000);
 }
 
 void make_lrvn(graph_t* g)
 {
 	node_t		*ln,*rn;
 
-	if (g->u.ln) return;
-	ln = virtual_node(g->root); ln->u.node_type = SLACKNODE;
-	rn = virtual_node(g->root); rn->u.node_type = SLACKNODE;
-	g->u.ln = ln; g->u.rn = rn;
+	if (GD_ln(g)) return;
+	ln = virtual_node(g->root); ND_node_type(ln) = SLACKNODE;
+	rn = virtual_node(g->root); ND_node_type(rn) = SLACKNODE;
+	GD_ln(g) = ln; GD_rn(g) = rn;
 }
 
 /* contain_nodes: make left and right bounding box virtual nodes,
@@ -670,17 +679,17 @@ void contain_nodes(graph_t* g)
 	int			r;
 	node_t		*ln,*rn,*v;
 
-	make_lrvn(g); ln = g->u.ln; rn = g->u.rn;
-	for (r = g->u.minrank; r <= g->u.maxrank; r++) {
-		if (g->u.rank[r].n == 0) continue;
-		v = g->u.rank[r].v[0];
+	make_lrvn(g); ln = GD_ln(g); rn = GD_rn(g);
+	for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+		if (GD_rank(g)[r].n == 0) continue;
+		v = GD_rank(g)[r].v[0];
 		if (v == NULL) {
 			fprintf(stderr,"contain_nodes clust %s rank %d missing node\n",g->name,r);
 			continue;
 		}
-		make_aux_edge(ln,v,v->u.lw + CL_OFFSET,0);
-		v = g->u.rank[r].v[g->u.rank[r].n - 1];
-		make_aux_edge(v,rn,v->u.rw + CL_OFFSET,0);
+		make_aux_edge(ln,v,ND_lw(v) + CL_OFFSET,0);
+		v = GD_rank(g)[r].v[GD_rank(g)[r].n - 1];
+		make_aux_edge(v,rn,ND_rw(v) + CL_OFFSET,0);
 	}
 }
 
@@ -693,12 +702,12 @@ int idealsize(graph_t* g, double minallowed)
 	point		b,relpage,margin;
 
 	/* try for one page */
-	relpage = g->u.drawing->page;
+	relpage = GD_drawing(g)->page;
 	if (relpage.x == 0) return FALSE;				/* no page was specified */
-	margin = g->u.drawing->margin;
+	margin = GD_drawing(g)->margin;
 	relpage = sub_points(relpage,margin);
 	relpage = sub_points(relpage,margin);
-	b.x = g->u.bb.UR.x; b.y = g->u.bb.UR.y;
+	b.x = GD_bb(g).UR.x; b.y = GD_bb(g).UR.y;
 	xf = (double)relpage.x / b.x;
 	yf = (double)relpage.y / b.y;
 	if ((xf >= 1.0) && (yf >= 1.0)) return FALSE;	/* fits on one page */
@@ -710,13 +719,13 @@ int idealsize(graph_t* g, double minallowed)
 	xf = ((R * relpage.x) / b.x);
 	R = ceil((yf * b.y)/relpage.y);
 	yf = ((R * relpage.y) / b.y);
-	g->u.drawing->size.x = b.x * xf;
-	g->u.drawing->size.y = b.y * yf;
+	GD_drawing(g)->size.x = b.x * xf;
+	GD_drawing(g)->size.y = b.y * yf;
 	return TRUE;
 }
 
 /*
- * recursively compute cluster ht requirements.  assumes subg->u.ht1 and ht2
+ * recursively compute cluster ht requirements.  assumes GD_ht1(subg) and ht2
  * are computed from primitive nodes only.  updates ht1 and ht2 to reflect
  * cluster nesting and labels.  also maintains global rank ht1 and ht2.
  */
@@ -725,30 +734,30 @@ clust_ht(Agraph_t *g)
 {
 	int	c, ht1, ht2;
 	graph_t	*subg;
-	rank_t	*rank = g->root->u.rank;
+	rank_t	*rank = GD_rank(g->root);
 
-	ht1 = g->u.ht1;
-	ht2 = g->u.ht2;
+	ht1 = GD_ht1(g);
+	ht2 = GD_ht2(g);
 
 	/* account for sub-clusters */
-	for (c = 1; c <= g->u.n_cluster; c++) {
-		subg = g->u.clust[c];
+	for (c = 1; c <= GD_n_cluster(g); c++) {
+		subg = GD_clust(g)[c];
 		clust_ht(subg);
-		if (subg->u.maxrank == g->u.maxrank)
-			ht1 = MAX(ht1,subg->u.ht1 + CL_OFFSET);
-		if (subg->u.minrank == g->u.minrank)
-			ht2 = MAX(ht2,subg->u.ht2 + CL_OFFSET);
+		if (GD_maxrank(subg) == GD_maxrank(g))
+			ht1 = MAX(ht1,GD_ht1(subg) + CL_OFFSET);
+		if (GD_minrank(subg) == GD_minrank(g))
+			ht2 = MAX(ht2,GD_ht2(subg) + CL_OFFSET);
 	}
 
 	/* account for a possible cluster label */
-	ht1 += g->u.border[BOTTOM_IX].y;
-	ht2 += g->u.border[TOP_IX].y;
-	g->u.ht1 = ht1;
-	g->u.ht2 = ht2;
+	ht1 += GD_border(g)[BOTTOM_IX].y;
+	ht2 += GD_border(g)[TOP_IX].y;
+	GD_ht1(g) = ht1;
+	GD_ht2(g) = ht2;
 
 	/* update the global ranks */
 	if (g != g->root) {
-		rank[g->u.minrank].ht2 = MAX(rank[g->u.minrank].ht2,ht2);
-		rank[g->u.maxrank].ht1 = MAX(rank[g->u.maxrank].ht1,ht1);
+		rank[GD_minrank(g)].ht2 = MAX(rank[GD_minrank(g)].ht2,ht2);
+		rank[GD_maxrank(g)].ht1 = MAX(rank[GD_maxrank(g)].ht1,ht1);
 	}
 }
