@@ -8,6 +8,11 @@
 
 #include	"dot.h"
 #include	"pathplan.h"
+
+#ifdef DMALLOC
+#include "dmalloc.h"
+#endif
+
 	/* some of the following must be dispensible, see constants.h */
 #ifndef INT_MAX
 #define INT_MAX MAXINT
@@ -15,8 +20,10 @@
 #endif
 
 static box *bs = NULL;
+#ifdef UNUSED
 static int bn;
 static int maxbn = 0;
+#endif
 #define BINC 300
 
 static point *ps = NULL;
@@ -65,7 +72,9 @@ void routesplinesinit (void)
         fprintf (stderr, "cannot allocate bs\n");
         abort ();
     }
+#ifdef UNUSED
     maxbn = BINC;
+#endif
     if (!(ps = (point *) malloc (PINC * sizeof (point)))) {
         fprintf (stderr, "cannot allocate ps\n");
         abort ();
@@ -80,7 +89,7 @@ void routesplinesinit (void)
 void routesplinesterm (void)
 {
     free (ps), ps = NULL, maxpn = pn = 0;
-    free (bs), bs = NULL, maxbn = bn = 0;
+    free (bs), bs = NULL /*, maxbn = bn = 0 */;
     if (Verbose)
         fprintf (stderr, "routesplines: %d edges, %d boxes, %d splines %.2f sec\n",
                 nedges, nboxes, nsplines,elapsed_sec());
@@ -102,7 +111,7 @@ point *routesplines (path* pp, int* npoints)
     nboxes += pp->nbox;
 
     for (realedge = origedge = (edge_t *) pp->data;
-            realedge && realedge->u.edge_type != NORMAL; realedge = realedge->u.to_orig)
+            realedge && ED_edge_type(realedge) != NORMAL; realedge = ED_to_orig(realedge))
         ;
     if (!realedge) {
         fprintf (stderr, "in routesplines, cannot find NORMAL edge\n");
@@ -115,11 +124,11 @@ point *routesplines (path* pp, int* npoints)
 
     checkpath ();
 
-    if (realedge->head->graph->u.showboxes == 1 ||
-            realedge->tail->graph->u.showboxes == 1 ||
-            realedge->u.showboxes == 1 ||
-            realedge->head->u.showboxes == 1||
-            realedge->tail->u.showboxes == 1)
+    if (GD_showboxes(realedge->head->graph) == 1 ||
+            GD_showboxes(realedge->tail->graph) == 1 ||
+            ED_showboxes(realedge) == 1 ||
+            ND_showboxes(realedge->head) == 1||
+            ND_showboxes(realedge->tail) == 1)
         printpsboxes ();
 
     if (boxn * 8 > polypointn) {
@@ -290,11 +299,11 @@ point *routesplines (path* pp, int* npoints)
     }
     *npoints = spl.pn;
 
-    if (realedge->head->graph->u.showboxes == 2 ||
-            realedge->tail->graph->u.showboxes == 2 ||
-            realedge->u.showboxes == 2 ||
-            realedge->head->u.showboxes == 2||
-            realedge->tail->u.showboxes == 2)
+    if (GD_showboxes(realedge->head->graph) == 2 ||
+            GD_showboxes(realedge->tail->graph) == 2 ||
+            ED_showboxes(realedge) == 2 ||
+            ND_showboxes(realedge->head) == 2||
+            ND_showboxes(realedge->tail) == 2)
         printpsboxes ();
 
     return ps;
@@ -423,7 +432,7 @@ static void printpath (path* pp)
     int bi;
 
     fprintf (stderr, "edge %d from %s to %s\n", nedges, realedge->tail->name, realedge->head->name);
-    if (origedge->u.count > 1)
+    if (ED_count(origedge) > 1)
         fprintf (stderr, "    (it's part of a concentrator edge)\n");
     fprintf (stderr, "%d boxes:\n", pp->nbox);
     for (bi = 0; bi < pp->nbox; bi++)
