@@ -16,7 +16,7 @@
 #include "site.h"
 
 typedef struct Edge {
-    float      a,b,c;         /* edge on line ax + by = c */
+    double      a,b,c;         /* edge on line ax + by = c */
     Site       *ep[2];        /* endpoints (vertices) of edge; initially NULL */
     Site       *reg[2];       /* sites forming edge */
     int        edgenbr;
@@ -25,7 +25,7 @@ typedef struct Edge {
 #define le 0
 #define re 1
 
-extern float pxmin, pxmax, pymin, pymax;  /* clipping window */
+extern double pxmin, pxmax, pymin, pymax;  /* clipping window */
 extern void edgeinit(void);
 extern void endpoint(Edge*, int, Site*);
 extern void clip_line(Edge *e);
