@@ -20,6 +20,6 @@ extern Halfedge * PQextractmin();
 extern Point PQ_min();
 extern int PQempty();
 extern void PQdelete(Halfedge*);
-extern void PQinsert(Halfedge*, Site*, float);
+extern void PQinsert(Halfedge*, Site*, double);
 
 #endif
