@@ -22,7 +22,7 @@ typedef struct Halfedge {
     int                ELrefcnt;
     char               ELpm;
     Site               *vertex;
-    float              ystar;
+    double              ystar;
     struct Halfedge    *PQnext;
 } Halfedge;
 
