@@ -15,7 +15,7 @@
 #endif
 
 #ifdef HAVE_CONFIG_H
-#include "gvconfig.h"
+#include "config.h"
 #endif
 
 #include	"pathplan.h"
