@@ -21,7 +21,7 @@
 #define after(v) (((v)==((v)->poly->finish))?((v)->poly->start):((v)+1))
 #define prior(v) (((v)==((v)->poly->start))?((v)->poly->finish):((v)-1))
 
-struct position  { float x,y; };
+struct position  { double x,y; };
 
 
 struct vertex	{	
@@ -35,7 +35,7 @@ struct polygon	{ struct vertex *start,*finish;};
 struct intersection   { 
 	struct vertex *firstv,*secondv; 
 	struct polygon *firstp,*secondp; 
-	float x,y; } ;
+	double x,y; } ;
 
 struct active_edge { struct vertex *name; struct active_edge *next,*last; }; 
 struct active_edge_list { struct active_edge *first,*final ; int number ; } ;
