@@ -25,7 +25,7 @@ static int              nvertices;
 void
 siteinit()
 {
-    /* float sn; */
+    /* double sn; */
 
     freeinit(&sfl, sizeof (Site));
     nvertices = 0;
@@ -40,11 +40,11 @@ getsite ()
     return ((Site *) getfree(&sfl));
 }
 
-float 
+double 
 dist(Site *s, Site *t)
 {
-    float ans;
-    float dx,dy;
+    double ans;
+    double dx,dy;
 
     dx = s->coord.x - t->coord.x;
     dy = s->coord.y - t->coord.y;
