@@ -8,14 +8,24 @@
     it without violating AT&T's intellectual property rights.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "neato.h"
 #include "pathplan.h"
 #include "vispath.h"
+#ifndef HAVE_DRAND48
+extern double drand48(void);
+#endif
 
 #ifdef DMALLOC
 #include "dmalloc.h"
 #endif
 
+#define P2PF(p, pf) (pf.x = p.x, pf.y = p.y)
+#define PF2P(pf, p) (p.x = ROUND (pf.x), p.y = ROUND (pf.y))         
+
 extern void printvis (vconfig_t *cp);
 static boolean swap_ends_p (edge_t *);
 static void place_portlabel (edge_t *e, boolean head_p);
@@ -35,7 +45,7 @@ void neato_compute_bb(graph_t *g)
 	bb.UR = pointof(-MAXINT,-MAXINT);
 	for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 		pt = coord(n);
-		s2.x = n->u.xsize/2+1; s2.y = n->u.ysize/2+1;
+		s2.x = ND_xsize(n)/2+1; s2.y = ND_ysize(n)/2+1;
 		b.LL = sub_points(pt,s2);
 		b.UR = add_points(pt,s2);
 
@@ -44,10 +54,10 @@ void neato_compute_bb(graph_t *g)
 		bb.UR.x = MAX(bb.UR.x,b.UR.x);
 		bb.UR.y = MAX(bb.UR.y,b.UR.y);
 		for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
-			if (e->u.spl == 0) continue;
-			for (i = 0; i < e->u.spl->size; i++) {
-				for (j = 0; j < e->u.spl->list[i].size; j++) {
-					pt = e->u.spl->list[i].list[j];
+			if (ED_spl(e) == 0) continue;
+			for (i = 0; i < ED_spl(e)->size; i++) {
+				for (j = 0; j < ED_spl(e)->list[i].size; j++) {
+					pt = ED_spl(e)->list[i].list[j];
 					if (bb.LL.x > pt.x) bb.LL.x = pt.x;
 					if (bb.LL.y > pt.y) bb.LL.y = pt.y;
 					if (bb.UR.x < pt.x) bb.UR.x = pt.x;
@@ -56,7 +66,15 @@ void neato_compute_bb(graph_t *g)
 			}
 		}
 	}
-	g->u.bb = bb;
+
+    for (i = 1; i <= GD_n_cluster(g); i++) {
+      bb.LL.x = MIN(bb.LL.x,GD_clust(g)[i]->u.bb.LL.x);
+      bb.LL.y = MIN(bb.LL.y,GD_clust(g)[i]->u.bb.LL.y);
+      bb.UR.x = MAX(bb.UR.x,GD_clust(g)[i]->u.bb.UR.x);
+      bb.UR.y = MAX(bb.UR.y,GD_clust(g)[i]->u.bb.UR.y);
+    }
+
+	GD_bb(g) = bb;
 }
 
 
@@ -64,9 +82,9 @@ static bezier *new_spline (edge_t *e, int sz)
 {
 	bezier *rv;
 
-	if (e->u.spl == NULL) e->u.spl = NEW (splines);
-	e->u.spl->list = ALLOC (e->u.spl->size + 1, e->u.spl->list, bezier);
-	rv = &(e->u.spl->list[e->u.spl->size++]);
+	if (ED_spl(e) == NULL) ED_spl(e) = NEW (splines);
+	ED_spl(e)->list = ALLOC (ED_spl(e)->size + 1, ED_spl(e)->list, bezier);
+	rv = &(ED_spl(e)->list[ED_spl(e)->size++]);
 	rv->list = N_NEW (sz, point);
 	rv->size = sz;
 	rv->sflag = rv->eflag = FALSE;
@@ -122,53 +140,124 @@ numFields (char* pos)
     return cnt;
 }
 
-static point*
-user_spline (attrsym_t* symptr, edge_t* e, int* np, pointf* offset,
+static int
+user_spline (attrsym_t* symptr, edge_t* e, pointf* offset,
              int doScale, pointf* scalef)
 {
-    char*  pos;
-    int    i, n, nc;
-    point* ps = 0;
-    point* pp;
-    double x,y;
+    char*    pos;
+    int      i, n, npts, nc;
+    point*   ps = 0;
+    point*   pp;
+    double   x,y;
+    int      sflag = 0, eflag = 0;
+    point    sp, ep;
+	bezier*  newspl;
 
     if (symptr == NULL) return 0;
     pos = agxget(e,symptr->index);
     if (*pos == '\0') return 0;
-    n = numFields (pos);
-    *np = n;
-    if (n > 1) {
-      ps = ALLOC(n,0,point);
-      pp = ps;
-      while (n) {
-        i = sscanf(pos,"%lf,%lf%n",&x,&y,&nc);
-        if (i < 2) {
-          free (ps);
-          ps = 0;
-          break;
-        }
-        pos = pos+nc;
-        if (doScale) {
-          pp->x = (int)((scalef->x)*(x - offset->x));
-          pp->y = (int)((scalef->y)*(y - offset->y));
-        }
-        else {
-          pp->x = (int)(x - offset->x);
-          pp->y = (int)(y - offset->y);
-        }
-        pp++;
-        n--;
+    
+      /* check for s head */
+    i = sscanf(pos,"s,%lf,%lf%n",&x,&y,&nc);
+    if (i == 2) {
+      sflag = 1;
+      pos = pos+nc;
+      if (doScale) {
+        sp.x = (int)((scalef->x)*(x - offset->x));
+        sp.y = (int)((scalef->y)*(y - offset->y));
+      }
+      else {
+        sp.x = (int)(x - offset->x);
+        sp.y = (int)(y - offset->y);
       }
     }
-    return ps;
+
+      /* check for e head */
+    i = sscanf(pos,"e,%lf,%lf%n",&x,&y,&nc);
+    if (i == 2) {
+      eflag = 1;
+      pos = pos+nc;
+      if (doScale) {
+        ep.x = (int)((scalef->x)*(x - offset->x));
+        ep.y = (int)((scalef->y)*(y - offset->y));
+      }
+      else {
+        ep.x = (int)(x - offset->x);
+        ep.y = (int)(y - offset->y);
+      }
+    }
+
+    npts = numFields (pos);  /* count potential points */
+    n = npts;
+    if ((n < 4) || (n%3 != 1)) return 0;
+    ps = ALLOC(n,0,point);
+    pp = ps;
+    while (n) {
+      i = sscanf(pos,"%lf,%lf%n",&x,&y,&nc);
+      if (i < 2) {
+        free (ps);
+        return 0;
+      }
+      pos = pos+nc;
+      if (doScale) {
+        pp->x = (int)((scalef->x)*(x - offset->x));
+        pp->y = (int)((scalef->y)*(y - offset->y));
+      }
+      else {
+        pp->x = (int)(x - offset->x);
+        pp->y = (int)(y - offset->y);
+      }
+      pp++;
+      n--;
+    }
+
+      /* parsed successfully; create spline */
+	newspl = new_spline (e, npts);
+    if (sflag) {
+      newspl->sflag = 1;
+      newspl->sp = sp;
+    }
+    if (eflag) {
+      newspl->eflag = 1;
+      newspl->ep = ep;
+    }
+    for (i=0; i < npts; i++) {
+      newspl->list[i] = ps[i];
+    }
+    free (ps);
+    return 1;
 }
 
-void spline_edges(graph_t *g)
+/* spline_edges0:
+ * Main body for constructing edges.
+ * Assumes u.bb for has been computed for g and all clusters
+ * (not just top-level clusters), and 
+ * that GD_bb(g).LL is at the origin.
+ *
+ * This last criterion is, I believe, mainly to simplify the code
+ * in neato_set_aspect. It would be good to remove this constraint,
+ * as this would allow nodes pinned on input to have the same coordinates
+ * when output in dot or plain format.
+ *
+ * The offset parameter tells how the original graph was translated to
+ * get it to the origin. It is used here to translate user-supplied
+ * spline control points.
+ *
+ * As a side-effect, this function sets the u.coord attribute of nodes
+ * from the u.pos attribute. This is needed for output. In addition, 
+ * it guarantees that all bounding
+ * boxes are current; in particular, the bounding box of g reflects the
+ * addition of edges. NOTE: intra-cluster edges are not constrained to
+ * remain in the cluster's bounding box and, conversely, a cluster's box
+ * is not altered to reflect intra-cluster edges.
+ * 
+ */
+void spline_edges0(graph_t *g, pointf offset)
 {
 	node_t		*n;
 	edge_t		*e;
 	point		dumb[4],d,ld;
-	pointf		offset, polyp;
+	pointf		polyp;
 	Ppoly_t		**obs;
 	polygon_t	*poly;
 	int 		i=0, j, npoly, sides;
@@ -178,36 +267,24 @@ void spline_edges(graph_t *g)
 	Pedge_t     *barriers;
 	double		adj=0.0, SEP;
 	char		*str;
-    point       *sp;
     pointf      scalef;
     int         doScale;
-    int         pn;
 	attrsym_t	*symptr=NULL;
 
 	if ((str = agget(g,"sep"))) {SEP = 1.0 + atof(str);}
 	else SEP = 1.01;
-	neato_compute_bb(g);
-	offset = cvt2ptf(g->u.bb.LL);
-	for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
-		n->u.pos[0] -= offset.x;
-		n->u.pos[1] -= offset.y;
-	}
-	g->u.bb.UR.x -= g->u.bb.LL.x;
-	g->u.bb.UR.y -= g->u.bb.LL.y;
-	g->u.bb.LL.x = 0;
-	g->u.bb.LL.y = 0;
 	doScale = neato_set_aspect(g,&scalef);
 	
 	/* build configuration */
 	if (mapbool(agget(g,"splines"))) {
 		obs = N_NEW (agnnodes(g), Ppoly_t*);
 		for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
-			n->u.coord.x = POINTS(n->u.pos[0]);
-			n->u.coord.y = POINTS(n->u.pos[1]);
+			ND_coord(n).x = POINTS(ND_pos(n)[0]);
+			ND_coord(n).y = POINTS(ND_pos(n)[1]);
 	
-			if (n->u.shape->initfn == poly_init) {
+			if (ND_shape(n)->initfn == poly_init) {
 				obs[i] = NEW(Ppoly_t);
-				poly = (polygon_t*) n->u.shape_info;
+				poly = (polygon_t*) ND_shape_info(n);
 				if (poly->sides >= 3) {
 					sides = poly->sides;
 				}
@@ -227,11 +304,11 @@ void spline_edges(graph_t *g)
 						double	c, s;
 						c = cos(2.0 * PI * j / sides + adj);
 						s = sin(2.0 * PI * j / sides + adj);
-						polyp.x = SEP * c * (n->u.lw + n->u.rw) / 2.0;
-						polyp.y = SEP * s * n->u.ht / 2.0;
+						polyp.x = SEP * c * (ND_lw(n) + ND_rw(n)) / 2.0;
+						polyp.y = SEP * s * ND_ht(n) / 2.0;
 					}
-					obs[i]->ps[sides - j - 1].x = polyp.x + n->u.coord.x;
-					obs[i]->ps[sides - j - 1].y = polyp.y + n->u.coord.y;
+					obs[i]->ps[sides - j - 1].x = polyp.x + ND_coord(n).x;
+					obs[i]->ps[sides - j - 1].y = polyp.y + ND_coord(n).y;
 				}
 				i++;
 			}
@@ -240,8 +317,8 @@ void spline_edges(graph_t *g)
 	else {
     	obs = 0;
 		for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
-			n->u.coord.x = POINTS(n->u.pos[0]);
-			n->u.coord.y = POINTS(n->u.pos[1]);
+			ND_coord(n).x = POINTS(ND_pos(n)[0]);
+			ND_coord(n).y = POINTS(ND_pos(n)[1]);
     	}
     }
 	npoly = i;
@@ -255,7 +332,8 @@ void spline_edges(graph_t *g)
 	}
 
 	/* route edges  */
-	if (Verbose) fprintf(stderr,"neato: compute paths\n");
+	if (Verbose) fprintf(stderr,"Creating edges using %s\n",
+      (vconfig ? "splines" : "line segments"));
 	if (vconfig) {
 		/* path-finding pass */
 		for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
@@ -274,11 +352,10 @@ void spline_edges(graph_t *g)
 				}
 				/*Pobspath(vconfig, p, POLYID_UNKNOWN, q, POLYID_UNKNOWN, &line);*/
 				Pobspath(vconfig, p, pp, q, qp, &line);
-				e->u.path = line;
+				ED_path(e) = line;
 			}
 		}
 	}
-	if (Verbose && vconfig) fprintf(stderr,"neato: fit splines...\n");
 
     if (Nop > 1) {
       symptr = agfindattr(g->proto->e,"pos");
@@ -295,9 +372,7 @@ void spline_edges(graph_t *g)
 			p = mkPoint(coord(e->tail));
 			q = mkPoint(coord(e->head));
             if ((Nop > 1) && 
-              ((sp = user_spline(symptr,e,&pn,&offset,doScale,&scalef)) != 0)) {
-				clip_and_install(e,sp,pn);
-				free(sp);
+              user_spline(symptr,e,&offset,doScale,&scalef)) {
             }
 			else if (vconfig && (e->tail != e->head)) {
 				Ppolyline_t line, spline;
@@ -312,7 +387,7 @@ void spline_edges(graph_t *g)
 					if ((pp == POLYID_NONE) && in_poly(*obs[i], p)) pp = i;
 					if ((qp == POLYID_NONE) && in_poly(*obs[i], q)) qp = i;
 				}
-				line = e->u.path;
+				line = ED_path(e);
 
 				make_barriers(obs, npoly, pp, qp, &barriers, &n_barriers);
 				slopes[0].x = slopes[0].y = 0.0;
@@ -325,7 +400,7 @@ void spline_edges(graph_t *g)
 					ispline[i].x = ROUND(spline.ps[i].x);
 					ispline[i].y = ROUND(spline.ps[i].y);
 				}
-				if (Verbose) fprintf(stderr,"spline %s %s\n",e->tail->name,e->head->name);
+				if (Verbose > 1) fprintf(stderr,"spline %s %s\n",e->tail->name,e->head->name);
 				clip_and_install(e,ispline,spline.pn);
 				free(ispline);
 				free(barriers);
@@ -341,7 +416,7 @@ void spline_edges(graph_t *g)
 					dumb[2] = sub_points(dumb[3],d);
 				}
 				else {	/* self arc */
-					d.x = e->head->u.rw + POINTS(.66 * SEP);
+					d.x = ND_rw(e->head) + POINTS(.66 * SEP);
 					d.y = 0;
 					dumb[1] = dumb[2] = add_points(dumb[0],d);
 					dumb[1].y += d.x;
@@ -350,36 +425,83 @@ void spline_edges(graph_t *g)
 				clip_and_install(e,dumb,4);
 			}
 
-			if (e->u.label) {
+			if (ED_label(e)) {
 				d.x = (q.x + p.x)/ 2;
 				d.y = (p.y + q.y)/ 2;
 				if (abs(p.x - q.x) > abs(p.y - q.y)) {
-					ld.x = 0; ld.y = POINTS(e->u.label->dimen.y)/2 + 2;
+					ld.x = 0; ld.y = POINTS(ED_label(e)->dimen.y)/2 + 2;
 				}
 				else {
-					ld.x = POINTS(e->u.label->dimen.y)/2+e->u.label->fontsize;
+					ld.x = POINTS(ED_label(e)->dimen.y)/2+ED_label(e)->fontsize;
 					ld.y = 0;
 				}
 				d = add_points(d,ld);
-				e->u.label->p = d;
+				ED_label(e)->p = d;
 			}
 		}
 	}
-	/* g->u.bb.UR = sub_points(g->u.bb.UR,g->u.bb.LL); */
-	/* g->u.bb.LL = pointof(0,0); */
+	/* GD_bb(g).UR = sub_points(GD_bb(g).UR,GD_bb(g).LL); */
+	/* GD_bb(g).LL = pointof(0,0); */
 	neato_compute_bb(g);
 
     /* vladimir: place port labels */
     if (E_headlabel || E_taillabel)
       for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
         if (E_headlabel) for (e = agfstin(g,n); e; e = agnxtin(g,e))
-          if (e->u.head_label) place_portlabel (e, TRUE);
+          if (ED_head_label(e)) place_portlabel (e, TRUE);
         if (E_taillabel) for (e = agfstout(g,n); e; e = agnxtout(g,e))
-          if (e->u.tail_label) place_portlabel (e, FALSE);
+          if (ED_tail_label(e)) place_portlabel (e, FALSE);
       }
     /* end vladimir */
+	State = GVSPLINES;
 }
 
+void spline_edges(graph_t *g)
+{
+	node_t		*n;
+	pointf		offset;
+
+	neato_compute_bb(g);
+	offset = cvt2ptf(GD_bb(g).LL);
+	for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+		ND_pos(n)[0] -= offset.x;
+		ND_pos(n)[1] -= offset.y;
+	}
+	GD_bb(g).UR.x -= GD_bb(g).LL.x;
+	GD_bb(g).UR.y -= GD_bb(g).LL.y;
+	GD_bb(g).LL.x = 0;
+	GD_bb(g).LL.y = 0;
+    spline_edges0(g,offset);
+}
+
+
+static int neato_spline_merge(node_t *n)
+{
+    return FALSE;
+}
+
+static void neato_arrow_clip (Agedge_t *fe,Agedge_t *le,point *ps,int *startp,int *endp,bezier *spl)
+{
+	edge_t *e;
+	int i, j, sflag, eflag;
+
+	for (e = fe; ED_to_orig(e); e = ED_to_orig(e))
+	;
+	j = swap_ends_p(e);
+	arrow_flags (e, &sflag, &eflag);
+ 	if (neato_spline_merge (le->head)) eflag = ARR_NONE;
+ 	if (neato_spline_merge (fe->tail)) sflag = ARR_NONE;
+	if (j) {i=sflag; sflag=eflag; eflag=i;} /* swap the two ends */
+	if (sflag) *startp = arrowStartClip (e, ps, *startp, *endp, spl, sflag);
+	if (eflag) *endp = arrowEndClip (e, ps, *startp, *endp, spl, eflag);
+}
+
+static boolean swap_ends_p (edge_t *e)
+{
+	return FALSE;
+}
+
+
 void clip_and_install (edge_t *e, point *ps, int  pn)
 {
 	pointf 		p2;
@@ -395,18 +517,18 @@ void clip_and_install (edge_t *e, point *ps, int  pn)
 		pt = coord(tn);
 		p2.x = ps[start+3].x - pt.x;
 		p2.y = ps[start+3].y - pt.y;
-		if (tn->u.shape == NULL) break;
-		if (tn->u.shape->insidefn == NULL) break;
-		if (tn->u.shape->insidefn (tn, p2, e) == FALSE) break;
+		if (ND_shape(tn) == NULL) break;
+		if (ND_shape(tn)->insidefn == NULL) break;
+		if (ND_shape(tn)->insidefn (tn, p2, e) == FALSE) break;
 	}
 	neato_shape_clip (tn, &ps[start], e);
 	for (end = pn - 4; end > 0; end -= 3) {
 		pt = coord(hn);
 		p2.x = ps[end].x - pt.x;
 		p2.y = ps[end].y - pt.y;
-		if (hn->u.shape == NULL) break;
-		if (hn->u.shape->insidefn == NULL) break;
-		if (hn->u.shape->insidefn (hn, p2, e) == FALSE) break;
+		if (ND_shape(hn) == NULL) break;
+		if (ND_shape(hn)->insidefn == NULL) break;
+		if (ND_shape(hn)->insidefn (hn, p2, e) == FALSE) break;
 	}
 	neato_shape_clip (hn, &ps[end], e);
 	for (; start < pn - 4; start+=3)
@@ -429,15 +551,15 @@ void neato_shape_clip (node_t *n, point curve[4], edge_t *e)
 	point		p;
 	double		low, high, t;
 
-	if (n->u.shape == NULL) return;
-	if (n->u.shape->insidefn == NULL) return;
+	if (ND_shape(n) == NULL) return;
+	if (ND_shape(n)->insidefn == NULL) return;
 	for (i = 0; i < 4; i++) {
 		p = coord(n);
 		c[i].x = curve[i].x - p.x;
 		c[i].y = curve[i].y - p.y;
 	}
 
-	left_inside = n->u.shape->insidefn (n, c[0], e);
+	left_inside = ND_shape(n)->insidefn (n, c[0], e);
 	if (left_inside)
 		left = NULL, right = seg;
 	else
@@ -453,7 +575,7 @@ void neato_shape_clip (node_t *n, point curve[4], edge_t *e)
 		opt = pt;
 		t = (high + low) / 2.0;
 		pt = Bezier (c, 3, t, left, right);
-		inside = n->u.shape->insidefn (n, pt, e);
+		inside = ND_shape(n)->insidefn (n, pt, e);
 		if (inside == FALSE) {
 			for (i = 0; i < 4; i++)
 				best[i] = seg[i];
@@ -475,187 +597,48 @@ void neato_shape_clip (node_t *n, point curve[4], edge_t *e)
 	}
 }
 
-/* #define ARROWLENGTH 9  shouldn't this be ARROW_LENGTH from const.h? */
-/* #define ARROWLENGTHSQ 81  bah */
-#define sqr(a) ((long) (a) * (a))
-#define dstsq(a, b) (sqr (a.x - b.x) + sqr (a.y - b.y))
-#define ldstsq(a, b) (sqr ((long) a.x - b.x) + sqr ((long) a.y - b.y))
-#define dst(a, b) sqrt ((double) dstsq (a, b))
-#define P2PF(p, pf) (pf.x = p.x, pf.y = p.y)
-#define PF2P(pf, p) (p.x = ROUND (pf.x), p.y = ROUND (pf.y))         
-
-/* vladimir */
-static char *arrowdirname[] = {"forward","back","both","none",(char*)0};
-static char *arrowheadname[] = {"none","normal","inv","dot","odot", 
-                          "invdot","invodot",(char*)0};
-static int dir_sflag[] = {ARR_NONE,ARR_NORM,ARR_NORM,ARR_NONE};
-static int dir_eflag[] = {ARR_NORM,ARR_NONE,ARR_NORM,ARR_NONE};
-static int arr_type[] = {ARR_NONE,ARR_NORM,ARR_INV,ARR_DOT,ARR_ODOT,
-                         ARR_INVDOT,ARR_INVODOT};
-#define ARR_TYPES (sizeof(arr_type)/sizeof(arr_type[0]))
-static int arr_len[] = {0,ARROW_LENGTH,ARROW_INV_LENGTH,
-                        ARROW_DOT_RADIUS,ARROW_DOT_RADIUS, 
-                        /* or ARROW_DOT_RADIUS*2 if we want the dot to touch
-                           the node instead of being embedded in it */
-                        ARROW_INV_LENGTH+ARROW_DOT_RADIUS,
-                        ARROW_INV_LENGTH+ARROW_DOT_RADIUS};
-
-void neato_arrow_flags (edge_t *e, int *sflag, int *eflag)
+/* scaleBB:
+ * Scale bounding box of clusters of g by given factors.
+ */
+static void 
+scaleBB(graph_t *g, double xf, double yf)
 {
-  char *attr,*p;
-  int i;
+	int		i;
 
-  *sflag = ARR_NONE;
-  *eflag = AG_IS_DIRECTED(e->tail->graph) ? ARR_NORM : ARR_NONE;
-  if (E_dir && ((attr = agxget (e, E_dir->index)))[0]) {
-    for (i = 0; (p = arrowdirname[i]); i++) {
-      if (streq(attr,p)) {
-        *sflag = dir_sflag[i];
-        *eflag = dir_eflag[i];
-        break;
-      }
-    }
-  }
-  if (E_arrowhead && ((attr = agxget (e, E_arrowhead->index)))[0]) {
-    for (i = 0; (p = arrowheadname[i]); i++) {
-      if (strcmp(attr,p) == 0) {
-        *eflag = arr_type[i];
-        break;
-      }
-    }
-  }
-  if (E_arrowtail && ((attr = agxget (e, E_arrowtail->index)))[0]) {
-    for (i = 0; (p = arrowheadname[i]); i++) {
-      if (strcmp(attr,p) == 0) {
-        *sflag = arr_type[i];
-        break;
-      }
-    }
-  }
-}
-
-double neato_arrow_length (edge_t* e, int flag)
-{
-  int i;
-
-  /* we don't simply index with flag because arr_type's are not consecutive */
-  for (i=0; i<ARR_TYPES; i++) 
-    if (flag==arr_type[i]) {
-      return arr_len[i] * late_float(e,E_arrowsz,1.0,0.0);
-      /* The original was missing the factor E_arrowsz, but I believe it
-         should be here for correct arrow clipping */
-    }
-  return 0;
-}
-/* end vladimir */
-
-static int neato_spline_merge(node_t *n)
-{
-    return FALSE;
-}
-
-void neato_arrow_clip (fe, le, ps, startp, endp, spl)
-	edge_t *fe, *le;
-	point *ps;
-	int *startp, *endp;
-	bezier *spl;
-{
-	edge_t *e;
-	pointf sp[4], sp2[4], pf;
-	int i, j, sflag, eflag;
-	double d, t, slen, elen;
-
-	for (e = fe; e->u.to_orig; e = e->u.to_orig)
-      ;
-    j = swap_ends_p(e);
-    neato_arrow_flags (e, &sflag, &eflag);
- 	if (neato_spline_merge (le->head)) eflag = ARR_NONE;
- 	if (neato_spline_merge (fe->tail)) sflag = ARR_NONE;
-	if (j) {i=sflag; sflag=eflag; eflag=i;} /* swap the two ends */
-    slen = neato_arrow_length(e,sflag);
-    elen = neato_arrow_length(e,eflag);
- 	if (sflag) {
- 		spl->sflag = sflag, spl->sp = ps[*startp];
- 		if (*endp > *startp && ldstsq (ps[*startp], ps[*startp + 3]) < sqr(slen)) {
-			*startp += 3;
-		}
-		P2PF (ps[*startp], sp[0]);
-		P2PF (ps[*startp + 1], sp[1]);
-		P2PF (ps[*startp + 2], sp[2]);
-		P2PF (ps[*startp + 3], sp[3]);
-		d = dst (sp[0], sp[1]) + dst (sp[1], sp[2]) + dst (sp[2], sp[3]);
-		if ((t = slen / d) > 1.0)
-			t = 1.0;
-		else if (t < 0.1)
-			t = 0.1;
-		for (;;) {
-			pf = Bezier (sp, 3, t, NULL, sp2);
-			if (ldstsq (pf, spl->sp) <= sqr(slen))
-				break;
-			t *= (2.0/3.0);
-		}
-		PF2P (sp2[0], ps[*startp]);
-		PF2P (sp2[1], ps[*startp + 1]);
-		PF2P (sp2[2], ps[*startp + 2]);
-		PF2P (sp2[3], ps[*startp + 3]);
-	}
-	if (eflag) {
-		spl->eflag = eflag, spl->ep = ps[*endp + 3];
-		if (*endp > *startp && ldstsq (ps[*endp], ps[*endp + 3]) < sqr(elen)) {
-			*endp -= 3;
-		}
-		P2PF (ps[*endp], sp[3]);
-		P2PF (ps[*endp + 1], sp[2]);
-		P2PF (ps[*endp + 2], sp[1]);
-		P2PF (ps[*endp + 3], sp[0]);
-		d = dst (sp[0], sp[1]) + dst (sp[1], sp[2]) + dst (sp[2], sp[3]);
-		if ((t = elen / d) > 1.0)
-			t = 1.0;
-		else if (t < 0.1)
-			t = 0.1;
-		for (;;) {
-			pf = Bezier (sp, 3, t, NULL, sp2);
-			if (ldstsq (pf, spl->ep) <= sqr(elen))
-				break;
-			t *= (2.0/3.0);
-		}
-		PF2P (sp2[3], ps[*endp]);
-		PF2P (sp2[2], ps[*endp + 1]);
-		PF2P (sp2[1], ps[*endp + 2]);
-		PF2P (sp2[0], ps[*endp + 3]);
-	}
-}
-
-static boolean swap_ends_p (edge_t *e)
-{
-	return FALSE;
+	GD_bb(g).UR.x *= xf;
+	GD_bb(g).UR.y *= yf;
+	GD_bb(g).LL.x *= xf;
+	GD_bb(g).LL.y *= yf;
+    for (i = 1; i <= GD_n_cluster(g); i++)
+      scaleBB(GD_clust(g)[i],xf,yf);
 }
 
 /* neato_set_aspect;
- * Assume bounding box has LL at origin.
+ * Assume all bounding boxes are correct and
+ * that GD_bb(g).LL is at origin.
  */
 static int 
 neato_set_aspect(graph_t *g, pointf* pf)
 {
-	int		i;
+	/* int		i; */
 	double	xf,yf,actual,desired;
 	char	*str;
 	node_t	*n;
 
 	/* neato_compute_bb(g); */
-	if (/* (g->u.maxrank > 0) && */(str = agget(g,"ratio"))) {
+	if (/* (GD_maxrank(g) > 0) && */(str = agget(g,"ratio"))) {
 			/* normalize */
-        assert (g->u.bb.LL.x == 0);
-        assert (g->u.bb.LL.y == 0);
-		/* g->u.bb.UR.x -= g->u.bb.LL.x; */
-		/* g->u.bb.UR.y -= g->u.bb.LL.y; */
-		if (g->u.left_to_right)
-			{int t = g->u.bb.UR.x; g->u.bb.UR.x = g->u.bb.UR.y; g->u.bb.UR.y = t;}
+        assert (GD_bb(g).LL.x == 0);
+        assert (GD_bb(g).LL.y == 0);
+		/* GD_bb(g).UR.x -= GD_bb(g).LL.x; */
+		/* GD_bb(g).UR.y -= GD_bb(g).LL.y; */
+		if (GD_left_to_right(g))
+			{int t = GD_bb(g).UR.x; GD_bb(g).UR.x = GD_bb(g).UR.y; GD_bb(g).UR.y = t;}
 		if (strcmp(str,"fill") == 0) {
 			/* fill is weird because both X and Y can stretch */
-			if (g->u.drawing->size.x <= 0) return 0;
-			xf = (double)g->u.drawing->size.x / (double)g->u.bb.UR.x;
-			yf = (double)g->u.drawing->size.y / (double)g->u.bb.UR.y;
+			if (GD_drawing(g)->size.x <= 0) return 0;
+			xf = (double)GD_drawing(g)->size.x / (double)GD_bb(g).UR.x;
+			yf = (double)GD_drawing(g)->size.y / (double)GD_bb(g).UR.y;
 			if ((xf < 1.0) || (yf < 1.0)) {
 				if (xf < yf) {yf = yf / xf; xf = 1.0;}
 				else {xf = xf / yf; yf = 1.0;}
@@ -664,17 +647,21 @@ neato_set_aspect(graph_t *g, pointf* pf)
 		else {
 			desired = atof(str);
 			if (desired == 0.0) return 0;
-			actual = ((float)g->u.bb.UR.y)/((float)g->u.bb.UR.x);
+			actual = ((double)GD_bb(g).UR.y)/((double)GD_bb(g).UR.x);
 			if (actual < desired) {yf = desired/actual; xf = 1.0;}
 			else {xf = actual/desired; yf = 1.0;}
 		}
-		if (g->u.left_to_right) {float t = xf; xf = yf; yf = t;}
-		for (i = 0; (n = g->u.neato_nlist[i]); i++) {
-			n->u.pos[0] = n->u.pos[0] * xf;
-			n->u.pos[1] = n->u.pos[1] * yf;
+		if (GD_left_to_right(g)) {double t = xf; xf = yf; yf = t;}
+
+		/* Not relying on neato_nlist here allows us not to have to 
+         * allocate it in the root graph and the connected components. 
+		 */
+		for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+		/* for (i = 0; (n = GD_neato_nlist(g)[i]); i++) { */
+			ND_pos(n)[0] = ND_pos(n)[0] * xf;
+			ND_pos(n)[1] = ND_pos(n)[1] * yf;
 		}
-		g->u.bb.UR.x *= xf;
-		g->u.bb.UR.y *= yf;
+        scaleBB (g, xf, yf);
 		pf->x = xf;
 		pf->y = yf;
         return 1;
@@ -689,17 +676,19 @@ static void place_portlabel (edge_t *e, boolean head_p)
   textlabel_t *l;
   splines *spl;
   bezier *bez;
-  float dist, angle;
+  double dist, angle;
   point p;
   pointf c[4], pf;
   int i;
 
-  l = head_p ? e->u.head_label : e->u.tail_label;
+  l = head_p ? ED_head_label(e) : ED_tail_label(e);
   if (swap_ends_p(e)) head_p = !head_p;
   spl = getsplinepoints(e);
   if (!head_p) {
     bez = &spl->list[0];
-    if (bez->sflag) {
+    if (bez->sflag != ARR_NONE
+		    && bez->sflag != ARR_OPEN
+		    && bez->sflag != ARR_HALFOPEN) {
       p = bez->sp; 
       P2PF(bez->list[0],pf);
     }
@@ -710,7 +699,9 @@ static void place_portlabel (edge_t *e, boolean head_p)
     }
   } else {
     bez = &spl->list[spl->size-1];
-    if (bez->eflag) {
+    if (bez->eflag != ARR_NONE
+		    && bez->eflag != ARR_OPEN
+		    && bez->eflag != ARR_HALFOPEN) {
       p = bez->ep; 
       P2PF(bez->list[bez->size-1],pf);
     }
@@ -721,8 +712,8 @@ static void place_portlabel (edge_t *e, boolean head_p)
     }
   }
   angle = atan2 (pf.y-p.y, pf.x-p.x) + 
-    RADIANS(late_float(e,E_labelangle,PORT_LABEL_ANGLE,-180.0));
-  dist = PORT_LABEL_DISTANCE * late_float(e,E_labeldistance,1.0,0.0);
+    RADIANS(late_double(e,E_labelangle,PORT_LABEL_ANGLE,-180.0));
+  dist = PORT_LABEL_DISTANCE * late_double(e,E_labeldistance,1.0,0.0);
   l->p.x = p.x + ROUND(dist * cos(angle));
   l->p.y = p.y + ROUND(dist * sin(angle));
 }
@@ -731,7 +722,7 @@ static splines *getsplinepoints (edge_t* e)
 {
 	splines *sp;
 
-	sp = e->u.spl;
+	sp = ED_spl(e);
 	if (sp == NULL) abort ();
 	return sp;
 }
