@@ -14,13 +14,13 @@
  * Derived from Graham Wills' algorithm described in GD'97.
  */
 
-#include    "circle.h"
 #ifdef HAVE_CONFIG_H
-#include "gvconfig.h"
+#include "config.h"
 #endif
+#include   "circle.h"
 #include        <time.h>
-#ifndef MSWIN32
-#include        <unistd.h>
+#ifdef HAVE_UNISTD_H
+#include	<unistd.h>
 #endif
 
 #ifdef DMALLOC
@@ -30,7 +30,7 @@
 char *Info[] = {
     "twopi",            /* Program */
     VERSION,            /* Version */
-    DATE                /* Build Date */
+    BUILDDATE           /* Build Date */
 };
 
 static graph_t *G;
@@ -55,7 +55,10 @@ int main (int argc, char** argv)
 #endif
 
   while ((G = next_input_graph())) {
-    if (prev) twopi_cleanup(prev);
+	if (prev) {
+      twopi_cleanup(prev);
+      agclose(prev);
+	}
     prev = G;
     twopi_layout(G);
     dotneato_write(G);
