@@ -1,8 +1,13 @@
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
-#include <zlib.h>
+/* 2.03: don't include zlib here or we can't build without PNG */
 #include "gd.h"
 #include "gdhelpers.h"
 
@@ -59,6 +64,7 @@ extern int gdSinT[];
 
 static void gdImageBrushApply (gdImagePtr im, int x, int y);
 static void gdImageTileApply (gdImagePtr im, int x, int y);
+static int gdImageGetTrueColorPixel(gdImagePtr im, int x, int y);
 
 gdImagePtr
 gdImageCreate (int sx, int sy)
@@ -121,8 +127,13 @@ gdImageCreateTrueColor (int sx, int sy)
   im->transparent = (-1);
   im->interlace = 0;
   im->trueColor = 1;
-  im->saveAlphaFlag = 1;
-  im->alphaBlendingFlag = 0;
+  /* 2.0.2: alpha blending is now on by default, and saving of alpha is
+    off by default. This allows font antialiasing to work as expected
+    on the first try in JPEGs -- quite important -- and also allows 
+    for smaller PNGs when saving of alpha channel is not really 
+    desired, which it usually isn't! */
+  im->saveAlphaFlag = 0;
+  im->alphaBlendingFlag = 1;
   im->thick = 1;
   return im;
 }
@@ -186,7 +197,8 @@ gdImageColorClosestAlpha (gdImagePtr im, int r, int g, int b, int a)
       rd = (im->red[i] - r);
       gd = (im->green[i] - g);
       bd = (im->blue[i] - b);
-      ad = (im->blue[i] - b);
+      /* gd 2.02: whoops, was - b (thanks to David Marwood) */
+      ad = (im->blue[i] - a);
       dist = rd * rd + gd * gd + bd * bd + ad * ad;
       if (first || (dist < mindist))
 	{
@@ -290,6 +302,7 @@ HWB_Diff (int r1, int g1, int b1, int r2, int g2, int b2)
 }
 
 
+#if 0
 /*
  * This is not actually used, but is here for completeness, in case someone wants to
  * use the HWB stuff for anything else...
@@ -334,6 +347,7 @@ HWB_to_RGB (HWBType HWB, RGBType * RGB)
   return RGB;
 
 }
+#endif
 
 int
 gdImageColorClosestHWB (gdImagePtr im, int r, int g, int b)
@@ -473,6 +487,12 @@ gdImageColorResolveAlpha (gdImagePtr im, int r, int g, int b, int a)
 	  op = c;		/* Save open slot */
 	  continue;		/* Color not in use */
 	}
+      if (c == im->transparent)
+        {
+          /* don't ever resolve to the color that has
+           * been designated as the transparent color */
+          continue;
+	}
       rd = (long) (im->red[c] - r);
       gd = (long) (im->green[c] - g);
       bd = (long) (im->blue[c] - b);
@@ -686,8 +706,11 @@ gdImageBrushApply (gdImagePtr im, int x, int y)
 	      int p;
 	      p = gdImageGetTrueColorPixel (
 					     im->brush, srcx, srcy);
-	      gdImageSetPixel (im, lx, ly,
-			       p);
+	      /* 2.0.9, Thomas Winzig: apply simple full transparency */
+	      if (p != gdImageGetTransparent (im->brush))
+		{
+		  gdImageSetPixel (im, lx, ly, p);
+		}
 	      srcx++;
 	    }
 	  srcy++;
@@ -766,7 +789,8 @@ gdImageTileApply (gdImagePtr im, int x, int y)
 	    }
 	  else
 	    {
-	      gdImageSetPixel (im, x, y, im->tileColorMap[p]);
+	      gdImageSetPixel (im, x, y,
+			       im->tileColorMap[p]);
 	    }
 	}
     }
@@ -792,7 +816,7 @@ gdImageGetPixel (gdImagePtr im, int x, int y)
     }
 }
 
-int
+static int
 gdImageGetTrueColorPixel (gdImagePtr im, int x, int y)
 {
   int p = gdImageGetPixel (im, x, y);
@@ -821,10 +845,15 @@ gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
   if (dy <= dx)
     {
       /* More-or-less horizontal. use wid for vertical stroke */
-      wid = thick * cos (atan2 (dy, dx));
-      if (wid == 0)
-	wid = 1;
-
+      /* Doug Claar: watch out for NaN in atan2 (2.0.5) */
+      if ((dx == 0) && (dy == 0)) {
+        wid = 1;
+      } else {
+        wid = thick * cos (atan2 (dy, dx));
+        if (wid == 0) {
+          wid = 1;
+        }
+      }
       d = 2 * dy - dx;
       incr1 = 2 * dy;
       incr2 = 2 * (dy - dx);
@@ -967,7 +996,7 @@ gdImageDashedLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
   int dashStep = 0;
   int on = 1;
   int wid;
-  int w, wstart, vert;
+  int vert;
   int thick = im->thick;
 
   dx = abs (x2 - x1);
@@ -1297,10 +1326,7 @@ gdImageFilledArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e, int
   gdPoint pts[3];
   int i;
   int lx = 0, ly = 0;
-  int fx, fy;
-  int w2, h2;
-  w2 = w / 2;
-  h2 = h / 2;
+  int fx = 0, fy = 0;
   while (e < s)
     {
       e += 360;
@@ -1308,8 +1334,8 @@ gdImageFilledArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e, int
   for (i = s; (i <= e); i++)
     {
       int x, y;
-      x = ((long) gdCosT[i % 360] * (long) w2 / 1024) + cx;
-      y = ((long) gdSinT[i % 360] * (long) h2 / 1024) + cy;
+      x = ((long) gdCosT[i % 360] * (long) w / (2*1024) ) + cx;
+      y = ((long) gdSinT[i % 360] * (long) h / (2*1024) ) + cy;
       if (i != s)
 	{
 	  if (!(style & gdChord))
@@ -1679,6 +1705,7 @@ gdImageCopy (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int s
       for (x = srcX; (x < (srcX + w)); x++)
 	{
 	  int nc;
+          int mapTo;
 	  c = gdImageGetPixel (src, x, y);
 	  /* Added 7/24/95: support transparent copies */
 	  if (gdImageGetTransparent (src) == c)
@@ -1687,7 +1714,19 @@ gdImageCopy (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int s
 	      continue;
 	    }
 	  /* Have we established a mapping for this color? */
-	  if (colorMap[c] == (-1))
+	  if (src->trueColor)
+	    {
+	      /* 2.05: remap to the palette available in the
+	         destination image. This is slow and
+	         works badly, but it beats crashing! Thanks 
+                 to Padhrig McCarthy. */
+	      mapTo = gdImageColorResolveAlpha (dst,
+					      gdTrueColorGetRed (c),
+					    gdTrueColorGetGreen (c),
+					     gdTrueColorGetBlue (c),
+					   gdTrueColorGetAlpha (c));
+	    }
+	  else if (colorMap[c] == (-1))
 	    {
 	      /* If it's the same image, mapping is trivial */
 	      if (dst == src)
@@ -1704,8 +1743,13 @@ gdImageCopy (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int s
 					       src->blue[c], src->alpha[c]);
 		}
 	      colorMap[c] = nc;
+              mapTo = colorMap[c];  
 	    }
-	  gdImageSetPixel (dst, tox, toy, colorMap[c]);
+          else
+            {
+              mapTo = colorMap[c];  
+            }
+	  gdImageSetPixel (dst, tox, toy, mapTo);
 	  tox++;
 	}
       toy++;
@@ -1877,7 +1921,7 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX
 	  tox = dstX;
 	  for (x = srcX; (x < (srcX + srcW)); x++)
 	    {
-	      int nc;
+	      int nc = 0;
 	      int mapTo;
 	      if (!stx[x - srcX])
 		{
@@ -1885,7 +1929,21 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX
 		}
 	      if (dst->trueColor)
 		{
-		  int d;
+		  /* 2.0.9: Thorben Kundinger: Maybe the source image is not 
+		     a truecolor image */
+		  if (!src->trueColor)
+		    {
+		      int tmp = gdImageGetPixel (src, x, y);
+		      mapTo = gdImageGetTrueColorPixel (src, x, y);
+		      if (gdImageGetTransparent (src) == tmp)
+			{
+			  tox++;
+			  continue;
+			}
+		    }
+		  else
+		    {
+		      /* TK: old code follows */
 		  mapTo = gdImageGetTrueColorPixel (src, x, y);
 		  /* Added 7/24/95: support transparent copies */
 		  if (gdImageGetTransparent (src) == mapTo)
@@ -1894,6 +1952,7 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX
 		      continue;
 		    }
 		}
+		}
 	      else
 		{
 		  c = gdImageGetPixel (src, x, y);
@@ -1927,11 +1986,12 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX
 			  else
 			    {
 			      /* Find or create the best match */
-			      mapTo = gdImageColorResolveAlpha (dst,
-						      gdTrueColorGetRed (c),
-						    gdTrueColorGetGreen (c),
-						     gdTrueColorGetBlue (c),
-						   gdTrueColorGetAlpha (c));
+                  /* 2.0.5: can't use gdTrueColorGetRed, etc with palette */
+			      nc = gdImageColorResolveAlpha (dst,
+						      gdImageRed (src, c),
+						    gdImageGreen (src, c),
+						     gdImageBlue (src, c),
+						   gdImageAlpha (src, c));
 			    }
 			  colorMap[c] = nc;
 			}
@@ -1951,6 +2011,81 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX
   gdFree (sty);
 }
 
+/* gd 2.0.8: gdImageCopyRotated is added. Source 
+	is a rectangle, with its upper left corner at
+	srcX and srcY. Destination is the *center* of
+        the rotated copy. Angle is in degrees, same as
+        gdImageArc. Floating point destination center
+	coordinates allow accurate rotation of 
+	objects of odd-numbered width or height. */
+
+void
+gdImageCopyRotated (gdImagePtr dst,
+		    gdImagePtr src,
+		    double dstX, double dstY,
+		    int srcX, int srcY,
+		    int srcWidth, int srcHeight, int angle)
+{
+  double dx, dy;
+  double radius = sqrt (srcWidth * srcWidth + srcHeight * srcHeight);
+  double aCos = cos (angle * .0174532925);
+  double aSin = sin (angle * .0174532925);
+  double scX = srcX + ((double) srcWidth) / 2;
+  double scY = srcY + ((double) srcHeight) / 2;
+  int cmap[gdMaxColors];
+  int i;
+  for (i = 0; (i < gdMaxColors); i++)
+    {
+      cmap[i] = (-1);
+    }
+  for (dy = dstY - radius; (dy <= dstY + radius); dy++)
+    {
+      for (dx = dstX - radius; (dx <= dstX + radius); dx++)
+	{
+	  double sxd = (dx - dstX) * aCos - (dy - dstY) * aSin;
+	  double syd = (dy - dstY) * aCos + (dx - dstX) * aSin;
+	  int sx = sxd + scX;
+	  int sy = syd + scY;
+	  if ((sx >= srcX) && (sx < srcX + srcWidth) &&
+	      (sy >= srcY) && (sy < srcY + srcHeight))
+	    {
+	      int c = gdImageGetPixel (src, sx, sy);
+	      if (!src->trueColor)
+		{
+		  /* Use a table to avoid an expensive
+		     lookup on every single pixel */
+		  if (cmap[c] == -1)
+		    {
+		      cmap[c] = gdImageColorResolveAlpha (dst,
+							  gdImageRed (src, c),
+							  gdImageGreen (src,
+									c),
+							  gdImageBlue (src,
+								       c),
+							  gdImageAlpha (src,
+									c));
+		    }
+		  gdImageSetPixel (dst, dx, dy, cmap[c]);
+		}
+	      else
+		{
+		  gdImageSetPixel (dst,
+				   dx, dy,
+				   gdImageColorResolveAlpha (dst,
+							     gdImageRed (src,
+									 c),
+							     gdImageGreen
+							     (src, c),
+							     gdImageBlue (src,
+									  c),
+							     gdImageAlpha
+							     (src, c)));
+		}
+	    }
+	}
+    }
+}
+
 /* When gd 1.x was first created, floating point was to be avoided.
    These days it is often faster than table lookups or integer
    arithmetic. The routine below is shamelessly, gloriously
@@ -1965,7 +2100,6 @@ gdImageCopyResampled (gdImagePtr dst,
 		      int srcW, int srcH)
 {
   int x, y;
-  float sx, sy;
   if (!dst->trueColor)
     {
       gdImageCopyResized (
@@ -1977,7 +2111,6 @@ gdImageCopyResampled (gdImagePtr dst,
     {
       for (x = dstX; (x < dstX + dstW); x++)
 	{
-	  int pd = gdImageGetPixel (dst, x, y);
 	  float sy1, sy2, sx1, sx2;
 	  float sx, sy;
 	  float spixels = 0;
@@ -2035,10 +2168,11 @@ gdImageCopyResampled (gdImagePtr dst,
 		      xportion = 1.0;
 		    }
 		  pcontribution = xportion * yportion;
-		  p = gdImageGetTrueColorPixel (
-						 src,
-						 (int) sx,
-						 (int) sy);
+		  /* 2.08: previously srcX and srcY were ignored.
+		     Andrew Pattison */
+		  p = gdImageGetTrueColorPixel (src,
+						(int) sx + srcX,
+						(int) sy + srcY);
 		  red += gdTrueColorGetRed (p) * pcontribution;
 		  green += gdTrueColorGetGreen (p) * pcontribution;
 		  blue += gdTrueColorGetBlue (p) * pcontribution;
@@ -2157,7 +2291,7 @@ gdImageCreateFromXbm (FILE * fd)
   for (i = 0; (i < bytes); i++)
     {
       char h[3];
-      int b;
+      unsigned int b;
       /* Skip spaces, commas, CRs, 0x */
       while (1)
 	{
