@@ -5,11 +5,12 @@
 extern "C" {
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include "gvconfig.h"
-#else
-/* default fontpath for unix systems */
-#define DEFAULT_FONTPATH "/usr/share/fonts/truetype"
+/* some might want to set DEFAULT_FONTPATH from configure in config.h */
+#ifndef DEFAULT_FONTPATH
+/* default fontpath for unix systems  - whatever happened to standards ! */
+#define DEFAULT_FONTPATH "/usr/X11R6/lib/X11/fonts/TrueType:/usr/X11R6/lib/X11/fonts/truetype:/usr/X11R6/lib/X11/fonts/TTF:/usr/share/fonts/TrueType:/usr/share/fonts/truetype:/usr/openwin/lib/X11/fonts/TrueType:/usr/X11R6/lib/X11/fonts/Type1"
+#endif
+#ifndef PATHSEPARATOR
 #define PATHSEPARATOR ":"
 #endif
 
@@ -249,6 +250,9 @@ void gdImageStringUp(gdImagePtr im, gdFontPtr f, int x, int y, unsigned char *s,
 void gdImageString16(gdImagePtr im, gdFontPtr f, int x, int y, unsigned short *s, int color);
 void gdImageStringUp16(gdImagePtr im, gdFontPtr f, int x, int y, unsigned short *s, int color);
 
+/* clean up after using fonts in gdImageStringFT() */
+void gdFreeFontCache();
+
 /* Calls gdImageStringFT. Provided for backwards compatibility only. */
 char *gdImageStringTTF(gdImage *im, int *brect, int fg, char *fontlist,
                 double ptsize, double angle, int x, int y, char *string);
@@ -257,6 +261,20 @@ char *gdImageStringTTF(gdImage *im, int *brect, int fg, char *fontlist,
 char *gdImageStringFT(gdImage *im, int *brect, int fg, char *fontlist,
                 double ptsize, double angle, int x, int y, char *string);
 
+/* 2.0.5: provides an extensible way to pass additional parameters.
+       Thanks to Wez Furlong, sorry for the delay. */
+
+typedef struct {
+       int flags; /* for future expansion logical OR of gdFTEX_ values */
+       double linespacing; /* fine tune line spacing for '\n' */
+} gdFTStringExtra, *gdFTStringExtraPtr;
+
+#define gdFTEX_LINESPACE 1
+
+char *gdImageStringFTEx(gdImage *im, int *brect, int fg, char *fontlist,
+               double ptsize, double angle, int x, int y, char *string,
+               gdFTStringExtraPtr strex);
+
 /* Point type for use in polygon drawing. */
 typedef struct {
 	int x, y;
@@ -277,6 +295,8 @@ int gdImageColorClosest(gdImagePtr im, int r, int g, int b);
 	beats the exact same color with radically different
 	transparency */
 int gdImageColorClosestAlpha(gdImagePtr im, int r, int g, int b, int a);
+/* An alternate method */
+int gdImageColorClosestHWB(gdImagePtr im, int r, int g, int b);
 /* Returns exact, 100% opaque matches only */
 int gdImageColorExact(gdImagePtr im, int r, int g, int b);
 /* Returns an exact match only, including alpha */
@@ -350,6 +370,8 @@ void *gdImageWBMPPtr(gdImagePtr im, int *size, int fg);
 void gdImageGif(gdImagePtr im, FILE *outFile);
 void gdImageGifCtx(gdImagePtr im, gdIOCtx *out);
 
+/* 100 is highest quality (there is always a little loss with JPEG).
+       0 is lowest. 10 is about the lowest useful setting. */
 void gdImageJpeg(gdImagePtr im, FILE *out, int quality);
 void gdImageJpegCtx(gdImagePtr im, gdIOCtx *out, int quality);
 
@@ -426,6 +448,19 @@ void gdImageCopyResized(gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int
 	substituted automatically. */
 void gdImageCopyResampled(gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int srcY, int dstW, int dstH, int srcW, int srcH);
 
+/* gd 2.0.8: gdImageCopyRotated is added. Source
+        is a rectangle, with its upper left corner at
+        srcX and srcY. Destination is the *center* of
+        the rotated copy. Angle is in degrees, same as
+        gdImageArc. Floating point destination center
+        coordinates allow accurate rotation of
+        objects of odd-numbered width or height. */
+void gdImageCopyRotated (gdImagePtr dst,
+                        gdImagePtr src,
+                        double dstX, double dstY,
+                        int srcX, int srcY,
+                        int srcWidth, int srcHeight, int angle);
+
 void gdImageSetBrush(gdImagePtr im, gdImagePtr brush);
 void gdImageSetTile(gdImagePtr im, gdImagePtr tile);
 void gdImageSetStyle(gdImagePtr im, int *style, int noOfPixels);
