@@ -51,7 +51,7 @@ void gdImageGif(gdImagePtr im, FILE *outFile)
 {
         gdIOCtx *out = gdNewFileCtx(outFile);
         gdImageGifCtx(im, out);
-        out->free(out);
+        out->gd_free(out);
 }
 
 void* gdImageGifPtr(gdImagePtr im, int *size)
@@ -60,7 +60,7 @@ void* gdImageGifPtr(gdImagePtr im, int *size)
         gdIOCtx *out = gdNewDynamicCtx(2048, NULL);
         gdImageGifCtx(im, out);
         rv = gdDPExtractData(out, size);
-        out->free(out);
+        out->gd_free(out);
         return rv;
 }
 
