@@ -1,5 +1,3 @@
-
-
 /*
  * gd_jpeg.c: Read and write JPEG (JFIF) format image files using the
  * gd graphics library (http://www.boutell.com/gd/).
@@ -20,6 +18,10 @@
  * major CGI brain damage
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <setjmp.h>
@@ -27,12 +29,11 @@
 #include <string.h>
 
 #include "gd.h"
-
 /* TBB: move this up so include files are not brought in */
-/* JCE: HAVE_LIBJPEG is set by gvconfig.h in gd.h */
+/* JCE: arrange HAVE_LIBJPEG so that it can be set in gd.h */
 #ifdef HAVE_LIBJPEG
-
 #include "gdhelpers.h"
+
 /* 1.8.1: remove dependency on jinclude.h */
 #include "jpeglib.h"
 #include "jerror.h"
@@ -87,7 +88,7 @@ gdImageJpeg (gdImagePtr im, FILE * outFile, int quality)
 {
   gdIOCtx *out = gdNewFileCtx (outFile);
   gdImageJpegCtx (im, out, quality);
-  out->free (out);
+  out->gd_free (out);
 }
 
 void *
@@ -97,7 +98,7 @@ gdImageJpegPtr (gdImagePtr im, int *size, int quality)
   gdIOCtx *out = gdNewDynamicCtx (2048, NULL);
   gdImageJpegCtx (im, out, quality);
   rv = gdDPExtractData (out, size);
-  out->free (out);
+  out->gd_free (out);
   return rv;
 }
 
@@ -251,7 +252,6 @@ gdImageJpegCtx (gdImagePtr im, gdIOCtx * outfile, int quality)
 	}
     }
   jpeg_finish_compress (&cinfo);
-error:
   jpeg_destroy_compress (&cinfo);
   gdFree (row);
 }
@@ -262,7 +262,7 @@ gdImageCreateFromJpeg (FILE * inFile)
   gdImagePtr im;
   gdIOCtx *in = gdNewFileCtx (inFile);
   im = gdImageCreateFromJpegCtx (in);
-  in->free (in);
+  in->gd_free (in);
   return im;
 }
 
@@ -851,4 +851,4 @@ jpeg_gdIOCtx_dest (j_compress_ptr cinfo, gdIOCtx * outfile)
   dest->outfile = outfile;
 }
 
-#endif /* HAVE_JPEG */
+#endif /* HAVE_LIBJPEG */
