@@ -1,9 +1,15 @@
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
 #include "gd.h"
 
+/* JCE: Arrange HAVE_LIBPNG so that it can be set in gd.h */
 #ifdef HAVE_LIBPNG
 
 #include "gdhelpers.h"
@@ -101,7 +107,7 @@ gdImageCreateFromPng (FILE * inFile)
   gdImagePtr im;
   gdIOCtx *in = gdNewFileCtx (inFile);
   im = gdImageCreateFromPngCtx (in);
-  in->free (in);
+  in->gd_free (in);
   return im;
 }
 
@@ -125,7 +131,7 @@ gdImageCreateFromPngCtx (gdIOCtx * infile)
   png_bytep image_data = NULL;
   png_bytepp row_pointers = NULL;
   gdImagePtr im = NULL;
-  int i, j, *open;
+  int i, j, *open = NULL;
   volatile int transparent = -1;
   volatile int palette_allocated = FALSE;
 
@@ -225,7 +231,8 @@ gdImageCreateFromPngCtx (gdIOCtx * infile)
 	      im->alpha[i] = gdAlphaMax - (trans[i] >> 1);
 	      if ((trans[i] == 0) && (firstZero))
 		{
-		  im->transparent = i;
+                  /* 2.0.5: long-forgotten patch from Wez Furlong */
+		  transparent = i;
 		  firstZero = 0;
 		}
 	    }
@@ -421,7 +428,7 @@ gdImagePng (gdImagePtr im, FILE * outFile)
 {
   gdIOCtx *out = gdNewFileCtx (outFile);
   gdImagePngCtx (im, out);
-  out->free (out);
+  out->gd_free (out);
 }
 
 void *
@@ -431,7 +438,7 @@ gdImagePngPtr (gdImagePtr im, int *size)
   gdIOCtx *out = gdNewDynamicCtx (2048, NULL);
   gdImagePngCtx (im, out);
   rv = gdDPExtractData (out, size);
-  out->free (out);
+  out->gd_free (out);
   return rv;
 }
 
@@ -442,7 +449,7 @@ gdImagePngPtr (gdImagePtr im, int *size)
 void
 gdImagePngCtx (gdImagePtr im, gdIOCtx * outfile)
 {
-  int i, j, bit_depth, interlace_type;
+  int i, j, bit_depth = 0, interlace_type;
   int width = im->sx;
   int height = im->sy;
   int colors = im->colorsTotal;
@@ -563,9 +570,10 @@ gdImagePngCtx (gdImagePtr im, gdIOCtx * outfile)
     }
   if (im->trueColor && (!im->saveAlphaFlag) && (transparent >= 0))
     {
-      trans_rgb_value.red = gdTrueColorGetRed (im->trueColor);
-      trans_rgb_value.green = gdTrueColorGetGreen (im->trueColor);
-      trans_rgb_value.blue = gdTrueColorGetBlue (im->trueColor);
+      /* 2.0.9: fixed by Thomas Winzig */
+      trans_rgb_value.red = gdTrueColorGetRed (im->transparent);
+      trans_rgb_value.green = gdTrueColorGetGreen (im->transparent);
+      trans_rgb_value.blue = gdTrueColorGetBlue (im->transparent);
       png_set_tRNS (png_ptr, info_ptr, 0, 0, &trans_rgb_value);
     }
   if (!im->trueColor)
@@ -579,7 +587,6 @@ gdImagePngCtx (gdImagePtr im, gdIOCtx * outfile)
       int i;
       int j;
       int k;
-      int highTrans = -1;
       for (i = 0; (i < im->colorsTotal); i++)
 	{
 	  if ((!im->open[i]) &&
@@ -595,7 +602,7 @@ gdImagePngCtx (gdImagePtr im, gdIOCtx * outfile)
 	    {
 	      trans_values[i] = 255 -
 		((im->alpha[i] << 1) +
-		 (im->alpha[i] >> 7));
+		 (im->alpha[i] >> 6));
 	    }
 	  png_set_tRNS (png_ptr, info_ptr, trans_values, 256, NULL);
 #endif
@@ -614,9 +621,10 @@ gdImagePngCtx (gdImagePtr im, gdIOCtx * outfile)
 		{
 		  if (im->alpha[i] != gdAlphaOpaque)
 		    {
+                      /* Andrew Hull: >> 6, not >> 7! (gd 2.0.5) */ 
 		      trans_values[j] = 255 -
 			((im->alpha[i] << 1) +
-			 (im->alpha[i] >> 7));
+			 (im->alpha[i] >> 6));
 		      mapping[i] = j++;
 		    }
 		  else
@@ -697,7 +705,8 @@ gdImagePngCtx (gdImagePtr im, gdIOCtx * outfile)
 		     127 maps to 255. We also have to invert to match
 		     PNG's convention in which 255 is opaque. */
 		  a = gdTrueColorGetAlpha (im->tpixels[j][i]);
-		  row_pointers[j][bo++] = 255 - ((a << 1) + (a >> 7));
+                  /* Andrew Hull: >> 6, not >> 7! (gd 2.0.5) */ 
+		  row_pointers[j][bo++] = 255 - ((a << 1) + (a >> 6));
 		}
 	    }
 	}
