@@ -2,9 +2,13 @@
 /********************************************/
 /* gd interface to freetype library         */
 /*                                          */
-/* John Ellson   ellson@lucent.com          */
+/* John Ellson   ellson@graphviz.org        */
 /********************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -15,22 +19,36 @@
 #ifndef MSWIN32
 #include <unistd.h>
 #else
-#define R_OK 4
+#include <io.h>
+#define R_OK 04    /* Needed in Windows */
 #endif
 
 /* number of antialised colors for indexed bitmaps */
 #define NUMCOLORS 8
 
+char *
+gdImageStringTTF (gdImage * im, int *brect, int fg, char *fontlist,
+		  double ptsize, double angle, int x, int y, char *string)
+{
+  /* 2.0.6: valid return */ 
+  return gdImageStringFT (im, brect, fg, fontlist, ptsize,
+		   angle, x, y, string);
+}
+
 #ifndef HAVE_LIBFREETYPE
+char *
+gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
+		 double ptsize, double angle, int x, int y, char *string,
+		 gdFTStringExtraPtr strex)
+{
+  return "libgd was not built with FreeType font support\n";
+}
+
 char *
 gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 		 double ptsize, double angle, int x, int y, char *string)
 {
-#ifdef HAVE_LIBTTF
-  return gdImageStringTTF(im,brect,fg,fontlist,ptsize,angle,x,y,string);
-#else
   return "libgd was not built with FreeType font support\n";
-#endif
 }
 #else
 
@@ -57,19 +75,6 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
  */
 #define LISTSEPARATOR " "
 
-/*
- * DEFAULT_FONTPATH and PATHSEPARATOR are host type dependent and
- * are normally set by configure in gvconfig.h.  These are just
- * some last resort values that might match some Un*x system
- * if building this version of gd separate from graphviz.
- */
-#ifndef DEFAULT_FONTPATH
-#define DEFAULT_FONTPATH "/usr/share/fonts/truetype"
-#endif
-#ifndef PATHSEPARATOR
-#define PATHSEPARATOR ":"
-#endif
-
 #ifndef TRUE
 #define FALSE 0
 #define TRUE !FALSE
@@ -362,10 +367,11 @@ fontFetch (char **error, void *key)
        */
       fullname = gdRealloc (fullname,
 			    strlen (fontsearchpath) + strlen (name) + 6);
-      /* if name is an absolute filename then test directly */
-      if (*name == '/' || (name[0] != 0 && name[1] == ':' && (name[2] == '/' || name[2] == '\\')))
+      /* if name is an absolute or relative pathname then test directly */
+      if (strchr(name, '/') || (name[0] != 0 && name[1] == ':' && (name[2] == '/' || name[2] == '\\')))
 	{
 	  sprintf (fullname, "%s", name);
+
 	  if (access (fullname, R_OK) == 0)
 	    {
 	      font_found++;
@@ -381,11 +387,23 @@ fontFetch (char **error, void *key)
 	      font_found++;
 	      break;
 	    }
+          sprintf (fullname, "%s/%s.pfa", dir, name);
+          if (access (fullname, R_OK) == 0)
+            {
+	      font_found++;
+	      break;
 	}
-      if (font_found)
-	 break;
-      gdFree (path);
+	  sprintf (fullname, "%s/%s.pfb", dir, name);
+	  if (access (fullname, R_OK) == 0)
+	    {
+	      font_found++;
+	break;
     }
+	}
+  gdFree (path);
+      if (font_found)
+	break;
+  }
   gdFree (fontlist);
   if (!font_found)
     {
@@ -442,7 +460,8 @@ fontFetch (char **error, void *key)
       *error = "Unable to find a CharMap that I can handle";
       return NULL;
     }
-
+  /* 2.0.5: we should actually return this */
+  a->face->charmap = found;
   return (void *) a;
 }
 
@@ -475,7 +494,10 @@ tweenColorTest (void *element, void *key)
  * Computes a color in im's color table that is part way between
  * the background and foreground colors proportional to the gray
  * pixel value in the range 0-NUMCOLORS. The fg and bg colors must already
- * be in the color table.
+ * be in the color table for palette images. For truecolor images the
+ * returned value simply has an alpha component and gdImageAlphaBlend
+ * does the work so that text can be alpha blended across a complex
+ * background (TBB; and for real in 2.0.2).
  */
 static void *
 tweenColorFetch (char **error, void *key)
@@ -494,31 +516,34 @@ tweenColorFetch (char **error, void *key)
   /* if fg is specified by a negative color idx, then don't antialias */
   if (fg < 0)
     {
+      if ((pixel + pixel) >= NUMCOLORS)
       a->tweencolor = -fg;
+      else
+	  a->tweencolor = bg;
     }
   else
     {
       npixel = NUMCOLORS - pixel;
       if (im->trueColor)
-	{
-	  /* 2.0.1: use gdImageSetPixel to do the alpha blending work,
-	     or to just store the alpha level. All we have to do here
-	     is incorporate our knowledge of the percentage of this
-	     pixel that is really "lit" by pushing the alpha value
-	     up toward transparency in edge regions. */
-	  a->tweencolor = gdTrueColorAlpha (
-					     gdTrueColorGetRed (fg),
-					     gdTrueColorGetGreen (fg),
-					     gdTrueColorGetBlue (fg),
-	       gdAlphaMax - (gdTrueColorGetAlpha (fg) * pixel / NUMCOLORS));
-	}
+       {
+         /* 2.0.1: use gdImageSetPixel to do the alpha blending work,
+            or to just store the alpha level. All we have to do here
+            is incorporate our knowledge of the percentage of this
+            pixel that is really "lit" by pushing the alpha value
+            up toward transparency in edge regions. */
+         a->tweencolor = gdTrueColorAlpha (
+                                            gdTrueColorGetRed (fg),
+                                            gdTrueColorGetGreen (fg),
+                                            gdTrueColorGetBlue (fg),
+              gdAlphaMax - (gdTrueColorGetAlpha (fg) * pixel / NUMCOLORS));
+       }
       else
-	{
+       {
 	  a->tweencolor = gdImageColorResolve (im,
 		   (pixel * im->red[fg] + npixel * im->red[bg]) / NUMCOLORS,
 	       (pixel * im->green[fg] + npixel * im->green[bg]) / NUMCOLORS,
 		(pixel * im->blue[fg] + npixel * im->blue[bg]) / NUMCOLORS);
-	}
+    }
     }
   return (void *) a;
 }
@@ -535,7 +560,7 @@ gdft_draw_bitmap (gdCache_head_t *tc_cache, gdImage * im, int fg, FT_Bitmap bitm
 {
   unsigned char *pixel = NULL;
   int *tpixel = NULL;
-  int x, y, row, col, pc;
+  int x, y, row, col, pc, pcr;
 
   tweencolor_t *tc_elem;
   tweencolorkey_t tc_key;
@@ -543,11 +568,77 @@ gdft_draw_bitmap (gdCache_head_t *tc_cache, gdImage * im, int fg, FT_Bitmap bitm
   /* copy to image, mapping colors */
   tc_key.fgcolor = fg;
   tc_key.im = im;
+  /* Truecolor version; does not require the cache */
+  if (im->trueColor) 
+    {
+    for (row = 0; row < bitmap.rows; row++)
+      {
+        pc = row * bitmap.pitch;
+        pcr = pc;
+        y = pen_y + row;
+        /* clip if out of bounds */
+        if (y >= im->sy || y < 0)
+	  continue;
+      for (col = 0; col < bitmap.width; col++, pc++)
+	{
+          int level;  
+	  if (bitmap.pixel_mode == ft_pixel_mode_grays)
+	    {
+	      /*
+	       * Scale to 128 levels of alpha for gd use.
+               * alpha 0 is opacity, so be sure to invert at the end
+	       */
+	      level = (bitmap.buffer[pc] * gdAlphaMax /
+                (bitmap.num_grays - 1)); 
+            }
+          else if (bitmap.pixel_mode == ft_pixel_mode_mono)
+            {
+              /* 2.0.5: mode_mono fix from Giuliano Pochini */
+              level = ((bitmap.buffer[(col>>3)+pcr]) & (1<<(~col&0x07)))
+		? gdAlphaTransparent :
+                gdAlphaOpaque;
+            }  
+          else 
+	    {
+	      return "Unsupported ft_pixel_mode";
+	    }
+          if ((fg >= 0) && (im->trueColor)) {
+            /* Consider alpha in the foreground color itself to be an
+              upper bound on how opaque things get, when truecolor is
+              available. Without truecolor this results in far too many
+              color indexes. */ 
+            level = level * (gdAlphaMax - gdTrueColorGetAlpha(fg)) / gdAlphaMax;
+          }
+          level = gdAlphaMax - level;  
+          x = pen_x + col;
+	      /* clip if out of bounds */
+	      if (x >= im->sx || x < 0)
+		continue;
+	      /* get pixel location in gd buffer */
+	    tpixel = &im->tpixels[y][x];
+            if (fg < 0) {
+              if (level < (gdAlphaMax / 2)) {
+                *tpixel = -fg;
+              }
+            } else {
+              if (im->alphaBlendingFlag) { 
+                *tpixel = gdAlphaBlend(*tpixel, (level << 24) + (fg & 0xFFFFFF));
+              } else {    
+                *tpixel = (level << 24) + (fg & 0xFFFFFF);
+              }
+            }
+	  } 
+        }
+      return (char *) NULL;
+    }
+    /* Non-truecolor case, restored to its more or less original form */ 
   for (row = 0; row < bitmap.rows; row++)
     {
+      int pcr;
       pc = row * bitmap.pitch;
+      pcr = pc;
       if(bitmap.pixel_mode==ft_pixel_mode_mono)
-	      pc *= 8;    /* pc is measured in bits for monochrome images */
+             pc *= 8;    /* pc is measured in bits for monochrome images */
 
       y = pen_y + row;
 
@@ -572,61 +663,37 @@ gdft_draw_bitmap (gdCache_head_t *tc_cache, gdImage * im, int fg, FT_Bitmap bitm
 	    {
 	      tc_key.pixel = ((bitmap.buffer[pc / 8]
 			       << (pc % 8)) & 128) ? NUMCOLORS : 0;
+              /* 2.0.5: mode_mono fix from Giuliano Pochini */
+              tc_key.pixel = ((bitmap.buffer[(col>>3)+pcr]) & (1<<(~col&0x07)))
+		? NUMCOLORS : 0;
 	    }
 	  else
 	    {
 	      return "Unsupported ft_pixel_mode";
 	    }
-
-	  if (tc_key.pixel > 0)
-	    {			/* if not background */
+	  if (tc_key.pixel > 0) /* if not background */
+	    {			
 	      x = pen_x + col;
 
 	      /* clip if out of bounds */
 	      if (x >= im->sx || x < 0)
 		continue;
 	      /* get pixel location in gd buffer */
-	      if (im->trueColor)
-		{
-		  tpixel = &im->tpixels[y][x];
-		}
-	      else
-		{
-		  pixel = &im->pixels[y][x];
-		}
+	      pixel = &im->pixels[y][x];
 	      if (tc_key.pixel == NUMCOLORS)
 		{
-		  /* use fg color directly */
-		  if (im->trueColor)
-		    {
-		      *tpixel = abs( fg );
-		    }
-		  else
-		    {
-		      *pixel = abs( fg );
-		    }
+		  /* use fg color directly. gd 2.0.2: watch out for
+                     negative indexes (thanks to David Marwood). */ 
+		    *pixel = (fg < 0) ? -fg : fg;
 		}
 	      else
 		{
 		  /* find antialised color */
-		  if (im->trueColor)
-		    {
-		      tc_key.bgcolor = *tpixel;
-		    }
-		  else
-		    {
-		      tc_key.bgcolor = *pixel;
-		    }
+	
+		  tc_key.bgcolor = *pixel;
 		  tc_elem = (tweencolor_t *) gdCacheGet (
 							  tc_cache, &tc_key);
-		  if (im->trueColor)
-		    {
-		      *tpixel = tc_elem->tweencolor;
-		    }
-		  else
-		    {
-		      *pixel = tc_elem->tweencolor;
-		    }
+		  *pixel = tc_elem->tweencolor;
 		}
 	    }
 	}
@@ -642,15 +709,38 @@ gdroundupdown (FT_F26Dot6 v1, int updown)
     : (v1 > 0 ? ((v1 + 63) >> 6) : v1 >> 6);
 }
 
-
 extern int any2eucjp (char *, char *, unsigned int);
 
+/* Persistent font cache until explicitly cleared */
+/*     Fonts can be used across multiple images */
+static gdCache_head_t *fontCache;
+static FT_Library library;
+
+void
+gdFreeFontCache()
+{
+  if (fontCache)
+    {
+      gdCacheDelete(fontCache);
+      FT_Done_FreeType(library);
+    }
+}
+
 /********************************************************************/
 /* gdImageStringFT -  render a utf8 string onto a gd image          */
 
 char *
 gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 		 double ptsize, double angle, int x, int y, char *string)
+{
+	return gdImageStringFTEx(im, brect, fg, fontlist,
+		ptsize, angle, x, y, string, 0);
+}
+
+char *
+gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
+		 double ptsize, double angle, int x, int y, char *string,
+		gdFTStringExtraPtr strex)
 {
   FT_BBox bbox, glyph_bbox;
   FT_Matrix matrix;
@@ -672,29 +762,33 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
   int render = (im && (im->trueColor || (fg <= 255 && fg >= -255)));
   FT_BitmapGlyph bm;
   int render_mode = FT_LOAD_RENDER | FT_LOAD_FORCE_AUTOHINT;
-
-
-/***** initialize font library and font cache on first call ******/
-  static gdCache_head_t *fontCache;
-  static FT_Library library;
-
-  /* 
-   *   make a new tweenColorCache on every call 
+  /* Now tuneable thanks to Wez Furlong */
+  double linespace = LINESPACE;
+  /* 2.0.6: put this declaration with the other declarations! */
+  /*
+   *   make a new tweenColorCache on every call
    *   because caching colormappings between calls
-   *   is not safe. If the im-pointer points to a 
+   *   is not safe. If the im-pointer points to a
    *   brand new image, the cache gives out bogus
    *   colorindexes.          -- 27.06.2001 <krisku@arrak.fi>
    */
-  gdCache_head_t  *tc_cache; 
+  gdCache_head_t  *tc_cache;
 
+  if (strex) {
+    if ((strex->flags & gdFTEX_LINESPACE) == gdFTEX_LINESPACE) {
+      linespace = strex->linespacing;
+    }
+  }
   tc_cache = gdCacheCreate( TWEENCOLORCACHESIZE,
-		tweenColorTest, tweenColorFetch, tweenColorRelease );
+               tweenColorTest, tweenColorFetch, tweenColorRelease );
+
+/***** initialize font library and font cache on first call ******/
 
   if (!fontCache)
     {
       if (FT_Init_FreeType (&library))
 	{
-          gdCacheDelete( tc_cache );
+	  gdCacheDelete( tc_cache );
 	  return "Failure to initialize font library";
 	}
       fontCache = gdCacheCreate (FONTCACHESIZE,
@@ -729,6 +823,7 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
   penf.x = penf.y = 0;		/* running position of non-rotated string */
   pen.x = pen.y = 0;		/* running position of rotated string */
   bbox.xMin = bbox.xMax = bbox.yMin = bbox.yMax = 0;
+
   use_kerning = FT_HAS_KERNING (face);
   previous = 0;
   if (fg < 0)
@@ -774,7 +869,7 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
       /* newlines */
       if (ch == '\n')
 	{
-	  penf.y -= face->size->metrics.height * LINESPACE;
+	  penf.y -= face->size->metrics.height * linespace;
 	  penf.y = (penf.y - 32) & -64;		/* round to next pixel row */
 	  x1 = (penf.x * cos_a - penf.y * sin_a + 32) / 64;
 	  y1 = (penf.x * sin_a + penf.y * cos_a + 32) / 64;
@@ -840,10 +935,8 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 	      next++;
 	    }
 	}
-
       /* set rotation transform */
       FT_Set_Transform(face, &matrix, NULL);
-
       /* Convert character code to glyph index */
       glyph_index = FT_Get_Char_Index (face, ch);
 
@@ -859,8 +952,8 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
       err = FT_Load_Glyph (face, glyph_index, render_mode);
       if (err)
 	{
-          gdCacheDelete( tc_cache );
-	  return "Problem loading glyph";
+	  gdCacheDelete( tc_cache );
+	return "Problem loading glyph";
 	}
 
       /* transform glyph image */
@@ -872,24 +965,26 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 	  glyph_bbox.yMin += penf.y;
 	  glyph_bbox.xMax += penf.x;
 	  glyph_bbox.yMax += penf.y;
-	  if (!i)
-	    {			/* if first character, init BB corner values */
-	      bbox.xMin = glyph_bbox.xMin;
-	      bbox.yMin = glyph_bbox.yMin;
-	      bbox.xMax = glyph_bbox.xMax;
-	      bbox.yMax = glyph_bbox.yMax;
-	    }
-	  else
-	    {
-	      if (bbox.xMin > glyph_bbox.xMin)
+	  if (ch == ' ')   /* special case for trailing space */
+             glyph_bbox.xMax += slot->metrics.horiAdvance;
+          if (!i)
+	     {                   /* if first character, init BB corner values */
 	        bbox.xMin = glyph_bbox.xMin;
-	      if (bbox.yMin > glyph_bbox.yMin)
-	        bbox.yMin = glyph_bbox.yMin;
-	      if (bbox.xMax < glyph_bbox.xMax)
-	        bbox.xMax = glyph_bbox.xMax;
-	      if (bbox.yMax < glyph_bbox.yMax)
-	        bbox.yMax = glyph_bbox.yMax;
-	    }
+		bbox.yMin = glyph_bbox.yMin;
+		bbox.xMax = glyph_bbox.xMax;
+		bbox.yMax = glyph_bbox.yMax;
+	     }
+	   else
+	     {
+	  if (bbox.xMin > glyph_bbox.xMin)
+	    bbox.xMin = glyph_bbox.xMin;
+	  if (bbox.yMin > glyph_bbox.yMin)
+	    bbox.yMin = glyph_bbox.yMin;
+	  if (bbox.xMax < glyph_bbox.xMax)
+	    bbox.xMax = glyph_bbox.xMax;
+	  if (bbox.yMax < glyph_bbox.yMax)
+	    bbox.yMax = glyph_bbox.yMax;
+	     }
 	  i++;
 	}
 
@@ -899,10 +994,10 @@ gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 	    {
 	      err = FT_Glyph_To_Bitmap (&image, ft_render_mode_normal, 0, 1);
 	      if (err)
-	        {
-        	  gdCacheDelete( tc_cache );
-		  return "Problem rendering glyph";
-	        }
+		{
+		  gdCacheDelete( tc_cache );
+		return "Problem rendering glyph";
+	    }
 	    }
 
 	  /* now, draw to our target surface */
