@@ -7,7 +7,7 @@
  * Date:		  Fri Aug 19 1994
  * Copyright (c) 1994, University of Michigan
  *
- * Modifications by John Ellson (ellson@lucent.com):
+ * Modifications by John Ellson (ellson@graphviz.org):
  *     - renamed gdtclft.c to match module
  *     - conversion to freetype text rendering
  *     - support for tcl8.0 Tcl_Objects
@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "gvconfig.h"
+#include "config.h"
 #endif
 
 #include <stdio.h>
@@ -119,8 +119,12 @@ static cmdOptions subcmdVec[] = {
 			"gdhandle filehandle"},
 #endif
 #endif
+#ifdef HAVE_LIBPNG
+#ifdef HAVE_LIBZ
 	{"writePNGvar",		tclGdWriteBufCmd,			2, 2,	0, 1,
 			"gdhandle var"},
+#endif
+#endif
 
 	{"interlace",		tclGdInterlaceCmd,		1, 2,	0, 1,
 		 "gdhandle ?on-off?"},
@@ -1388,6 +1392,8 @@ BufferSinkFunc(void *context, const char *buffer, int len)
 static int
 tclGdWriteBufCmd(Tcl_Interp *interp, GdData *gdData, int argc, Tcl_Obj *CONST objv[])
 {
+#ifdef HAVE_LIBPNG
+#ifdef HAVE_LIBZ
 	gdImagePtr im;
 	Tcl_Obj *output;
 	/* char *cmd; */
@@ -1422,5 +1428,7 @@ tclGdWriteBufCmd(Tcl_Interp *interp, GdData *gdData, int argc, Tcl_Obj *CONST ob
 	if (Tcl_ObjSetVar2(interp, objv[3], NULL, output, 0) == NULL)
 	    return TCL_ERROR;
 	else
+#endif
+#endif
 	    return TCL_OK;
 }
