@@ -30,32 +30,35 @@ int Gbuttonsdown;
 int Gerrflag;
 int Gerrno;
 
+char *texts;
+int textn;
+
 static long wsizes[G_WTYPESIZE];
 static Gtextline_t tlarray[1000];
 
 Gwattrmap_t Gwattrmap[] = {
-    G_ATTRORIGIN,      G_ATTRTYPEPOINT, "origin",
-    G_ATTRSIZE,        G_ATTRTYPESIZE,  "size",
-    G_ATTRBORDERWIDTH, G_ATTRTYPEINT,   "borderwidth",
-    G_ATTRNAME,        G_ATTRTYPETEXT,  "name",
-    G_ATTRTEXT,        G_ATTRTYPETEXT,  "text",
-    G_ATTRAPPENDTEXT,  G_ATTRTYPETEXT,  "appendtext",
-    G_ATTRSELECTION,   G_ATTRTYPETEXT,  "selection",
-    G_ATTRCURSOR,      G_ATTRTYPETEXT,  "cursor",
-    G_ATTRMODE,        G_ATTRTYPETEXT,  "mode",
-    G_ATTRLAYOUT,      G_ATTRTYPETEXT,  "layout",
-    G_ATTRZORDER,      G_ATTRTYPETEXT,  "zorder",
-    G_ATTRCOLOR,       G_ATTRTYPECOLOR, "color",
-    G_ATTRVIEWPORT,    G_ATTRTYPESIZE,  "viewport",
-    G_ATTRWINDOW,      G_ATTRTYPERECT,  "window",
-    G_ATTRWINDOWID,    G_ATTRTYPETEXT,  "windowid",
-    G_ATTRCHILDCENTER, G_ATTRTYPEPOINT, "childcenter",
-    G_ATTRNEWLINECB,   G_ATTRTYPEFUNC,  "newlinecb",
-    G_ATTRRESIZECB,    G_ATTRTYPEFUNC,  "resizecb",
-    G_ATTRBUTTONCB,    G_ATTRTYPEFUNC,  "buttoncb",
-    G_ATTREVENTCB,     G_ATTRTYPEFUNC,  "eventcb",
-    G_ATTRUSERDATA,    G_ATTRTYPEFUNC,  "userdata",
-    -1,                -1,              NULL,
+    { G_ATTRORIGIN,      G_ATTRTYPEPOINT, "origin",      },
+    { G_ATTRSIZE,        G_ATTRTYPESIZE,  "size",        },
+    { G_ATTRBORDERWIDTH, G_ATTRTYPEINT,   "borderwidth", },
+    { G_ATTRNAME,        G_ATTRTYPETEXT,  "name",        },
+    { G_ATTRTEXT,        G_ATTRTYPETEXT,  "text",        },
+    { G_ATTRAPPENDTEXT,  G_ATTRTYPETEXT,  "appendtext",  },
+    { G_ATTRSELECTION,   G_ATTRTYPETEXT,  "selection",   },
+    { G_ATTRCURSOR,      G_ATTRTYPETEXT,  "cursor",      },
+    { G_ATTRMODE,        G_ATTRTYPETEXT,  "mode",        },
+    { G_ATTRLAYOUT,      G_ATTRTYPETEXT,  "layout",      },
+    { G_ATTRZORDER,      G_ATTRTYPETEXT,  "zorder",      },
+    { G_ATTRCOLOR,       G_ATTRTYPECOLOR, "color",       },
+    { G_ATTRVIEWPORT,    G_ATTRTYPESIZE,  "viewport",    },
+    { G_ATTRWINDOW,      G_ATTRTYPERECT,  "window",      },
+    { G_ATTRWINDOWID,    G_ATTRTYPETEXT,  "windowid",    },
+    { G_ATTRCHILDCENTER, G_ATTRTYPEPOINT, "childcenter", },
+    { G_ATTRNEWLINECB,   G_ATTRTYPEFUNC,  "newlinecb",   },
+    { G_ATTRRESIZECB,    G_ATTRTYPEFUNC,  "resizecb",    },
+    { G_ATTRBUTTONCB,    G_ATTRTYPEFUNC,  "buttoncb",    },
+    { G_ATTREVENTCB,     G_ATTRTYPEFUNC,  "eventcb",     },
+    { G_ATTRUSERDATA,    G_ATTRTYPEFUNC,  "userdata",    },
+    { -1,                -1,              NULL,          },
 };
 
 static int arrayattr[] = {
@@ -236,6 +239,9 @@ int Ginit (void) {
     Gbuttonsdown = 0;
     Gdefaultfont = "";
     Gerrflag = FALSE;
+    textn = 100;
+    texts = Marrayalloc ((long) textn);
+    texts[0] = '\0';
     Ginitgraphics ();
     return 0;
 }
@@ -247,6 +253,7 @@ int Gterm (void) {
         if (Gwidgets[wi].inuse)
             Gdestroywidget (wi);
     Gtermgraphics ();
+    Marrayfree (texts), texts = NULL, textn = 0;
     Marrayfree (Gwidgets), Gwidgets = NULL, Gwidgetn = 0;
     Marrayfree (Gbitmaps), Gbitmaps = NULL, Gbitmapn = 0;
     return 0;
@@ -843,21 +850,31 @@ void Gerr (char *file, int line, int errnum, ...) {
 }
 
 static int unpackstring (char *s) {
-    char *p;
+    char *p1, *p2;
     int n;
 
+    if ((n = strlen (s) + 1) > textn)
+        textn = n, texts = Marraygrow (texts, (long) textn);
+    strcpy (texts, s);
     n = 0;
-    p = s;
-    tlarray[0].p = p;
-    while (*p) {
-        if (p[0] == '\\' && (p[1] == 'n' || p[1] == 'l' || p[1] == 'r')) {
-            tlarray[n].n = p - tlarray[n].p;
-            tlarray[n].j = p[1];
-            tlarray[++n].p = (p += 2);
+    p1 = p2 = texts;
+    tlarray[0].p = p1;
+    while (*p1) {
+        if (p1[0] == '\\') {
+            if (p1[1] == 'n' || p1[1] == 'l' || p1[1] == 'r') {
+                tlarray[n].n = p1 - tlarray[n].p;
+                tlarray[n].j = p1[1];
+                tlarray[++n].p = (p1 += 2);
+            } else {
+                for (p2 = p1 + 1; *p2; p2++)
+                    p2[-1] = p2[0];
+                p2[-1] = 0;
+                p1++;
+            }
         } else
-            p++;
+            p1++;
     }
-    if ((tlarray[n].n = p - tlarray[n].p) > 0)
+    if ((tlarray[n].n = p1 - tlarray[n].p) > 0)
         tlarray[n++].j = 'n';
     return n;
 }
