@@ -178,9 +178,10 @@ void GFXprune (void) {
 /* callback for g.c module - calls the appropriate lefty function */
 void GFXlabelcb (Gevent_t *evp) {
     Tobj fo, to, co, wo;
-    char *fn;
+    char *fn; 
     char s[2];
     long fm;
+    fn = 0;
 
     wo = Tfindi (rootwo, evp->wi);
     switch (evp->type) {
@@ -237,13 +238,14 @@ void GFXviewcb (Gevent_t *evp) {
 
 /* callback for g.c module - calls the appropriate lefty function */
 void GFXevent (Gevent_t *evp) {
-    Tobj vo, pvo, fo, to, po, co, wo;
+    Tobj vo, pvo=NULL, fo, to, po, co, wo;
     gfxnode_t *np;
     Gpoint_t pp;
     char *fn;
     char s[2];
     long fm;
     int ni;
+    fn = 0;
 
     ni = Gwidgets[evp->wi].udata;
     np = &gfxnodes[ni];
@@ -437,7 +439,7 @@ void GFXbuttoncb (int wi, void *data) {
 }
 
 void GFXarrayresizecb (int wi, Gawdata_t *dp) {
-    Tobj wo, fo, co, so, to, lrtno, sxo, syo;
+    Tobj wo, fo, co, so, to, lrtno=NULL, sxo, syo;
     Tkvindex_t tkvi;
     Gawcarray_t *cp;
     int sx, sy, csx, csy, ci;
@@ -700,6 +702,8 @@ int GFXgetwidgetattr (int argc, lvar_t *argv) {
             }
             wattri2 += (G_MAXCOLORS - 1);
             break;
+	default:
+	    break;
         }
     }
     Mpopmark (rtnm);
@@ -766,9 +770,9 @@ int GFXsetgfxattr (int argc, lvar_t *argv) {
 int GFXgetgfxattr (int argc, lvar_t *argv) {
     Tkvindex_t tkvi;
     Ggattr_t gattr;
-    long rtnm;
+    long rtnm = 0;
     int wi;
-    char *s;
+    char *s = 0;
 
     gattr.flags = 0;
     if (getint (argv[0].o, &wi) == -1 || !ISAWIDGET (wi) ||
@@ -1223,6 +1227,8 @@ static int getwattr (Tobj ao, int *type) {
                 }
             }
             break;
+	default:
+	    break;
         }
     }
     return wattri;
@@ -1331,11 +1337,63 @@ static int getstr (Tobj so, char **s) {
     return -1;
 }
 
+/* scan a string for 3 floating point numbers separated by
+ * white-space or commas.
+ */
+static int scanhsv (char* strp, double* h, double* s, double* v) {
+    char* endp;
+
+    *h = strtod (strp, &endp);
+    if (endp == strp) return 0;
+    strp = endp;
+    while ((*strp == ',') || isspace(*strp)) strp++;
+    *s = strtod (strp, &endp);
+    if (endp == strp) return 0;
+    strp = endp;
+    while ((*strp == ',') || isspace(*strp)) strp++;
+    *v = strtod (strp, &endp);
+    if (endp == strp) return 0;
+    return 1;
+}
+
+static void hsv2rgb (double h, double s, double v, Gcolor_t *cp) {
+    double r, g, b;
+    double f, p, q, t;
+    int i;
+
+      /* clip to reasonable values */
+    h = max(min(h,1.0),0.0);
+    s = max(min(s,1.0),0.0);
+    v = max(min(v,1.0),0.0);
+
+    r = g = b = v;
+    if (s != 0.0) {
+        if (h == 1.0) h = 0.0;
+        h = h * 6.0;
+        i = (int)h;
+        f = h - (double)i;
+        p = v * (1 - s);
+        q = v * (1 - (s * f));
+        t = v * (1 - (s * (1 - f)));
+        switch (i) {
+            case 0: r = v; g = t; b = p; break;
+            case 1: r = q; g = v; b = p; break;
+            case 2: r = p; g = v; b = t; break;
+            case 3: r = p; g = q; b = v; break;
+            case 4: r = t; g = p; b = v; break;
+            case 5: r = v; g = p; b = q; break;
+        }
+    }
+    cp->r = (int)(255.0 * r);
+    cp->g = (int)(255.0 * g);
+    cp->b = (int)(255.0 * b);
+}
+
 static int getcolor (Tobj ko, Tobj co, Gcolor_t *cp) {
     Tobj ro, go, bo, ho, so, vo;
     char *s, *s1, *s2;
-    float hc, sc, vc, fc, pc, qc, tc;
-    int cni, ic;
+    double hc, sc, vc;
+    int cni;
 
     if (ko && T_ISNUMBER (ko))
         cp->index = Tgetnumber (ko);
@@ -1344,7 +1402,20 @@ static int getcolor (Tobj ko, Tobj co, Gcolor_t *cp) {
     if (!co || !(T_ISSTRING (co) || T_ISTABLE (co)))
         return -1;
     if (T_ISSTRING (co)) {
+        int     r,g,b,a;
+
         s = Tgetstring (co);
+        while (*s == ' ') s++; /* skip over any leading spaces */
+        if ((*s == '#') && (sscanf(s,"#%2x%2x%2x%2x",&r,&g,&b,&a) >= 3)) {
+            cp->r = r;
+            cp->g = g;
+            cp->b = b;
+            return 0;
+        }
+        if ((isdigit(*s) || (*s == '.')) && scanhsv(s,&hc,&sc,&vc)) {
+            hsv2rgb(hc,sc,vc,cp);
+            return 0;
+        }
         for (cni = 0; cni < sizeof (colornames) / sizeof (colorname_t); cni++) {
             for (s1 = colornames[cni].name, s2 = s; *s1 && *s2; s1++, s2++) {
                 if (*s2 == ' ' || *s2 == '_')
@@ -1379,34 +1450,7 @@ static int getcolor (Tobj ko, Tobj co, Gcolor_t *cp) {
                 hc = Tgetnumber (ho);
                 sc = Tgetnumber (so);
                 vc = Tgetnumber (vo);
-                if (hc == 1)
-                    hc = 0;
-                hc = hc * 6;
-                ic = hc;
-                fc = hc - ic;
-                pc = vc * (1 - sc);
-                qc = vc * (1 - (2 * fc));
-                tc = vc * (1 - (sc * (1 - fc)));
-                switch (ic) {
-                case 0:
-                    cp->r = vc * 255, cp->g = tc * 255, cp->b = pc * 255;
-                    break;
-                case 1:
-                    cp->r = qc * 255, cp->g = vc * 255, cp->b = pc * 255;
-                    break;
-                case 2:
-                    cp->r = pc * 255, cp->g = vc * 255, cp->b = tc * 255;
-                    break;
-                case 3:
-                    cp->r = pc * 255, cp->g = qc * 255, cp->b = vc * 255;
-                    break;
-                case 4:
-                    cp->r = tc * 255, cp->g = pc * 255, cp->b = vc * 255;
-                    break;
-                case 5:
-                    cp->r = vc * 255, cp->g = pc * 255, cp->b = qc * 255;
-                    break;
-                }
+                hsv2rgb(hc,sc,vc,cp);
                 return 0;
             } else
                 return -1;
