@@ -340,6 +340,11 @@ int Isqrt (int argc, lvar_t *argv) {
     return L_SUCCESS;
 }
 
+/* the gnu way is to test for a feature explicitly */
+#ifndef HAVE_LRAND48
+#define lrand48 rand
+#endif
+/* the othe way is to presume a feature based on the OS */
 #if defined(FEATURE_MS) || defined(__Mac_OSX__)
 #define lrand48 rand
 #endif
@@ -480,10 +485,11 @@ int Iconcat (int argc, lvar_t *argv) {
 }
 
 int Iquote (int argc, lvar_t *argv) {
-    Tobj so, ao, qo;
+    Tobj so, ao=NULL, qo=NULL;
     char *s, *s1, *s2, *qs, *as;
     char buf2[50];
     int n, bufi;
+    s = 0;
 
     if ((Tgettype ((so = argv[0].o)) != T_STRING && !T_ISNUMBER (so)) ||
             (argc > 1 && Tgettype ((qo = argv[1].o)) != T_STRING) ||
