@@ -7,9 +7,18 @@
  * http://www.research.att.com/orgs/ssr/book/reuse
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+/* avoid compiler warnings with template changes in Tcl8.4 */
+/*    specifically just the change to Tcl_CmdProc */
+#define USE_NON_CONST
+
 #include "tcldot.h"
 #include "tclhandle.h"
-#include <gvconfig.h>
+#include "neatoprocs.h"
+#include "circle.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
@@ -18,7 +27,7 @@
 char	*Info[] = {
 	"tcldot",			/* Program */
 	VERSION,			/* Version */
-	DATE				/* Build Date */
+	BUILDDATE			/* Build Date */
 };
 
 /* ******* not ready yet
@@ -52,8 +61,8 @@ static tkgendata_t tkgendata;
 static void
 reset_layout(Agraph_t *g)
 {
-	if (g->root->u.drawing) { /* only cleanup once between layouts */
-		switch (g->root->u.drawing->engine) {
+	if (GD_drawing(g->root)) { /* only cleanup once between layouts */
+		switch (GD_drawing(g->root)->engine) {
 		case DOT:
 			dot_cleanup(g->root);
 			break;
@@ -177,7 +186,7 @@ edgecmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryattributes", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char ***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->e, argv2[j]))) {
@@ -194,7 +203,7 @@ edgecmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryattributevalues", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->e, argv2[j]))) {
@@ -213,7 +222,7 @@ edgecmd(ClientData clientData, Tcl_Interp *interp,
 	} else if ((c == 's') && (strncmp(argv[1], "setattributes", length) == 0)) {
 		g = g->root;
 		if (argc == 3) {
-			if (Tcl_SplitList(interp, argv[2], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[2], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			if ((argc2 == 0) || (argc2 % 2)) {
 				Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -320,11 +329,11 @@ nodecmd(ClientData clientData, Tcl_Interp *interp,
 		*ep = e = agedge(g, n, head);
 		e->handle = id;
 #ifndef TCLOBJ
-		Tcl_CreateCommand(interp, interp->result, 
-			edgecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateCommand(interp, interp->result, edgecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-		Tcl_CreateObjCommand(interp, interp->result, 
-			edgecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateObjCommand(interp, interp->result, edgecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 		g = g->root;
 		for (i = 3; i < argc; i++) {
@@ -396,7 +405,7 @@ nodecmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryattributes", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->n, argv2[j]))) {
@@ -413,7 +422,7 @@ nodecmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryattributevalues", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->n, argv2[j]))) {
@@ -432,7 +441,7 @@ nodecmd(ClientData clientData, Tcl_Interp *interp,
 	} else if ((c == 's') && (strncmp(argv[1], "setattributes", length) == 0)) {
 		g = g->root;
 		if (argc == 3) {
-			if (Tcl_SplitList(interp, argv[2], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[2], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			if ((argc2 == 0) || (argc2 % 2)) {
 				Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -504,14 +513,14 @@ tcldot_layout(Agraph_t *g, char *engine)
 /* set bb attribute for basic layout.
  * doesn't yet include margins, scaling or page sizes because
  * those depend on the renderer being used. */
-	if (g->u.drawing->landscape)
+	if (GD_drawing(g)->landscape)
 		sprintf(buf,"%d %d %d %d",
-			ROUND(g->u.bb.LL.y -3), ROUND(g->u.bb.LL.x -2),
-			ROUND(g->u.bb.UR.y +4), ROUND(g->u.bb.UR.x +4));
+			ROUND(GD_bb(g).LL.y), ROUND(GD_bb(g).LL.x),
+			ROUND(GD_bb(g).UR.y), ROUND(GD_bb(g).UR.x));
 	else
 		sprintf(buf,"%d %d %d %d",
-			ROUND(g->u.bb.LL.x -1), ROUND(g->u.bb.LL.y +3),
-			ROUND(g->u.bb.UR.x +4), ROUND(g->u.bb.UR.y +6));
+			ROUND(GD_bb(g).LL.x), ROUND(GD_bb(g).LL.y),
+			ROUND(GD_bb(g).UR.x), ROUND(GD_bb(g).UR.y));
 	if (!(a = agfindattr(g, "bb"))) {
 		a = agraphattr(g, "bb", "");
 	}
@@ -588,12 +597,13 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 		*ep = e = agedge(g, tail, head);
 		e->handle = id;
 #ifndef TCLOBJ
-		Tcl_CreateCommand(interp, interp->result, 
-			edgecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateCommand(interp, interp->result, edgecmd,
+		       	(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-		Tcl_CreateObjCommand(interp, interp->result, 
-			edgecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateObjCommand(interp, interp->result, edgecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
+		g = g->root;
 		for (i = 4; i < argc; i++) {
 			if (!(a = agfindattr(g->proto->e, argv[i])))
 				a = agedgeattr(g, argv[i], "");
@@ -616,11 +626,11 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 		*np = n;
 		n->handle = id;
 #ifndef TCLOBJ
-		Tcl_CreateCommand(interp, interp->result, 
-			nodecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateCommand(interp, interp->result, nodecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-		Tcl_CreateObjCommand(interp, interp->result, 
-			nodecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateObjCommand(interp, interp->result, nodecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 		g = g->root;
 		for (i = j; i < argc; i++) {
@@ -650,11 +660,11 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 		*sgp = sg;
 		sg->handle = id;
 #ifndef TCLOBJ
-		Tcl_CreateCommand(interp, interp->result, 
-			graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateCommand(interp, interp->result, graphcmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-		Tcl_CreateObjCommand(interp, interp->result, 
-			graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateObjCommand(interp, interp->result, graphcmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 		g = g->root;
 		for (i = j; i < argc; i++) {
@@ -732,12 +742,12 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 		return TCL_OK;
 
 	} else if ((c == 'l') && (strncmp(argv[1], "layoutedges", length) == 0)) {
-		if (!g->root->u.drawing) 
+		if (!GD_drawing(g->root)) 
 			tcldot_layout(g->root,(argc>2)?argv[2]:(char *) NULL);
 		return TCL_OK;
 
 	} else if ((c == 'l') && (strncmp(argv[1], "layoutnodes", length) == 0)) {
-		if (!g->root->u.drawing)
+		if (!GD_drawing(g->root))
 			tcldot_layout(g->root,(argc>2)?argv[2]:(char *) NULL);
 		return TCL_OK;
 
@@ -780,9 +790,9 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'l') && (strncmp(argv[1], "listsubgraphs", length) == 0)) {
 	    if (g->meta_node) {
-			for (n = agfstnode(g->meta_node->graph); n; n = agnxtnode(g->meta_node->graph, n)) {
-			    sg = agusergraph(n);
-				tclhandleString(graphTblPtr, buf, sg->handle);
+			for (e = agfstout(g->meta_node->graph, g->meta_node); e; e = agnxtout(g->meta_node->graph, e)) {
+			    sg = agusergraph(e->head);
+			    tclhandleString(graphTblPtr, buf, sg->handle);
 			    Tcl_AppendElement(interp, buf);
 			}
 	    }
@@ -790,7 +800,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryattributes", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->root, argv2[j]))) {
@@ -807,7 +817,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryattributevalues", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->root, argv2[j]))) {
@@ -825,7 +835,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryedgeattributes", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->e, argv2[j]))) {
@@ -842,7 +852,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "queryedgeattributevalues", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->e, argv2[j]))) {
@@ -860,7 +870,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "querynodeattributes", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->n, argv2[j]))) {
@@ -877,7 +887,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 'q') && (strncmp(argv[1], "querynodeattributevalues", length) == 0)) {
 		for (i = 2; i < argc; i++) {
-			if (Tcl_SplitList(interp, argv[i], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[i], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			for (j = 0; j < argc2; j++) {
 				if ((a = agfindattr(g->proto->n, argv2[j]))) {
@@ -916,7 +926,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 		g = g->root;
 
 		/* make sure that layout is done */
-		if (!g->u.drawing)
+		if (!GD_drawing(g))
 			tcldot_layout(g->root,(argc>5)?argv[5]:(char *) NULL);
 
 		/* render graph TK canvas commands */
@@ -941,9 +951,12 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 		g = g->root;
 
 		/* make sure that layout is done */
-		if (!g->u.drawing)
+		if (!GD_drawing(g))
 			tcldot_layout(g->root,(argc>4)?argv[4]:(char *) NULL);
 
+		/* set a default bitmap output language (to get default margins) */
+		Output_lang = memGD;
+		dotneato_set_margins(g);
 		/* render graph to open GD structure */
 		CodeGen = &memGD_CodeGen;
 		/* emit graph in sorted order, all nodes then all edges */
@@ -953,7 +966,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 	} else if ((c == 's') && (strncmp(argv[1], "setattributes", length) == 0)) {
 		g = g->root;
 		if (argc == 3) {
-			if (Tcl_SplitList(interp, argv[2], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[2], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			if ((argc2 == 0) || (argc2 % 2)) {
 				Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -987,7 +1000,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 's') && (strncmp(argv[1], "setedgeattributes", length) == 0)) {
 		if (argc == 3) {
-			if (Tcl_SplitList(interp, argv[2], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[2], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			if ((argc2 == 0) || (argc2 % 2)) {
 				Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -1018,7 +1031,7 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 
 	} else if ((c == 's') && (strncmp(argv[1], "setnodeattributes", length) == 0)) {
 		if (argc == 3) {
-			if (Tcl_SplitList(interp, argv[2], &argc2, &argv2) != TCL_OK)
+			if (Tcl_SplitList(interp, argv[2], &argc2, (CONST84 char***) &argv2) != TCL_OK)
 				return TCL_ERROR;
 			if ((argc2 == 0) || (argc2 % 2)) {
 				Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -1061,18 +1074,17 @@ graphcmd(ClientData clientData, Tcl_Interp *interp,
 			return TCL_ERROR;
 		if (argc < 4) {
 			Output_lang = ATTRIBUTED_DOT;
-	    } else {
+	        } else {
 			Output_lang = lang_select(argv[3]);
 		}
 		/* convert attempts to write subgraphs to writes of the main graph */
 		g = g->root;
 
 		/* make sure that layout is done  - unless canonical output */
-		if (!g->u.drawing && Output_lang != CANONICAL_DOT)
+		if (!GD_drawing(g) && Output_lang != CANONICAL_DOT)
 			tcldot_layout(g->root,(argc>5)?argv[5]:(char *) NULL);
 
-		emit_reset(g);		/* reset page numbers in postscript */
-		dotneato_write(g);
+		dotneato_write_one(g);
 		return TCL_OK;
 
 	} else {
@@ -1138,20 +1150,20 @@ dotnew(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
 	*gp = g;
 	g->handle = id;
 #ifndef TCLOBJ
-	Tcl_CreateCommand(interp, interp->result, 
-		graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateCommand(interp, interp->result, graphcmd,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-	Tcl_CreateObjCommand(interp, interp->result, 
-		graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateObjCommand(interp, interp->result, graphcmd,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 	for (i = j; i < argc; i++) {
 	    if (!(a = agfindattr(g, argv[i])))
 			a = agraphattr(g, argv[i], "");
 		agxset(g, a->index, argv[++i]);
 	}
-	/* we use g->u.drawing as a flag that layout has been done.
+	/* we use GD_drawing(g) as a flag that layout has been done.
 	 * so we make sure that it is initialized to "not done" */
-	g->u.drawing=NULL;
+	GD_drawing(g)=NULL;
 
 	return TCL_OK;
 }
@@ -1176,11 +1188,11 @@ tcldot_fixup(Tcl_Interp *interp, Agraph_t *g) {
 			*sgp = sg;
 			sg->handle = id;
 #ifndef TCLOBJ
-			Tcl_CreateCommand(interp, buf, 
-				graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+			Tcl_CreateCommand(interp, buf, graphcmd,
+				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-			Tcl_CreateObjCommand(interp, buf, 
-				graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+			Tcl_CreateObjCommand(interp, buf, graphcmd,
+				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 			if (sg == g) strcpy (interp->result, buf);
 		}
@@ -1189,11 +1201,11 @@ tcldot_fixup(Tcl_Interp *interp, Agraph_t *g) {
 	    *gp = g;
 		g->handle = id;
 #ifndef TCLOBJ
-	    Tcl_CreateCommand(interp, interp->result, 
-			graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	    Tcl_CreateCommand(interp, interp->result, graphcmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-	    Tcl_CreateObjCommand(interp, interp->result, 
-			graphcmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	    Tcl_CreateObjCommand(interp, interp->result, graphcmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 	}
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -1201,22 +1213,22 @@ tcldot_fixup(Tcl_Interp *interp, Agraph_t *g) {
 		*np = n;
 		n->handle = id;
 #ifndef TCLOBJ
-		Tcl_CreateCommand(interp, buf, 
-			nodecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateCommand(interp, buf, nodecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-		Tcl_CreateObjCommand(interp, buf, 
-			nodecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+		Tcl_CreateObjCommand(interp, buf, nodecmd,
+			(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 		for (e = agfstedge(g, n); e; e = agnxtedge(g, e, n)) {
 			ep = (Agedge_t **) tclhandleAlloc (edgeTblPtr, buf, &id);
 			*ep = e;
 			e->handle = id;
 #ifndef TCLOBJ
-			Tcl_CreateCommand(interp, buf, 
-				edgecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+			Tcl_CreateCommand(interp, buf, edgecmd,
+				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-			Tcl_CreateObjCommand(interp, buf, 
-				edgecmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+			Tcl_CreateObjCommand(interp, buf, edgecmd,
+				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 		}
 	}
@@ -1359,9 +1371,9 @@ dotread(ClientData clientData, Tcl_Interp *interp,
 			argv[1], " \"", (char *) NULL);
 		return TCL_ERROR;
 	}
-	/* we use g->u.drawing as a flag that layout has been done.
+	/* we use GD_drawing(g) as a flag that layout has been done.
 	 * so we make sure that it is initialized to "not done" */
-	g->u.drawing=NULL;
+	GD_drawing(g)=NULL;
 
 	return (tcldot_fixup(interp, g));
 }
@@ -1395,9 +1407,9 @@ dotstring(ClientData clientData, Tcl_Interp *interp,
 			argv[1], " \"", (char *) NULL);
 		return TCL_ERROR;
 	}
-	/* we use g->u.drawing as a flag that layout has been done.
+	/* we use GD_drawing(g) as a flag that layout has been done.
 	 * so we make sure that it is initialized to "not done" */
-	g->u.drawing=NULL;
+	GD_drawing(g)=NULL;
 
 	return (tcldot_fixup(interp, g));
 }
@@ -1428,19 +1440,19 @@ Tcldot_Init(Tcl_Interp *interp)
 	agnodeattr(NULL, "label", NODENAME_ESC);
 
 #ifndef TCLOBJ
-	Tcl_CreateCommand(interp, "dotnew", 
-		dotnew, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
-	Tcl_CreateCommand(interp, "dotread", 
-		dotread, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
-	Tcl_CreateCommand(interp, "dotstring", 
-		dotstring, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateCommand(interp, "dotnew", dotnew,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateCommand(interp, "dotread", dotread,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateCommand(interp, "dotstring", dotstring,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #else /* TCLOBJ */
-	Tcl_CreateObjCommand(interp, "dotnew", 
-		dotnew, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
-	Tcl_CreateObjCommand(interp, "dotread", 
-		dotread, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
-	Tcl_CreateObjCommand(interp, "dotstring", 
-		dotstring, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateObjCommand(interp, "dotnew", dotnew,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateObjCommand(interp, "dotread", dotread,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+	Tcl_CreateObjCommand(interp, "dotstring", dotstring,
+		(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 #endif /* TCLOBJ */
 
 	graphTblPtr = (void *) tclhandleInit("graph", sizeof(Agraph_t *), 10);
