@@ -18,10 +18,10 @@ pathcat(char* path, register const char* dirs, int sep, const char* a, register
 	if (s != path) *s++ = '/';
 	if (a)
 	{
-		while (*s = *a++) s++;
+		while ((*s = *a++)) s++;
 		if (b) *s++ = '/';
 	}
 	else if (!b) b = ".";
-	if (b) while (*s++ = *b++);
+	if (b) while ((*s++ = *b++));
 	return(*dirs ? (char*)++dirs : 0);
 }
