@@ -49,7 +49,7 @@ excontext(Expr_t* p, char* buf, int n)
 			*s++ = *t++;
 		p->linep = p->line;
 		t = "<<< ";
-		while (*s = *t++)
+		while ((*s = *t++))
 			s++;
 	}
 	*s = 0;
