@@ -6,6 +6,10 @@
  * expression library evaluator
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "exlib.h"
 #include "exop.h"
 #include <string.h>
@@ -113,7 +117,7 @@ format(Sfio_t* sp, void* vp, Sffmt_t* dp)
 	register char*		txt;
 	int			n;
 	int			from;
-	int			to;
+	int			to = 0;
 	/* time_t			tm; */
 
 	static char		nmbuf[64];
@@ -122,7 +126,7 @@ format(Sfio_t* sp, void* vp, Sffmt_t* dp)
 	dp->flags |= SFFMT_VALUE;
 	if (fmt->args)
 	{
-		if (node = (dp->fmt == '*') ? fmt->args->param[dp->size] : fmt->args->arg)
+		if ((node = (dp->fmt == '*') ? fmt->args->param[dp->size] : fmt->args->arg))
 			fmt->value = exeval(fmt->expr, node, fmt->env);
 		else
 			fmt->value.integer = 0;
@@ -275,7 +279,10 @@ print(Expr_t* ex, Exnode_t* expr, void* env, Sfio_t* sp)
 	if (!sp)
 	{
 		v = eval(ex, expr->data.print.descriptor, env);
-		if (v.integer < 0 || v.integer >= elementsof(ex->file) || !(sp = ex->file[v.integer]) && !(sp = ex->file[v.integer] = sfnew(NiL, NiL, SF_UNBOUND, v.integer, SF_READ|SF_WRITE)))
+		if (v.integer < 0
+			|| v.integer >= elementsof(ex->file)
+			|| ( !((sp = ex->file[v.integer]))
+				&& !((sp = ex->file[v.integer] = sfnew(NiL, NiL, SF_UNBOUND, v.integer, SF_READ|SF_WRITE)))))
 		{
 			exerror("printf: %d: invalid descriptor", v.integer);
 			return 0;
@@ -298,7 +305,7 @@ print(Expr_t* ex, Exnode_t* expr, void* env, Sfio_t* sp)
 			}
 			else
 				sfputr(sp, x->format, -1);
-		} while (x = x->next);
+		} while ((x = x->next));
 	else
 	{
 		v = eval(ex, x->arg->data.operand.left, env);
@@ -334,10 +341,10 @@ str_ior(Expr_t* ex, register char* l, register char* r)
 	register int	c;
 	register char*	s = l;
 
-	while (c = *s++)
+	while ((c = *s++))
 		if (!strchr(s, c))
 			sfputc(ex->tmp, c);
-	while (c = *r++)
+	while ((c = *r++))
 		if (!strchr(l, c) && !strchr(r, c))
 			sfputc(ex->tmp, c);
 	return vmstrdup(ex->vc, sfstruse(ex->tmp));
@@ -352,7 +359,7 @@ str_and(Expr_t* ex, register char* l, register char* r)
 {
 	register int	c;
 
-	while (c = *l++)
+	while ((c = *l++))
 		if (strchr(r, c) && !strchr(l, c))
 			sfputc(ex->tmp, c);
 	return vmstrdup(ex->vc, sfstruse(ex->tmp));
@@ -368,10 +375,10 @@ str_xor(Expr_t* ex, register char* l, register char* r)
 	register int	c;
 	register char*	s = l;
 
-	while (c = *s++)
+	while ((c = *s++))
 		if (!strchr(r, c) && !strchr(s, c))
 			sfputc(ex->tmp, c);
-	while (c = *r++)
+	while ((c = *r++))
 		if (!strchr(l, c) && !strchr(r, c))
 			sfputc(ex->tmp, c);
 	return vmstrdup(ex->vc, sfstruse(ex->tmp));
@@ -386,7 +393,7 @@ str_mod(Expr_t* ex, register char* l, register char* r)
 {
 	register int	c;
 
-	while (c = *l++)
+	while ((c = *l++))
 		if (!strchr(r, c) && !strchr(l, c))
 			sfputc(ex->tmp, c);
 	return vmstrdup(ex->vc, sfstruse(ex->tmp));
@@ -482,7 +489,7 @@ eval(Expr_t* ex, register Exnode_t* expr, void* env)
 			if (x->type == STRING)
 			{
 				v.string = vmstrdup(ex->vm, v.string);
-				if (e = assoc ? assoc->value.string : x->data.variable.symbol->value->data.constant.value.string)
+				if ((e = assoc ? assoc->value.string : x->data.variable.symbol->value->data.constant.value.string))
 					vmfree(ex->vm, e);
 			}
 			if (assoc)
@@ -556,7 +563,7 @@ eval(Expr_t* ex, register Exnode_t* expr, void* env)
 		x = expr->data.operand.right;
 		a = x->data.select.statement;
 		n = 0;
-		while (x = x->data.select.next)
+		while ((x = x->data.select.next))
 		{
 			if (!(t = x->data.select.constant))
 				n = 1;
@@ -614,7 +621,7 @@ eval(Expr_t* ex, register Exnode_t* expr, void* env)
 				if (n)
 					expr->data.generate.index->value->data.constant.value.string = assoc->name;
 				else
-					expr->data.generate.index->value->data.constant.value.integer = strtoll(assoc->name, NiL, 0);
+					expr->data.generate.index->value->data.constant.value.integer = strToL (assoc->name, 0);
 				eval(ex, expr->data.generate.statement, env);
 				if (ex->loopcount > 0 && (--ex->loopcount > 0 || ex->loopop != CONTINUE))
 				{
@@ -880,7 +887,7 @@ eval(Expr_t* ex, register Exnode_t* expr, void* env)
 		return v.integer ? v : eval(ex, expr->data.operand.right, env);
 	}
 	v = eval(ex, x, env);
-	if (x = expr->data.operand.right)
+	if ((x = expr->data.operand.right))
 		r = eval(ex, x, env);
 	switch (expr->data.operand.left->type)
 	{
@@ -1148,7 +1155,7 @@ eval(Expr_t* ex, register Exnode_t* expr, void* env)
 			tmp.data.constant.value = v;
 			if ((*ex->disc->convertf)(ex, &tmp, INTEGER, expr->data.operand.right ? expr->data.operand.right->data.variable.symbol : (Exid_t*)0, 0, ex->disc))
 			{
-				tmp.data.constant.value.integer = strtoll(v.string, &e, 0);
+				tmp.data.constant.value.integer = strToL(v.string, &e);
 				if (*e)
 					tmp.data.constant.value.integer = *v.string != 0;
 			}
@@ -1204,9 +1211,14 @@ eval(Expr_t* ex, register Exnode_t* expr, void* env)
 	}
  huh:
 	if (expr->binary)
-		exerror("operator %s %s %s not implemented", lexname(expr->data.operand.left->type, -1), lexname(expr->op, expr->subop), expr->data.operand.right ? lexname(expr->data.operand.right->type, -1) : "UNARY");
+		exerror("operator %s %s %s not implemented",
+			lexname(expr->data.operand.left->type, -1),
+			lexname(expr->op, expr->subop),
+			expr->data.operand.right ? lexname(expr->data.operand.right->type, -1) : "UNARY");
 	else
-		exerror("operator %s %s not implemented", lexname(expr->op, expr->subop), lexname(expr->data.operand.left->type, -1));
+		exerror("operator %s %s not implemented",
+			lexname(expr->op, expr->subop),
+			lexname(expr->data.operand.left->type, -1));
 	return exzero(expr->type);
 }
 
@@ -1247,3 +1259,19 @@ exeval(Expr_t* ex, Exnode_t* expr, void* env)
 	}
 	return v;
 }
+
+Sflong_t
+strToL (char* s, char** p)
+{
+  Sflong_t   v;
+  int        i;
+
+  v = 0;
+  if (p) {
+    sfsscanf(s, "%I*i%n", sizeof(v), &v, &i);
+    *p = s+i;
+  }
+  else
+    sfsscanf(s, "%I*i", sizeof(v), &v);
+  return v;
+}
