@@ -46,7 +46,7 @@ exopen(register Exdisc_t* disc)
 	if (!(disc->flags & EX_PURE))
 		for (sym = exbuiltin; *sym->name; sym++)
 			dtinsert(program->symbols, sym);
-	if (sym = disc->symbols)
+	if ((sym = disc->symbols))
 		for (; *sym->name; sym++)
 			dtinsert(program->symbols, sym);
 	return program;
