@@ -10,6 +10,13 @@
 #include <gdefs.h>
 
 #define BITS_PER_BYTE 8
+#ifdef HAVE_INTPTR_T
+#define INT2PTR(t,v) ((t)(intptr_t)(v))
+#define PTR2INT(v) ((Sflong_t)(intptr_t)(v))
+#else
+#define INT2PTR(t,v) ((t)(v))
+#define PTR2INT(v) ((Sflong_t)(v))
+#endif
 
 static char*
 symName (int op)
@@ -34,7 +41,7 @@ xargs (char* args)
   Agdesc_t desc = Agdirected;
   char     c;
   
-  while (c = *args++) {
+  while ((c = *args++)) {
     switch (c) {
     case 'u' :
     case 'U' :
@@ -86,7 +93,7 @@ deref (Exnode_t* x, Exref_t* ref, Agobj_t* objp, Gpr_t* state)
   if (ref == 0)
     return objp;
   else if (ref->symbol->lex == DYNAMIC) {
-    ptr = x->data.variable.dyna->data.variable.dyna->data.constant.value.user;
+    ptr = INT2PTR(void*,x->data.variable.dyna->data.variable.dyna->data.constant.value.integer);
     return deref (x, ref->next, (Agobj_t*)ptr, state);
   }
   else switch (ref->symbol->index) {    /* sym->lex == ID */
@@ -138,14 +145,14 @@ lookup (Agobj_t* objp, Exid_t* sym, Extype_t* v)
   if (sym->lex == ID) {
     switch (sym->index) {
     case M_head :
-      if (ISEDGE(objp)) v->user = AGHEAD((Agedge_t*)objp);
+      if (ISEDGE(objp)) v->integer = PTR2INT(AGHEAD((Agedge_t*)objp));
       else {
         error (1, "head of non-edge");
         return -1;
       }
       break;
     case M_tail :
-      if (ISEDGE(objp)) v->user = AGTAIL((Agedge_t*)objp);
+      if (ISEDGE(objp)) v->integer = PTR2INT(AGTAIL((Agedge_t*)objp));
       else {
         error (1, "tail of non-edge");
         return -1;
@@ -197,6 +204,10 @@ getval(Expr_t* pgm, Exnode_t* node, Exid_t* sym, Exref_t* ref,
   Gpr_t*          state;
   Extype_t*       args;
   Agobj_t*        objp;
+  char*           key;
+  Agraph_t*       gp;
+  Agnode_t*       np;
+  Agedge_t*       ep;
 
   assert (sym->lex != CONSTANT);
   if (elt == EX_CALL) {
@@ -204,53 +215,69 @@ getval(Expr_t* pgm, Exnode_t* node, Exid_t* sym, Exref_t* ref,
     state = (Gpr_t*)(disc->user);
     switch (sym->index) {
     case F_graph :
-      v.user = agopen (args[0].string, xargs(args[1].string), &AgDefaultDisc);
-      if (v.user)
-        agbindrec (v.user,UDATA,sizeof(gdata),0);
+      gp = agopen (args[0].string, xargs(args[1].string), &AgDefaultDisc);
+      if (gp)
+        agbindrec (gp,UDATA,sizeof(gdata),0);
+      v.integer = PTR2INT(gp);
       break;
     case F_node :
-      v.user = agnode ((Agraph_t*)args[0].user, args[1].string, 1);
-      if (v.user)
-        agbindrec (v.user,UDATA,sizeof(ndata),0);
+      np = agnode (INT2PTR(Agraph_t*,args[0].integer), args[1].string, 1);
+      if (np)
+        agbindrec (np,UDATA,sizeof(ndata),0);
+      v.integer = PTR2INT(np);
+      break;
+    case F_isnode :
+      v.integer = PTR2INT(agnode (INT2PTR(Agraph_t*,args[0].integer), args[1].string, 0));
       break;
     case F_edge :
-      v.user = agedge ((Agnode_t*)args[0].user, (Agnode_t*)args[1].user, 
-                       args[2].string, 1);
-      if (v.user)
-        agbindrec (v.user,UDATA,sizeof(edata),0);
+      key = args[2].string;
+      if (*key == '\0') key = 0;
+      ep = agedge (INT2PTR(Agnode_t*,args[0].integer), 
+                     INT2PTR(Agnode_t*,args[1].integer), key, 1);
+      if (ep)
+        agbindrec (ep,UDATA,sizeof(edata),0);
+      v.integer = PTR2INT(ep);
+      break;
+    case F_isedge :
+      key = args[2].string;
+      if (*key == '\0') key = 0;
+      v.integer = PTR2INT(agedge (INT2PTR(Agnode_t*,args[0].integer), 
+                                    INT2PTR(Agnode_t*,args[1].integer), 
+                                    key, 0));
       break;
     case F_clone :
-      v.user = clone ((Agraph_t*)args[0].user, args[1].user);
+      v.integer = PTR2INT(clone (INT2PTR(Agraph_t*,args[0].integer), 
+                                    INT2PTR(Agobj_t*,args[1].integer)));
       break;
     case F_induce :
-      nodeInduce ((Agraph_t*)args[0].user);
+      nodeInduce (INT2PTR(Agraph_t*,args[0].integer));
       v.integer = 0;
       break;
     case F_write :
-      v.integer = agwrite ((Agraph_t*)args[0].user, state->outFile);
+      v.integer = agwrite (INT2PTR(Agraph_t*,args[0].integer), state->outFile);
     case F_isdirect :
-      v.integer = agisdirected ((Agraph_t*)args[0].user);
+      v.integer = agisdirected (INT2PTR(Agraph_t*,args[0].integer));
       break;
     case F_isstrict :
-      v.integer = agisstrict ((Agraph_t*)args[0].user);
+      v.integer = agisstrict (INT2PTR(Agraph_t*,args[0].integer));
       break;
     case F_delete :
-      if (args[1].user == state->curgraph) {
+      if (INT2PTR(Agraph_t*,args[1].integer) == state->curgraph) {
         error (1, "cannot delete current graph $G");
         v.integer = 1;
       }
-      else if (args[1].user == state->target) {
+      else if (INT2PTR(Agraph_t*,args[1].integer) == state->target) {
         error (1, "cannot delete current graph $T");
         v.integer = 1;
       }
       else
-        v.integer = agdelete ((Agraph_t*)args[0].user, args[1].user);
+        v.integer = agdelete (INT2PTR(Agraph_t*,args[0].integer), INT2PTR(Agobj_t*,args[1].integer));
       break;
     case F_nnodes :
-      v.integer = agnnodes ((Agraph_t*)args[0].user);
+      v.integer = agnnodes (INT2PTR(Agraph_t*,args[0].integer));
       break;
     case F_nedges :
-      v.integer = agnedges ((Agraph_t*)args[0].user);
+      v.integer = agnedges (INT2PTR(Agraph_t*,args[0].integer));
       break;
     default :
       error (3, "unknown function call: %s", sym->name);
@@ -268,16 +295,16 @@ getval(Expr_t* pgm, Exnode_t* node, Exid_t* sym, Exref_t* ref,
   else if ((sym->lex == ID) && (sym->index <= LAST_V)) {
     switch (sym->index) {
     case V_this :
-      v.user = state->curobj;
+      v.integer = PTR2INT(state->curobj);
       break;
     case V_thisg :
-      v.user = state->curgraph;
+      v.integer = PTR2INT(state->curgraph);
       break;
     case V_targt :
-      v.user = state->target;
+      v.integer = PTR2INT(state->target);
       break;
     case V_outgraph :
-      v.user = state->outgraph;
+      v.integer = PTR2INT(state->outgraph);
       break;
     case V_tgtname :
       v.string = state->tgtname;
@@ -325,7 +352,7 @@ setval(Expr_t* pgm, Exnode_t* x, Exid_t* sym, Exref_t* ref,
   else if ((MINNAME <= sym->index) && (sym->index <= MAXNAME)) {
     switch (sym->index) {
     case V_outgraph :
-      state->outgraph = v.user;
+      state->outgraph = INT2PTR(Agraph_t*,v.integer);
       break;
     case V_travtype :
       iv = v.integer;
@@ -378,7 +405,7 @@ static int codePhase;
 static tctype
 typeChk (tctype intype, Exid_t* sym)
 {
-  tctype dom, rng;
+  tctype dom = 0, rng = 0;
 
   switch (sym->lex) {
   case DYNAMIC :
@@ -503,6 +530,7 @@ refval(Expr_t* pgm, Exnode_t* node, Exid_t* sym, Exref_t* ref,
 
 #include <../expr/exop.h>
 
+#if 0 /* not used */
 static char*
 typeName (int op)
 {
@@ -511,6 +539,7 @@ typeName (int op)
   else
     return symName(op);
 }
+#endif
 
 static void
 cvtError (Exid_t* xref, char* msg)
@@ -541,13 +570,11 @@ convert(Expr_t* prog, register Exnode_t* x, int type, register Exid_t* xref, int
       if (x->type < MINTOKEN) ret = 0;
     }
     else if (type == INTEGER) {
-      if (x->type != T_tvtyp)
-        x->data.constant.value.integer = (Sflong_t)x->data.constant.value.user;
       ret = 0;
     }
     else if (x->type == T_obj) {
         /* check dynamic type */
-      objp = (Agobj_t*)x->data.constant.value.user;
+      objp = INT2PTR(Agobj_t*,x->data.constant.value.integer);
       if (arg) ret = 0;
       else {
         if (!objp)
@@ -568,7 +595,8 @@ convert(Expr_t* prog, register Exnode_t* x, int type, register Exid_t* xref, int
     else if (type == STRING) {
       if (arg) ret = 0;
       else {
-        objp = (Agobj_t*)x->data.constant.value.user;
+        /* objp = (Agobj_t*)(intptr_t)(x->data.constant.value.integer); */
+        objp = INT2PTR(Agobj_t*,x->data.constant.value.integer);
         if (objp) {
           x->data.constant.value.string = agnameof (objp);
           ret = 0;
@@ -586,6 +614,7 @@ convert(Expr_t* prog, register Exnode_t* x, int type, register Exid_t* xref, int
     return ret;
 }
 
+int strgrpmatch(const char* b, const char* p, int* sub, int n, int flags);
 /* matchval:
  * Pattern match strings.
  */
